const express = require("express");
const connection = require("../connection/dbconfig");
const languageMessages = require("./languageMessages");
const moment = require("moment");
const moment1 = require("moment-timezone");
const {
  getNotificationArrSingle,
  oneSignalNotificationSendCall,
  getNotificationCountOtherUser,
} = require("./notification");

const musaController = require("./musaController");

const uploadToGallery = async (request, response) => {
  const { user_id, image, audio } = request.body;

  if (!user_id) {
    return response
      .status(200)
      .json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "user_id",
      });
  }

  try {
    // Check if user exists
    const sqlCheckUser =
      "SELECT user_id, active_flag FROM user_master WHERE user_id = ? AND delete_flag = 0";
    connection.query(sqlCheckUser, [user_id], async (err, result) => {
      if (err) {
        return response
          .status(200)
          .json({
            success: false,
            msg: languageMessages.internalServerError,
            error: err,
          });
      }
      if (result.length === 0) {
        return response
          .status(200)
          .json({ success: false, msg: languageMessages.msgDataNotFound });
      }
      if (result[0].active_flag === 0) {
        return response
          .status(200)
          .json({
            success: false,
            msg: languageMessages.accountdeactivated,
            active_status: 0,
          });
      }

      // Insert audio into the gallery
      const date = moment().format("YYYY-MM-DD");
      const updatetime = moment().format("YYYY-MM-DD HH:mm:ss");

      if (Array.isArray(audio) && audio.length > 0) {
        for (const data1 of audio) {
          const query1 =
            "INSERT INTO gallery_master(user_id, image, date, createtime, updatetime, type,size) VALUES (?, ?, ?, ?, ?, ?,?)";
          let values1 = [
            user_id,
            data1.audio,
            date,
            updatetime,
            updatetime,
            data1.type,
            data1.size,
          ];

          try {
            await new Promise((resolve, reject) => {
              connection.query(query1, values1, (qErr1, qResult1) => {
                if (qErr1) {
                  reject(qErr1);
                } else {
                  resolve(qResult1);
                }
              });
            });
          } catch (qErr1) {
            return response
              .status(200)
              .json({
                success: false,
                msg: languageMessages.internalServerError,
                error: qErr1,
              });
          }
        }
      }

      // Insert images into the gallery
      if (Array.isArray(image) && image.length > 0) {
        for (const data of image) {
          const query =
            "INSERT INTO gallery_master(user_id, image, thumbnail, date, createtime, updatetime, type,size,size_flag,height,width,duration) VALUES (?, ?, ?, ?, ?, ?, ?,?,?,?,?,?)";
          let values = [
            user_id,
            data.image,
            data.thumbnail || "",
            date,
            updatetime,
            updatetime,
            data.type,
            data.size,
            data.size_flag,
            data.height,
            data.width,
            data.duration
          ];

          try {
            await new Promise((resolve, reject) => {
              connection.query(query, values, (qErr, qResult) => {
                if (qErr) {
                  reject(qErr);
                } else {
                  resolve(qResult);
                }
              });
            });
          } catch (qErr) {
            return response
              .status(200)
              .json({
                success: false,
                msg: languageMessages.internalServerError,
                error: qErr,
              });
          }
        }
      }
      return response
        .status(200)
        .json({ success: true, msg: languageMessages.fileUploadSuccess });
    });
  } catch (error) {
    return response
      .status(200)
      .json({
        success: false,
        msg: languageMessages.internalServerError,
        error,
      });
  }
};

// get gallery images start

const getGallery = async (request, response) => {
  const { user_id } = request.query;

  // Helper function to format dates
  const formatDate = (date) => {
    const today = new Date();
    const yesterday = new Date();
    yesterday.setDate(today.getDate() - 1);

    const dateToCheck = new Date(date);

    // Compare dates
    if (
      dateToCheck.getDate() === today.getDate() &&
      dateToCheck.getMonth() === today.getMonth() &&
      dateToCheck.getFullYear() === today.getFullYear()
    ) {
      return "Today";
    } else if (
      dateToCheck.getDate() === yesterday.getDate() &&
      dateToCheck.getMonth() === yesterday.getMonth() &&
      dateToCheck.getFullYear() === yesterday.getFullYear()
    ) {
      return "Yesterday";
    } else {
      // Format date as DD MMM YYYY
      const options = { day: "2-digit", month: "long", year: "numeric" };
      return dateToCheck
        .toLocaleDateString("en-GB", options)
        .replace(/ /g, " ");
    }
  };

  try {
    // Check if user exists
    const sqlCheckUser =
      "SELECT user_id, active_flag FROM user_master WHERE user_id = ? AND delete_flag = 0";
    connection.query(sqlCheckUser, [user_id], async (err, result) => {
      if (err) {
        return response
          .status(200)
          .json({
            success: false,
            msg: languageMessages.internalServerError,
            error: err,
          });
      }
      if (result.length === 0) {
        return response
          .status(200)
          .json({ success: false, msg: languageMessages.msgDataNotFound });
      }
      if (result[0].active_flag === 0) {
        return response
          .status(200)
          .json({
            success: false,
            msg: languageMessages.accountdeactivated,
            active_status: 0,
          });
      }

      // Get images
      let query =
        "SELECT DISTINCT date, gallery_id, image, thumbnail, type, createtime FROM gallery_master WHERE delete_flag=0 AND user_id=? group by date ORDER BY date DESC";
      let val = [user_id];
      await connection.query(query, val, async (queryErr, queryResult) => {
        if (queryErr) {
          return response
            .status(200)
            .json({
              success: false,
              msg: languageMessages.internalServerError,
              error: queryErr,
            });
        }

        let galleryArr = [];
        let videoArr = [];
        let audioArr = [];
        var image_count = 0;
        var audio_count = 0;
        var video_count = 0;
        // if (queryResult.length > 0) {
        for (let data of queryResult) {
          // if (data.type == 1 || data.type == 3) {
          let imageArr = await getGalleryImages(user_id, data.date);
          if (imageArr != "NA") {
            image_count += imageArr.length;
            galleryArr.push({
              date: data.date,
              title: formatDate(data.date),
              media_arr: imageArr,
            });
          }

          let video_arr = await getGalleryVideos(user_id, data.date);
          if (video_arr != "NA") {
            video_count += video_arr.length;
            videoArr.push({
              date: data.date,
              title: formatDate(data.date),
              media_arr: video_arr,
            });
          }

          // }
          // if (data.type == 2) {
          let imageArr1 = await getGalleryAudios(user_id, data.date);
          if (imageArr1 != "NA") {
            audio_count += imageArr1.length;
            audioArr.push({
              date: data.date,
              title: formatDate(data.date),
              media_arr: imageArr1,
            });
          }

          // }
        }

        if (galleryArr.length <= 0) {
          galleryArr = "NA";
        }
        if (audioArr.length <= 0) {
          audioArr = "NA";
        }

        let img_video = await getAllGalleryImages(user_id);

        let img_video_new = await getAllGalleryVideo(user_id);
        let allGalleryImages = await getAllGalleryImagesWithoutDate(user_id);
        let allGalleryImagesOnly =
          await getAllGalleryImagesWithoutDateOnlyImages(user_id);
        let getAllGalleryAudios = await getAllGalleryAudios1(user_id);
        let data = {
          musa_id: 0,
          user_id: user_id,
          img_video: img_video.length > 0 ? img_video : "NA",
          img_video_new: img_video_new,
          all_gallery_images: allGalleryImages,
          all_gallery_images_only: allGalleryImagesOnly,
          get_all_gallery_audios: getAllGalleryAudios,
        };

        if (galleryArr.length <= 0) {
          galleryArr = "NA";
        }
        if (audioArr.length <= 0) {
          audioArr = "NA";
        }
        if (videoArr.length <= 0) {
          videoArr = "NA";
        }

        

        return response
          .status(200)
          .json({
            success: true,
            msg: languageMessages.msgDataFound,
            image_count: image_count,
            audio_count: audio_count,
            video_count: video_count,
            gallery_arr: galleryArr,
            audio_arr: audioArr,
            video_arr: videoArr,
            data: data,
          });
       
      });
    });
  } catch (error) {
    return response
      .status(200)
      .json({
        success: false,
        msg: languageMessages.internalServerError,
        error,
      });
  }
};

// get gallery images end

async function getGalleryImages(user_id, date) {
  let query =
    "SELECT gallery_id,image,thumbnail,type,date,size,size_flag,height,width,duration FROM gallery_master WHERE delete_flag=0 and user_id=? and date=? and type =1 order by gallery_id desc";
  return new Promise((resolve, reject) => {
    connection.query(query, [user_id, date], async (err, result) => {
      if (err) {
        return reject(err);
      } else {
        let image_arr = [];
        if (result.length > 0) {
          for (let data of result) {
            image_arr.push({
              gallery_id: data.gallery_id,
              duration: data.duration,
              image:
                "https://d38i924xuvvpvh.cloudfront.net/images/" + data.image,
              thumbnail:
                "https://d38i924xuvvpvh.cloudfront.net/" + data.thumbnail,
              thumbnail_image:
                "https://d38i924xuvvpvh.cloudfront.net/" + data.thumbnail,
              type: data.type,
              image_audio_type: data.type,
              size: data.size,
              size_flag_text: "0=normal,1=lengthy",
              size_flag: data.size_flag,
              height: data.height,
              width: data.width,
              mime: "image/jpeg",
              path:
                "https://d38i924xuvvpvh.cloudfront.net/images/" + data.image,
              duration: 600000,
              status: false,
            });
          }
          return resolve(image_arr);
        } else {
          return resolve("NA");
        }
      }
    });
  });
}

async function getGalleryVideos(user_id, date) {
  let query =
    "SELECT gallery_id,image,thumbnail,type,date,size,size_flag,height,width,duration FROM gallery_master WHERE delete_flag=0 and user_id=? and date=? and type =3 order by gallery_id desc";
  return new Promise((resolve, reject) => {
    connection.query(query, [user_id, date], async (err, result) => {
      if (err) {
        return reject(err);
      } else {
        let image_arr = [];
        if (result.length > 0) {
          for (let data of result) {
            image_arr.push({
              gallery_id: data.gallery_id,
              duration: data.duration,
              image:
                "https://d38i924xuvvpvh.cloudfront.net/images/" + data.image,
              thumbnail:
                "https://d38i924xuvvpvh.cloudfront.net/" + data.thumbnail,
              thumbnail_image:
                "https://d38i924xuvvpvh.cloudfront.net/" + data.thumbnail,
              type: data.type,
              image_audio_type: data.type,
              size: data.size,
              size_flag_text: "0=normal,1=lengthy",
              size_flag: data.size_flag,
              height: data.height,
              width: data.width,
              mime: "video/mp4",
              path:
                "https://d38i924xuvvpvh.cloudfront.net/images/" + data.image,
              duration: 600000,
              status: false,
            });
          }
          return resolve(image_arr);
        } else {
          return resolve("NA");
        }
      }
    });
  });
}

async function getAllGalleryImagesWithoutDate(user_id) {
  let query =
    "SELECT gallery_id,image,thumbnail,type,size,size_flag,height,width,duration FROM gallery_master WHERE delete_flag=0 and user_id=? and (type =1 OR type=3) order by gallery_id desc";
  return new Promise((resolve, reject) => {
    connection.query(query, [user_id], async (err, result) => {
      if (err) {
        return reject(err);
      } else {
        let image_arr = [];
        if (result.length > 0) {
          for (let data of result) {
            image_arr.push({
              gallery_id: data.gallery_id,
              duration: data.duration,
              image:
                "https://d38i924xuvvpvh.cloudfront.net/images/" + data.image,
              thumbnail:
                "https://d38i924xuvvpvh.cloudfront.net/" + data.thumbnail,
              thumbnail_image:
                "https://d38i924xuvvpvh.cloudfront.net/" + data.thumbnail,
              type: data.type,
              image_audio_type: data.type,
              size: data.size,
              size_flag_text: "0=normal,1=lengthy",
              size_flag: data.size_flag,
              height: data.height,
              width: data.width,
              mime: data.type == 1 ? "image/jpeg" : "video/mp4",
              path:
                "https://d38i924xuvvpvh.cloudfront.net/images/" + data.image,
              duration: 600000,
              status: false,
            });
          }
          return resolve(image_arr);
        } else {
          return resolve("NA");
        }
      }
    });
  });
}

async function getAllGalleryImagesWithoutDateOnlyImages(user_id) {
  let query =
    "SELECT gallery_id,image,thumbnail,type,size,size_flag,height,width,duration FROM gallery_master WHERE delete_flag=0 and user_id=? and type =1 order by gallery_id desc";
  return new Promise((resolve, reject) => {
    connection.query(query, [user_id], async (err, result) => {
      if (err) {
        return reject(err);
      } else {
        let image_arr = [];
        if (result.length > 0) {
          for (let data of result) {
            image_arr.push({
              gallery_id: data.gallery_id,
              duration: data.duration,
              image:
                "https://d38i924xuvvpvh.cloudfront.net/images/" + data.image,
              thumbnail:
                "https://d38i924xuvvpvh.cloudfront.net/" + data.thumbnail,
              thumbnail_image:
                "https://d38i924xuvvpvh.cloudfront.net/" + data.thumbnail,
              type: data.type,
              image_audio_type: data.type,
              size: data.size,
              size_flag_text: "0=normal,1=lengthy",
              size_flag: data.size_flag,
              height: data.height,
              width: data.width,
              mime: "image/jpeg",
              path:
                "https://d38i924xuvvpvh.cloudfront.net/images/" + data.image,
              duration: 600000,
              status: false,
            });
          }
          return resolve(image_arr);
        } else {
          return resolve("NA");
        }
      }
    });
  });
}
async function getGalleryAudios(user_id, date) {
  let query =
    "SELECT gallery_id,image,thumbnail,type,date,size,size_flag,height,width,duration FROM gallery_master WHERE delete_flag=0 and user_id=? and date=? and type = 2 order by gallery_id desc";
  return new Promise((resolve, reject) => {
    connection.query(query, [user_id, date], async (err, result) => {
      if (err) {
        return reject(err);
      } else {
        let image_arr = [];
        if (result.length > 0) {
          for (let data of result) {
            image_arr.push({
              gallery_id: data.gallery_id,
              duration: data.duration,
              image:
                "https://d38i924xuvvpvh.cloudfront.net/images/" + data.image,
              thumbnail:
                "https://d38i924xuvvpvh.cloudfront.net/" + data.thumbnail,
              thumbnail_image:
                "https://d38i924xuvvpvh.cloudfront.net/" + data.thumbnail,
              type: data.type,
              image_audio_type: data.type,
              size: data.size,
              size_flag_text: "0=normal,1=lengthy",
              size_flag: data.size_flag,
              height: data.height,
              width: data.width,
              path:
                "https://d38i924xuvvpvh.cloudfront.net/images/" + data.image,
              duration: 600000,
              name: data.image,
              status: false,
            });
          }
          return resolve(image_arr);
        } else {
          return resolve("NA");
        }
      }
    });
  });
}

async function getAllGalleryAudios1(user_id) {
  let query =
    "SELECT gallery_id,image,thumbnail,type,date,size,size_flag,height,width,duration FROM gallery_master WHERE delete_flag=0 and user_id=? and type = 2 order by gallery_id desc";
  return new Promise((resolve, reject) => {
    connection.query(query, [user_id], async (err, result) => {
      if (err) {
        return reject(err);
      } else {
        let image_arr = [];
        if (result.length > 0) {
          for (let data of result) {
            image_arr.push({
              gallery_id: data.gallery_id,
              duration: data.duration,
              image:
                "https://d38i924xuvvpvh.cloudfront.net/images/" + data.image,
              thumbnail:
                "https://d38i924xuvvpvh.cloudfront.net/" + data.thumbnail,
              thumbnail_image:
                "https://d38i924xuvvpvh.cloudfront.net/" + data.thumbnail,
              type: data.type,
              image_audio_type: data.type,
              size: data.size,
              size_flag_text: "0=normal,1=lengthy",
              size_flag: data.size_flag,
              height: data.height,
              width: data.width,
              uri: "https://d38i924xuvvpvh.cloudfront.net/images/" + data.image,
              duration: 600000,
              name: data.image,
              status: false,
            });
          }
          return resolve(image_arr);
        } else {
          return resolve("NA");
        }
      }
    });
  });
}

const deleteGalleryImages = async (request, response) => {
  let { user_id, gallery_id } = request.body;

  try {
    // Check if user exists
    const sqlCheckUser =
      "SELECT user_id, active_flag FROM user_master WHERE user_id = ? AND delete_flag = 0";
    connection.query(sqlCheckUser, [user_id], async (err, result) => {
      if (err) {
        return response
          .status(200)
          .json({
            success: false,
            msg: languageMessages.internalServerError,
            error: err,
          });
      }
      if (result.length === 0) {
        return response
          .status(200)
          .json({ success: false, msg: languageMessages.msgDataNotFound });
      }
      if (result[0].active_flag === 0) {
        return response
          .status(200)
          .json({
            success: false,
            msg: languageMessages.accountdeactivated,
            active_status: 0,
          });
      }

      // delete query start
      let gallery_id_arr = gallery_id.split(",");

      try {
        for (let id of gallery_id_arr) {
          await new Promise((resolve, reject) => {
            connection.query(
              `UPDATE gallery_master SET delete_flag=1, updatetime=now() WHERE delete_flag=0 AND gallery_id=?`,
              [id],
              (errId, errResult) => {
                if (errId) {
                  reject(errId);
                } else {
                  resolve(errResult);
                }
              }
            );
          });
        }
        return response
          .status(200)
          .json({ success: true, msg: languageMessages.imageDeletedSuccess });
      } catch (updateError) {
        return response
          .status(500)
          .json({
            success: false,
            msg: languageMessages.internalServerError,
            error: updateError,
          });
      }
    });
  } catch (error) {
    return response
      .status(500)
      .json({
        success: false,
        msg: languageMessages.internalServerError,
        error,
      });
  }
};

// delete images form gallery end

async function getAllGalleryImages(user_id) {
  const sqlSelect =
    "SELECT gallery_id, `image`, `thumbnail`, `type`, `date`, `createtime`,size,size_flag,height,width,duration FROM gallery_master WHERE user_id = ? and delete_flag=0 and type=1 order by gallery_id desc";
  return new Promise((resolve, reject) => {
    connection.query(sqlSelect, [user_id], (err, result) => {
      if (err) {
        reject(err);
      } else {
        let videos = [];
        let images = [];

        let normalPlacementIndexes = [
          0, 3, 4, 5, 6, 7, 10, 11, 12, 13, 14, 17, 18, 19, 20, 21, 24, 25, 26,
          27, 28, 31, 32, 33, 34, 35, 38, 39, 40, 41, 42, 45, 46, 47, 48, 49,
          52, 53, 54, 55, 56, 59, 60, 61, 62, 63, 66, 67, 68, 69, 70, 73, 74,
          75, 76, 77, 80, 81, 82, 83, 84, 87, 88, 89, 90, 91, 94, 95, 96, 97,
          98, 101, 102, 103, 104, 105, 108, 109, 110, 111, 112, 115, 116, 117,
          118, 119, 122, 123, 124, 125, 126, 129, 130, 131, 132, 133, 136, 137,
          138, 139, 140, 143, 144, 145, 146, 147, 150, 151, 152, 153, 124, 157,
          158, 159, 160, 161, 164, 165, 166, 167, 168, 171, 172, 173, 174, 175,
        ];

        console.log(normalPlacementIndexes);

        let videoPlacementIndexes = [
          1, 2, 8, 9, 15, 16, 22, 23, 29, 30, 36, 37, 43, 44, 50, 51, 57, 58,
          64, 65, 71, 72, 78, 79, 85, 86, 92, 93, 99, 100, 106, 107, 113, 114,
          120, 121, 127, 128, 134, 135, 141, 142, 148, 149, 155, 156, 162, 163,
          169, 170, 176, 177, 183, 184, 190, 191, 197, 198, 204, 205, 211, 212,
          218, 219, 225, 226, 232, 233, 239, 240, 246, 247, 253, 254, 260, 261,
          267, 268, 274, 275, 281, 282, 288, 289, 295, 296, 302, 303, 309, 310,
          316, 317, 323, 324, 330, 331, 337, 343, 344, 350, 351, 357, 358, 364,
          365, 371, 372, 378, 379, 385, 386, 392, 393, 399, 400, 406, 407, 413,
          414, 420, 421, 427, 428, 434, 435, 441, 442, 448, 449, 455, 456, 462,
          463, 469, 470, 476, 477, 483, 484, 490, 491, 497, 498, 504, 505, 511,
          512, 518, 519, 525, 526, 532, 533, 539, 540, 546, 547, 553, 554, 560,
          561, 567, 568, 574, 575, 581, 582, 588, 589, 595, 596, 602, 603, 609,
          610, 616, 617, 623, 624, 630, 631, 637, 638, 644, 645, 651, 652, 658,
          659, 665, 666, 672, 673, 679, 680, 686, 687, 693, 694, 700, 701, 707,
          708, 714, 715, 721, 722, 728, 729, 735, 736, 742, 743, 749, 750, 756,
          757, 763, 764, 770, 771, 777, 778, 784, 785, 791, 792, 798, 799, 805,
          806, 812, 813, 819, 820, 826, 827, 833, 834, 840, 841, 847, 848, 856,
          857, 863, 864, 870, 871, 877, 878, 884, 885, 891, 892, 898, 899, 905,
          906, 912, 913, 919, 920, 926, 927, 933, 934, 940, 941, 947, 948, 954,
          955, 961, 962, 968, 969, 975, 976, 982, 983, 989, 990, 996, 997, 1003,
          1004, 1010, 1011, 1017,
        ];

        for (const row of result) {
          if (row.type === 3) {
            // Video
            videos.push({
              height: row.height,
              width: row.width,
              size: row.size,
              musa_image_id: row.gallery_id,
              duration: row.duration,
              size_flag: row.size_flag,
              size_flag_text: "0=normal,1=lengthy",
              image_audio_type: row.type,
              image_audio_type_text: "1=image,3=video",
              image_audio:
                "https://d38i924xuvvpvh.cloudfront.net/images/" + row.image,
              thumbnail_image:
                "https://d38i924xuvvpvh.cloudfront.net/" + row.thumbnail,
              mime: "video/mp4",
              path: "https://d38i924xuvvpvh.cloudfront.net/images/" + row.image,
              duration: 600000,
            });
          } else {
            images.push({
              height: row.height,
              width: row.width,
              size: row.size,
              musa_image_id: row.gallery_id,
              duration: row.duration,
              size_flag: row.size_flag,
              size_flag_text: "0=normal,1=lengthy",
              image_audio_type: row.type,
              image_audio_type_text: "1=image,3=video",
              image_audio:
                "https://d38i924xuvvpvh.cloudfront.net/images/" + row.image,
              thumbnail_image: row.thumbnail_image
                ? "https://d38i924xuvvpvh.cloudfront.net/" + row.thumbnail
                : "NA",
              mime: "image/jpeg",
              path: "https://d38i924xuvvpvh.cloudfront.net/images/" + row.image,
              duration: 600000,
            });
          }
        }

        let maxIndex = Math.max(
          ...videoPlacementIndexes,
          ...normalPlacementIndexes
        );
        let finalArr = new Array(maxIndex + 1).fill("NA");

        videos.forEach((video, index) => {
          if (index < videoPlacementIndexes.length) {
            finalArr[videoPlacementIndexes[index]] = video;
          }
        });

        // Place other images in the remaining slots
        images.forEach((image, index) => {
          if (index < normalPlacementIndexes.length) {
            finalArr[normalPlacementIndexes[index]] = image;
          }
        });

        // Trim 'NA' from the end if no images or videos exist after the last defined index
        while (finalArr.length > 0 && finalArr[finalArr.length - 1] === "NA") {
          finalArr.pop();
        }

        resolve(finalArr);
      }
    });
  });
}

async function getAllGalleryVideo(user_id) {
  const sqlSelect =
    "SELECT gallery_id, `image`, `thumbnail`, `type`, `date`, `createtime`,size,size_flag,height,width,duration FROM gallery_master WHERE user_id = ? and delete_flag=0 and type=3 order by gallery_id desc";
  return new Promise((resolve, reject) => {
    connection.query(sqlSelect, [user_id], (err, result) => {
      if (err) {
        reject(err);
      } else {
        let videos = [];

        for (const row of result) {
          videos.push({
            height: row.height,
            width: row.width,
            size: row.size,
            musa_image_id: row.gallery_id,
            duration: row.duration,
            size_flag: row.size_flag,
            size_flag_text: "0=normal,1=lengthy",
            image_audio_type: row.type,
            image_audio_type_text: "1=image,3=video",
            image_audio:
              "https://d38i924xuvvpvh.cloudfront.net/images/" + row.image,
            thumbnail_image:
              "https://d38i924xuvvpvh.cloudfront.net/" + row.thumbnail,
            mime: "video/mp4",
            path: "https://d38i924xuvvpvh.cloudfront.net/images/" + row.image,
            duration: 600000,
          });
        }

        if (videos.length <= 0) {
          videos = "NA";
        }

        resolve(videos);
      }
    });
  });
}

// comment on images start

const commentOnImages = async (request, response) => {
  const { user_id, image_gallery_id, comment, type,comment_type} = request.body;
  // 0=musa image,1=gallery image
  try {
    if (!user_id) {
      return response
        .status(200)
        .json({
          success: false,
          msg: languageMessages.msg_empty_param,
          key: "user_id",
        });
    }
    if (!image_gallery_id) {
      return response
        .status(200)
        .json({
          success: false,
          msg: languageMessages.msg_empty_param,
          key: "image_gallery_id",
        });
    }

    // Check if user exists
    const sqlCheckUser =
      "SELECT user_id, active_flag,f_name FROM user_master WHERE user_id = ? AND delete_flag = 0";
    connection.query(sqlCheckUser, [user_id], async (err, result) => {
      if (err) {
        return response
          .status(200)
          .json({
            success: false,
            msg: languageMessages.internalServerError,
            error: err,
          });
      }
      if (result.length <= 0) {
        return response
          .status(200)
          .json({ success: false, msg: languageMessages.msgDataNotFound });
      }
      if (result[0].active_flag === 0) {
        return response
          .status(200)
          .json({
            success: false,
            msg: languageMessages.accountdeactivated,
            active_status: 0,
          });
      }
    

    // check images exists or not start

    let sql;
    if (type == 0) {
      sql =
        "SELECT mim.musa_image_id,mm.user_id,mim.size_flag,mim.image_audio_type,mim.image_audio,mim.thumbnail_image FROM musa_image_master as mim , musa_master as mm WHERE mim.delete_flag=0 and mim.musa_image_id=? and mim.musa_id=mm.musa_id";
    } else if (type == 1) {
      sql =
        "SELECT gallery_id FROM gallery_master WHERE delete_flag=0 and gallery_id=?";
    }

    let val = [image_gallery_id];

    await connection.query(sql, val, async (selectErr, selectResult) => {
      if (selectErr) {
        return response
          .status(200)
          .json({
            success: false,
            msg: languageMessages.internalServerError,
            selectErr: selectErr,
          });
      } else {
        if (selectResult.length <= 0) {
          return response
            .status(200)
            .json({ success: false, msg: languageMessages.imageNotFound });
        } else {
          // insert comment start
          let updatetime = moment1()
            .tz("Asia/Kolkata")
            .format("YYYY-MM-DD HH:mm:ss");
          let insertQuery =
            "INSERT INTO image_comment_master(image_gallery_id,user_id,comment,type,createtime,updatetime,comment_type) VALUES (?,?,?,?,?,?,?)";
          let insertVal = [
            image_gallery_id,
            user_id,
            comment,
            type,
            updatetime,
            updatetime,
            comment_type
          ];

          await connection.query(
            insertQuery,
            insertVal,
            async (insertErr, insertResult) => {
              if (insertErr) {
                return response
                  .status(200)
                  .json({
                    success: false,
                    msg: languageMessages.internalServerError,
                    insertErr: insertErr,
                  });
              }
              if (insertResult.affectedRows > 0) {

              
                // send notification start

            if (type == 0) {
              const user_id_notification = user_id;
              const other_user_id_notification = selectResult[0].user_id;
              const action = "musa_image_comment";
              const action_id = image_gallery_id;
              const title = "Comment";
              const messages = result[0].f_name + " commented on your musa.";
              const action_data = {
                user_id: user_id_notification,
                other_user_id: other_user_id_notification,
                action_id: action_id,
                action: action,
                musa_image_id : image_gallery_id,
                size_flag : selectResult[0].size_flag,
                image_audio_type : selectResult[0].image_audio_type,
                path :  "https://d38i924xuvvpvh.cloudfront.net/images/" +
                selectResult[0].image_audio,
                thumbnail_image:
                "https://d38i924xuvvpvh.cloudfront.net/" + selectResult[0].thumbnail_image,
                thumbnail_image_audio: selectResult[0].thumbnail_image,
              };
              if (user_id_notification != other_user_id_notification) {
                getNotificationArrSingle(
                  user_id_notification,
                  other_user_id_notification,
                  action,
                  action_id,
                  title,
                  title,
                  title,
                  title,
                  title,
                  messages,
                  messages,
                  messages,
                  messages,
                  messages,
                  action_data,
                  async (notification_arr_check) => {
                    let notification_arr_check_new = [notification_arr_check];

                    if (
                      notification_arr_check_new &&
                      notification_arr_check_new.length !== 0
                    ) {
                      const notiSendStatus =
                        await oneSignalNotificationSendCall(
                          notification_arr_check_new
                        );
                    } else {
                      console.log("Notification array is empty");
                    }
                  }
                );
              }
            }

            // send notification end
              

                return response
                  .status(200)
                  .json({
                    success: true,
                    msg: languageMessages.commentSuccesfully,
                  });
              } else {
                return response
                  .status(200)
                  .json({
                    success: false,
                    msg: languageMessages.commentUnsuccesfully,
                  });
              }
            }
          );

          // insert comment end
        }
      }
    });
  });

    // check images exists or not end
  } catch (error) {
    return response
      .status(200)
      .json({
        success: false,
        msg: languageMessages.internalServerError,
        error,
      });
  }
};

// comment on images end

// get all comment on images start

const getAllImagesComment = async (request, response) => {
  const { user_id, image_gallery_id, type } = request.query;
  try {
    if (!user_id) {
      return response
        .status(200)
        .json({
          success: false,
          msg: languageMessages.msg_empty_param,
          key: "user_id",
        });
    }
    if (!image_gallery_id) {
      return response
        .status(200)
        .json({
          success: false,
          msg: languageMessages.msg_empty_param,
          key: "image_gallery_id",
        });
    }

    // Check if user exists
    const sqlCheckUser =
      "SELECT user_id, active_flag FROM user_master WHERE user_id = ? AND delete_flag = 0";
    connection.query(sqlCheckUser, [user_id], async (err, result) => {
      if (err) {
        return response
          .status(200)
          .json({
            success: false,
            msg: languageMessages.internalServerError,
            error: err,
          });
      }
      if (result.length <= 0) {
        return response
          .status(200)
          .json({ success: false, msg: languageMessages.msgDataNotFound });
      }
      if (result[0].active_flag === 0) {
        return response
          .status(200)
          .json({
            success: false,
            msg: languageMessages.accountdeactivated,
            active_status: 0,
          });
      }
    });

    // check images exists or not start

    // get all comments start
    // 0=musa image,1=gallery image
    let sql;
    if (type == 0) {
        sql = "SELECT icm.image_gallery_id,icm.image_comment_id,icm.user_id,icm.comment,icm.createtime,um.f_name,um.l_name,um.image,icm.comment_type,um.username,um.name,mm.user_id as my_user_id FROM image_comment_master as icm , user_master as um , musa_image_master as mim , musa_master as mm WHERE mm.musa_id=mim.musa_id and mm.delete_flag=0 and mim.musa_image_id = icm.image_gallery_id and mim.delete_flag=0 and icm.delete_flag=0 and icm.image_gallery_id=? and icm.type=0 and icm.user_id=um.user_id and um.delete_flag=0 order by icm.image_comment_id desc";
    } else if (type == 1) {
        sql = "SELECT icm.image_gallery_id,icm.image_comment_id,icm.user_id,icm.comment,icm.createtime,um.f_name,um.l_name,um.image,icm.comment_type,um.username,um.name,gm.user_id as my_user_id FROM image_comment_master as icm , user_master as um , gallery_master as gm WHERE gm.gallery_id=icm.image_gallery_id and gm.delete_flag=0 and icm.delete_flag=0 and icm.image_gallery_id=? and icm.type=1 and icm.user_id=um.user_id and um.delete_flag=0 order by icm.image_comment_id desc";
    }

    let val = [image_gallery_id];

    await connection.query(sql,val,async(err1,CommentedData)=>{
        if(err1){
            return response
            .status(200)
            .json({
              success: false,
              msg: languageMessages.internalServerError,
              err1:err1,
            });
            }
            let comment_arr = [];
            if(CommentedData.length > 0){

                comment_arr = await Promise.all(CommentedData.map(async(data)=>{
                    return{
                     
                      musa_id : data.image_gallery_id,
                      comment_id : data.image_comment_id,
                      my_user_id : data.my_user_id,
                        user_id : data.user_id,
                        createtime : data.createtime,
                        time : await musaController.timeAgo(data.createtime),
                        username : data.username != null ? data.username : 'NA',
                        name : data.name != null ? data.name : 'NA',
                        f_name : data.f_name,
                        l_name : data.l_name,
                        comment_type : data.comment_type,
                        type_label: "0 for typing type, 1 for audio",
                        image : data.image != null ? data.image : 'NA',
                        comment_type_text: data.comment_type != 2 ? 'Text' : 'Audio',
                        comment: data.comment_type != 2 ? data.comment : "https://d38i924xuvvpvh.cloudfront.net/images/" + data.comment,
                        type_label: data.comment_type === 0 ? 'Typing' : 'Audio',
                        status : false
                    }
                }))

        }

        comment_arr = comment_arr.length <= 0 ? 'NA':comment_arr;

        return response
                  .status(200)
                  .json({
                    success: true,
                    msg: languageMessages.msgDataFound,
                    comment_arr : comment_arr
                  });

    })

    // get all comments end
  } catch (error) {
    return response
      .status(200)
      .json({
        success: false,
        msg: languageMessages.internalServerError,
        error,
      });
  }
};

// get all comment on images end

// delete musa comments start 

const deleteMusaComment = async(request,response) =>{

  const {user_id,comment_id} = request.body;

  try {

    if (!user_id) {
      return response
        .status(200)
        .json({
          success: false,
          msg: languageMessages.msg_empty_param,
          key: "user_id",
        });
    }
   
    if (!comment_id) {
      return response
        .status(200)
        .json({
          success: false,
          msg: languageMessages.msg_empty_param,
          key: "comment_id",
        });
    }

     // Check if user exists
     const sqlCheckUser =
     "SELECT user_id, active_flag FROM user_master WHERE user_id = ? AND delete_flag = 0";
   connection.query(sqlCheckUser, [user_id], async (err, result) => {
     if (err) {
       return response
         .status(200)
         .json({
           success: false,
           msg: languageMessages.internalServerError,
           error: err,
         });
     }
     if (result.length <= 0) {
       return response
         .status(200)
         .json({ success: false, msg: languageMessages.msgDataNotFound });
     }
     if (result[0].active_flag === 0) {
       return response
         .status(200)
         .json({
           success: false,
           msg: languageMessages.accountdeactivated,
           active_status: 0,
         });
     }
      // check images exists or not start
      let updatetime = moment1()
      .tz("Asia/Kolkata")
      .format("YYYY-MM-DD HH:mm:ss");
      await connection.query("UPDATE comment_master SET delete_flag=1,updatetime=? WHERE comment_id=?",[updatetime,comment_id],async(err1,result1)=>{
        if(err1){
          return response
          .status(200)
          .json({
            success: false,
            msg: languageMessages.commentDeleteUnsuccess,
            err1:err1,
          });
        }else{
          return response
          .status(200)
          .json({
            success: true,
            msg: languageMessages.commentDeleteSuccess,
          });
        }
        
      })
   });
    
  } catch (error) {
    return response
    .status(200)
    .json({
      success: false,
      msg: languageMessages.internalServerError,
      error,
    });
  }
}

// delete musa comments end

const deleteMusaImageComment = async(request,response) =>{

  const {user_id,comment_id,type} = request.body;
 // 0=musa image,1=gallery image
  try {

    if (!user_id) {
      return response
        .status(200)
        .json({
          success: false,
          msg: languageMessages.msg_empty_param,
          key: "user_id",
        });
    }
   
    if (!comment_id) {
      return response
        .status(200)
        .json({
          success: false,
          msg: languageMessages.msg_empty_param,
          key: "comment_id",
        });
    }

     // Check if user exists
     const sqlCheckUser =
     "SELECT user_id, active_flag FROM user_master WHERE user_id = ? AND delete_flag = 0";
   connection.query(sqlCheckUser, [user_id], async (err, result) => {
     if (err) {
       return response
         .status(200)
         .json({
           success: false,
           msg: languageMessages.internalServerError,
           error: err,
         });
     }
     if (result.length <= 0) {
       return response
         .status(200)
         .json({ success: false, msg: languageMessages.msgDataNotFound });
     }
     if (result[0].active_flag === 0) {
       return response
         .status(200)
         .json({
           success: false,
           msg: languageMessages.accountdeactivated,
           active_status: 0,
         });
     }
      // check images exists or not start
      let updatetime = moment1()
      .tz("Asia/Kolkata")
      .format("YYYY-MM-DD HH:mm:ss");
      await connection.query("UPDATE image_comment_master SET delete_flag=1,updatetime=? WHERE image_comment_id=? and type=?",[updatetime,comment_id,type],async(err1,result1)=>{
        if(err1){
          return response
          .status(200)
          .json({
            success: false,
            msg: languageMessages.commentDeleteUnsuccess,
            err1:err1,
          });
        }else{
          return response
          .status(200)
          .json({
            success: true,
            msg: languageMessages.commentDeleteSuccess
          });
        }
        
      })
   });
    
  } catch (error) {
    return response
    .status(200)
    .json({
      success: false,
      msg: languageMessages.internalServerError,
      error,
    });
  }
}

module.exports = {
  uploadToGallery,
  getGallery,
  deleteGalleryImages,
  commentOnImages,
  getAllImagesComment,
  deleteMusaComment,
  deleteMusaImageComment
};
