const connection = require("../connection/dbconfig");
const languageMessages = require("./languageMessages");
const { fetchUserData, DeviceTokenStore_1_Signal } = require('./function');
const socialLogincontroller = (request, response) => {
    const { social_type, social_id, social_email, device_type, player_id } = request.body;
    try {
        if (!social_type || !social_id || !social_email || !device_type || !player_id) {
            return response.status(200).json({ success: false, msg: languageMessages.allFieldsRequired });
        }

        connection.query('SELECT user_id FROM user_master WHERE delete_flag = 0 AND (google_id = ? OR apple_id = ?)', [social_id, social_id], async(err, result) => {
            if (err) {
                return response.status(200).json({ success: false, msg: languageMessages.internalServerError });
            }

            if (result.length <= 0) {
                connection.query('SELECT user_id,active_flag FROM user_master WHERE delete_flag = 0 AND email = ?', [social_email],async (err, result) => {
                    if (err) {
                        return response.status(200).json({ success: false, msg: languageMessages.internalServerError,err});
                    }
                    if (result.length <= 0) {
                        return response.status(200).json({success: true, msg: languageMessages.msgUserNotFound, user_exist: 'no', });
                    }
                    
                    const user_id = result[0].user_id;
                    if(result[0].active_flag === 0 ){
                       return response.status(200).json({ success: false, msg: languageMessages.accountdeactivated,active_status : 0 });
                    }

                    await updateSocialDetails(user_id, social_type, social_id).then((update_status) => {   
                            DeviceTokenStore_1_Signal(user_id, device_type, player_id, function (result) {
                                fetchUserData(user_id, (error, userDataArray) => {
                                    if (error) {
                                        return response.status(200).json({ success: false, msg: languageMessages.internalServerError,error});
                                    }
                                    return response.status(200).json({ success: true, msg: languageMessages.loginSuccessful, user_exist: 'yes', userDataArray: userDataArray });
                                });
                            });
                    }).catch((err) => {
                        return response.status(200).json({ success: false, msg: languageMessages.internalServerError, err: err.message });
                    });
                });
            } else {
                const user_id = result[0].user_id;
                await updateSocialDetails(user_id, social_type, social_id).then((update_status) => {
                        DeviceTokenStore_1_Signal(user_id, device_type, player_id, function (result) {
                            fetchUserData(user_id, (error, userDataArray) => {
                                if (error) {
                                    return response.status(200).json({ success: false, msg: languageMessages.internalServerError });
                                }
                                return response.status(200).json({ success: true, msg: languageMessages.loginSuccessful, user_exist: 'yes', userDataArray: userDataArray });
                            });
                        });
                }).catch((err) => {
                    return response.status(200).json({ success: false, msg: languageMessages.internalServerError });
                });
            }
        });
    } catch (error) {
        console.error('Error executing MySQL query:', error);
        return response.status(200).json({ success: false, msg: languageMessages.internalServerError,error : error.message});
    }
};

async function updateSocialDetails(user_id, social_type, social_id) {
    const socialTypeArr = ['app', 'google', 'apple'];
    const index = socialTypeArr.indexOf(social_type);

    let query = '';
    let params = [];

    if (social_type === 'google') {
        query = 'UPDATE user_master SET login_type=?, google_id=?, updatetime = now() WHERE user_id=?';
        params = [index, social_id, user_id];
    } else if (social_type === 'apple') {
        query = 'UPDATE user_master SET login_type=?, apple_id=?, updatetime = now() WHERE user_id=?';
        params = [index, social_id, user_id];
    } else {
        return Promise.reject('Invalid social type');
    }

    return new Promise((resolve, reject) => {
        connection.query(query, params, (error, results) => {
            if (error) {
                reject(error);
            } else {
                if (results.affectedRows <= 0) {
                    resolve('no');
                } else {
                    resolve('yes');
                }
            }
        });
    });
}

module.exports = {socialLogincontroller };