const express = require("express");
const connection = require("../connection/dbconfig");
const util = require("util");
const languageMessages = require("./languageMessages");
const {
  getNotificationArrSingle,
  oneSignalNotificationSendCall,
  getNotificationCountOtherUser,
} = require("./notification");



const moment = require("moment");
const { resolve } = require("path");

// create musa start
const createMussa = (request, response) => {
  var {
    user_id,
    description,
    type,
    image,
    audio,
    category_id,
    sub_category_id,
    musa_title,
  } = request.body;

  try {
    if (!user_id) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "user_id",
      });
    }
    if (!category_id) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "category_id",
      });
    }
    if (!sub_category_id) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "sub_category_id",
      });
    }

    if (!type) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "type",
      });
    }
    if (!image) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "image",
      });
    }
    // if (!musa_title) {
    //   return response.status(200).json({
    //     success: false,
    //     msg: languageMessages.msg_empty_param,
    //     key: "musa_title",
    //   });
    // }

    // Check user exist
    var sqlCheckUser =
      "SELECT user_id, active_flag FROM user_master WHERE user_id = ? AND delete_flag = 0";
    connection.query(sqlCheckUser, [user_id], (err, result) => {
      if (err) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.internalServerError,
          error: err,
        });
      }
      if (result.length === 0) {
        return response
          .status(200)
          .json({ success: false, msg: languageMessages.msgDataNotFound });
      }
      if (result[0].active_flag === 0) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.accountdeactivated,
          active_status: 0,
        });
      }

      const insertData =
        "INSERT INTO musa_master(category, sub_category, user_id, description, type,title, createtime, updatetime) VALUES (?, ?, ?, ?, ? , ? , now(), now())";
      const values = [
        category_id,
        sub_category_id,
        user_id,
        description,
        type,
        musa_title,
      ];
      connection.query(insertData, values, async (err, dataresult) => {
        if (err) {
          return response.status(200).json({
            success: false,
            msg: languageMessages.internalServerError,
            error: err.message,
          });
        }

        const musa_id = await dataresult.insertId;
        // Insert image

        if (image) {
          for (const imageData of image) {
            const insertimage =
              "INSERT INTO musa_image_master(duration,image_audio,size, musa_id, user_id, image_audio_type, createtime, updatetime,thumbnail_image,size_flag) VALUES (?,?, ?,  ?, ?, ? , now(), now(),?,?)";
            const values = [
              imageData.duration,
              imageData.image,
              imageData.size,
              musa_id,
              user_id,
              imageData.type,
              imageData.thumbnail,
              imageData.size_flag,
            ];

            connection.query(insertimage, values, (err, data) => {
              if (err) {
                return response.status(200).json({
                  success: false,
                  msg: languageMessages.internalServerError,
                  error: err.message,
                });
              }
            });
          }
        }

        if (audio) {
          for (const audioData of audio) {
            const insertAudio =
              "INSERT INTO musa_image_master(image_audio,size, musa_id, user_id, image_audio_type, createtime, updatetime) VALUES (?, ?, ?, ?, ?,  now(), now())";
            const values = [
              audioData.audio,
              audioData.size,
              musa_id,
              user_id,
              2,
            ];

            connection.query(insertAudio, values, (err, data) => {
              if (err) {
                return response.status(200).json({
                  success: false,
                  msg: languageMessages.internalServerError,
                  error: err.message,
                });
              }
            });
          }
        }

        // Fetch user data and respond
        fetchUserData1(user_id, (error, userDataArray) => {
          if (error) {
            return response.status(200).json({
              success: false,
              msg: languageMessages.internalServerError,
            });
          }
          return response.status(200).json({
            success: true,
            msg: languageMessages.postcreatesuccessfully,
            userDataArray: userDataArray,
            category_id: category_id,
            sub_category_id: sub_category_id,
          });
        });
      });
    });
  } catch (error) {
    return response
      .status(200)
      .json({ success: false, msg: languageMessages.internalServerError });
  }
};

// create musa end

// contributor follow start
const userFollow = (request, response) => {
  const { user_id, other_user_id } = request.body;
  try {
    if (!user_id) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "user_id",
      });
    }
    if (!other_user_id) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "other_user_id",
      });
    }
    // check user start
    var sqlcheak =
      "SELECT user_id,active_flag FROM user_master where user_id = ? and delete_flag = 0";
    connection.query(sqlcheak, [user_id], (err, result) => {
      if (err) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.internalServerError,
          error: err,
        });
      }
      if (result.length <= 0) {
        return response
          .status(200)
          .json({ success: false, msg: languageMessages.msgUserNotFound });
      }
      if (result[0].active_flag === 0) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.accountdeactivated,
          active_status: 0,
        });
      }
    });
    //  check user start

    // check other user start
    var sqlcheak =
      "SELECT user_id,active_flag FROM user_master where user_id = ? and delete_flag = 0";
    connection.query(sqlcheak, [other_user_id], (err, result) => {
      if (err) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.internalServerError,
          error: err,
        });
      }
      if (result.length <= 0) {
        return response
          .status(200)
          .json({ success: false, msg: languageMessages.msgOtherNotFound });
      }
      if (result[0].active_flag === 0) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.accountdeactivated,
          active_status: 0,
        });
      }
    });
    //  check other user start

    // check already followed start

    connection.query(
      "SELECT contributor_id FROM contributor_follow_master WHERE delete_flag=0 and user_id=? and other_user_id=?",
      [user_id, other_user_id],
      async (contErr, contResult) => {
        if (contErr) {
          return response.status(200).json({
            success: false,
            msg: languageMessages.internalServerError,
            contErr: contErr,
          });
        }
        if (contResult.length > 0) {
          connection.query(
            "DELETE FROM contributor_follow_master WHERE user_id=? and other_user_id=?",
            [user_id, other_user_id],
            async (delErr, delResult) => {
              if (delErr) {
                return response.status(200).json({
                  success: false,
                  msg: languageMessages.internalServerError,
                  delErr: delErr,
                });
              } else {
                return response.status(200).json({
                  success: true,
                  msg: languageMessages.unfollowSuccessfully,
                });
              }
            }
          );
        } else {
          sqlinsert =
            "insert into contributor_follow_master(user_id,other_user_id,createtime,updatetime) values (?, ?, now() ,now())";
          connection.query(
            sqlinsert,
            [user_id, other_user_id],
            (err, result) => {
              if (err) {
                return response.status(200).json({
                  success: false,
                  msg: languageMessages.internalServerError,
                  error: err,
                });
              } else {
                return response.status(200).json({
                  success: true,
                  msg: languageMessages.followSuccessfully,
                });
              }
            }
          );
        }
      }
    );

    // check already followed end
  } catch (error) {
    return response
      .status(200)
      .json({ success: false, msg: languageMessages.internalServerError });
  }
};

// contributor follow end

const getSocialMusaDetails = async (request, response) => {
  const { user_id, offset, search_text,device_type,player_id } = request.query;
  try {
    if (!user_id) {
      return response.status(400).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "user_id",
      });
    }
    // if (!device_type) {
    //   return response.status(400).json({
    //     success: false,
    //     msg: languageMessages.msg_empty_param,
    //     key: "device_type",
    //   });
    // }
    // if (!player_id) {
    //   return response.status(400).json({
    //     success: false,
    //     msg: languageMessages.msg_empty_param,
    //     key: "player_id",
    //   });
    // }

    // check user start
    const sqlcheak =
      "SELECT user_id,active_flag FROM user_master WHERE user_id = ? AND delete_flag = 0";
    const userResult = await new Promise((resolve, reject) => {
      connection.query(sqlcheak, [user_id], (err, result) => {
        if (err) reject(err);
        else resolve(result);
      });
    });

    if (userResult.length <= 0) {
      return response
        .status(200)
        .json({ success: false, msg: languageMessages.msgUserNotFound });
    }
    if (userResult[0].active_flag === 0) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.accountdeactivated,
        active_status: 0,
      });
    }
    // check user end
    let limit = 5;
    let sqlSelect;
    let val = [];
    let contibutors_ids = await getContributedIds(user_id);

    let category_ids = 0;
    if (search_text) {
      category_ids = await getCategorySearchId(search_text);
    }

  // check player id start

  if(player_id && device_type){
  await connection.query("SELECT player_id FROM user_notification WHERE user_id = ?",[user_id],async(err394,resultPlayer)=>{
    if(resultPlayer.length > 0){
      await connection.query('UPDATE user_notification SET player_id=?,device_type=? WHERE user_id=?',[player_id,device_type,user_id],async(err396,resultPlayerUpdate)=>{});
    }else{
      await connection.query("INSERT INTO user_notification (user_id, device_type, player_id, inserttime) VALUES (?,?,?,now())",[user_id,device_type,player_id],async(err400,resultPlayerInsert)=>{});
    }
  })
}
  // check player id end

    // return response.status(200).json({ success: false, msg: languageMessages.accountdeactivated, contibutors_ids: contibutors_ids });

    if (search_text) {
      let searchTextLIke = "%" + search_text + "%";
      // sqlSelect = `SELECT musa_id, category, sub_category, user_id, description, type, title, createtime, updatetime FROM musa_master WHERE (type=0 OR (type=1 AND user_id IN (${contibutors_ids}))) AND delete_flag = 0 AND (title LIKE ? OR category IN(${category_ids})) AND type !=2 ORDER BY musa_id DESC LIMIT ${limit} OFFSET ${offset}`;
      sqlSelect = `SELECT mm.musa_id, mm.category, mm.sub_category, mm.user_id, mm.description, mm.type, mm.title, mm.createtime, mm.updatetime FROM musa_master as mm , user_master as um WHERE um.user_id=mm.user_id and um.delete_flag=0 and (mm.type=0 OR (mm.type=1 AND mm.user_id IN (${contibutors_ids}))) AND mm.delete_flag = 0 AND (mm.title LIKE '${searchTextLIke}' OR mm.category IN(${category_ids})) AND mm.type !=2 ORDER BY mm.musa_id DESC LIMIT ${limit} OFFSET ${offset}`;
      val = [0];
    } else {
      // sqlSelect = `SELECT musa_id, category, sub_category, user_id, description, type, title, createtime, updatetime FROM musa_master WHERE (type=0 OR (type=1 AND user_id IN (${contibutors_ids}))) AND delete_flag = 0 and type !=2 ORDER BY musa_id DESC LIMIT ${limit} OFFSET ${offset}`;
      sqlSelect = `SELECT mm.musa_id, mm.category, mm.sub_category, mm.user_id, mm.description, mm.type, mm.title, mm.createtime, mm.updatetime FROM musa_master as mm , user_master as um WHERE um.user_id=mm.user_id and um.delete_flag=0 and (mm.type=0 OR (mm.type=1 AND mm.user_id IN (${contibutors_ids}))) AND mm.delete_flag = 0 and mm.type !=2 ORDER BY mm.musa_id DESC LIMIT ${limit} OFFSET ${offset}`;
      val = [0];
    }

    var result = await new Promise(async (resolve, reject) => {
      await connection.query(sqlSelect, val, async (err, result) => {
        if (err) reject(err);
        else resolve(result);
      });
    });

    var musa_data_arr = [];
    for (var row of result) {
      const userData = await getUser(row.user_id);
      const image_arr = await getMusaImage(row.musa_id, row.user_id);
      const musa_video = await getMusavideo(row.musa_id, row.user_id);
      const musa_audio = await getMusaAudio(row.musa_id, row.user_id);
      const musa_image = await getMusaMedia(row.musa_id, row.user_id);
      const img_video = await getMusavideoImage(row.musa_id, row.user_id);
      const img_video_new = await getMusavideoNew(row.musa_id, row.user_id);
      const musa_like_status = await getlikeStatus1(row.musa_id, user_id);
      const followstatus = await getFollowStatus1(user_id, row.user_id);
      // const follow_staus = await getFollowStatus(user_id, row.user_id);
      const categoryData = await getSingleAllCategoryData(row.category);
      const subCategory = await getSingleSubCategoryData(
        row.sub_category,
        row.category
      );
      const comment = await getComment1(row.musa_id);
      // const commentData = await getCommentData(row.musa_id)
      const time = await timeAgo(row.createtime);
      const like_count = await getlikecount(row.musa_id);
      const savemusa_status = await getsaveStatus(row.musa_id, user_id);
      const display_request_status = await displayStatus(row.musa_id, user_id);
      const share_count = await getShareCount(row.musa_id, response);
      musa_data_arr.push({
        name: userData ? userData.name : "NA",
        f_name: userData ? userData.f_name : "NA",
        l_name: userData ? userData.l_name : "NA",
        image: userData ? userData.image : "NA",
        musa_id: row.musa_id,
        user_id: row.user_id,
        // title : row.title,
        musa_title: row.title,
        category_data: categoryData ? categoryData[0].title : "NA",
        sub_category: subCategory ? subCategory[0].title : "NA",
        description: row.description,
        createtime: time,
        type: row.type,
        type_label: "0 for public, 1 for contributor, 3 for only me",
        follow_staus: followstatus ? followstatus : 0,
        follow_staus_label: "0 - no , 1- yes ",
        comment: comment,
        // commentData  : (commentData) ? commentData : "NA",
        musa_like_status: musa_like_status ? musa_like_status : "NA",
        musa_like_status_label: "0 - no , 1- yes ",
        like_count: like_count,
        total_share: share_count,
        savemusa_status: savemusa_status,
        display_request_status_text: "0=not sended,1=sended,2=accepted",
        display_request_status: display_request_status,
        savemusaStatus_lable: "0 for no , 1 for yes",
        musa_image: musa_image ? musa_image : "NA",
        image_arr: image_arr ? image_arr : "NA",
        musa_video: musa_video ? musa_video : "NA",
        musa_audio: musa_audio ? musa_audio : "NA",
        img_video: img_video.length > 0 ? img_video : "NA",
        img_video_new: img_video_new.length > 0 ? img_video_new : "NA",
        status: false,
      });
    }
    if (musa_data_arr.length <= 0) {
      musa_data_arr = "NA";
    }

    let loadmore = musa_data_arr.length >= limit ? true : false;

    return response.status(200).json({
      success: true,
      msg: languageMessages.msgDataFound,
      loadmore: loadmore,
      musa_data_arr: musa_data_arr,
    });
  } catch (error) {
    return response.status(200).json({
      success: false,
      msg: languageMessages.internalServerError,
      error: error.message,
    });
  }
};

async function getsaveStatus(musa_id, user_id) {
  var sqlstatus =
    "SELECT `save_musa_id`  FROM `saved_musaa` WHERE user_id = ? AND musa_id = ? AND delete_flag = 0 ";
  return new Promise((resolve, reject) => {
    connection.query(sqlstatus, [user_id, musa_id], (err, result) => {
      if (err) {
        reject(err);
      } else {
        if (result.length > 0) resolve(1);
        else resolve(0);
      }
    });
  });
}

// social musa details end

// signle musa details start

const getSingleMusaDetails = (request, response) => {
  try {
    const { musa_id, user_id } = request.query;
    if (!user_id) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "user_id",
      });
    }
    if (!musa_id) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "musa_id",
      });
    }

    // check user start
    var sqlcheak =
      "SELECT user_id,active_flag FROM user_master where user_id = ? and delete_flag = 0";
    connection.query(sqlcheak, [user_id], (err, result) => {
      if (err) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.internalServerError,
          error: err,
        });
      }
      if (result.length <= 0) {
        return response
          .status(200)
          .json({ success: false, msg: languageMessages.msgUserNotFound });
      }
      if (result[0].active_flag === 0) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.accountdeactivated,
          active_status: 0,
        });
      }
    });

    //  check user start

    // check musa start
    var sqlcheak =
      "SELECT musa_id FROM musa_master where musa_id = ? and delete_flag = 0";
    connection.query(sqlcheak, [musa_id], (err, result) => {
      if (err) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.internalServerError,
          error: err,
        });
      }
      if (result.length <= 0) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.musaNotFound,
          key: "1",
        });
      }
    });

    //  check musa start
    // const sqlSelect = "SELECT musa_id, category, sub_category, user_id, description, type, title, createtime, updatetime FROM musa_master WHERE  musa_id = ? AND  type = 0 AND delete_flag = 0 ORDER BY musa_id DESC";
    const sqlSelect =
      "SELECT musa_id, category, sub_category, user_id, description, type, title, createtime, updatetime FROM musa_master WHERE  musa_id = ? AND delete_flag = 0 ORDER BY musa_id DESC";
    connection.query(sqlSelect, [musa_id], async (err, result) => {
      if (err) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.internalServerError,
          error: err,
        });
      }

      if (result.length <= 0) {
        return response
          .status(200)
          .json({ success: true, msg: languageMessages.msgDataFound });
      }

      const userData = await getUser(result[0].user_id);
      const image_arr = await getMusaImage(
        result[0].musa_id,
        result[0].user_id
      );
      const musa_video = await getMusavideo(
        result[0].musa_id,
        result[0].user_id
      );
      const musa_audio = await getMusaAudio(
        result[0].musa_id,
        result[0].user_id
      );
      const musa_image = await getMusaMedia(
        result[0].musa_id,
        result[0].user_id
      );
      
      const img_video = await getMusavideoImage(result[0].musa_id, result[0].user_id);
      const img_video_new = await getMusavideoNew(result[0].musa_id, result[0].user_id);
      const musa_like_status = await getlikeStatus1(result[0].musa_id, user_id);
      const follow_staus = await getFollowStatus1(user_id, result[0].user_id);
      const categoryData = await getSingleAllCategoryData(result[0].category);
      const subCategory = await getSingleSubCategoryData(
        result[0].sub_category,
        result[0].category
      );
      const comment1 = await getComment1(result[0].musa_id);
      const commentData = await getCommentData(result[0].musa_id);
      const like_count = await getlikecount(result[0].musa_id);
      const time = await timeAgo(result[0].createtime);
      const savemusa_status = await getsaveStatus(result[0].musa_id, user_id);
      const imagesAndVideos = await getMusavideoImageSingleArr(
        result[0].musa_id,
        user_id
      );
      const display_request_status = user_id == result[0].user_id ? 2 : await displayStatus(result[0].musa_id, user_id);
      const share_count = await getShareCount(result[0].musa_id, response);
      var musa_data_arr = {
        name: userData.name ? userData.name : "NA",
        f_name: userData.f_name ? userData.f_name : "NA",
        l_name: userData.l_name ? userData.l_name : "NA",
        musa_id: result[0].musa_id,
        musa_title: result[0].title,
        categoryData: categoryData ? categoryData[0].title : "NA",
        subCategory: subCategory ? subCategory[0].title : "NA",
        user_id: result[0].user_id,
        description: result[0].description,
        createtime: time,
        type: result[0].type,
        type_label: "0 for public, 1 for contributor, 3 for only me",
        // createtime: result[0].createtime,
        follow_staus: follow_staus,
        follow_staus_lable: "0 - no , 1- yes ",
        musa_like_status: musa_like_status,
        musa_like_status_lable: "0 - no , 1- yes ",
        type: result[0].type,
        type_label: "0 for public, 1 for contributor, 3 for only me",
        like_count: like_count,
        total_share: share_count,
        savemusa_status: savemusa_status,
        savemusaStatus_lable: "0 for no , 1 for yes",
        comment: comment1,
        commentData: commentData,
        musa_image: musa_image,
        image_arr: image_arr ? image_arr : "NA",
        musa_video: musa_video ? musa_video : "NA",
        musa_audio: musa_audio ? musa_audio : "NA",
        imagesAndVideos: imagesAndVideos,
        img_video: img_video.length > 0 ? img_video : 'NA',
        img_video_new: img_video_new.length > 0 ? img_video_new : 'NA',
        display_request_status : display_request_status,
      };
      return response.status(200).json({
        success: true,
        msg: languageMessages.msgDataFound,
        musa_data_arr: musa_data_arr,
      });
    });
  } catch (error) {
    return response.status(200).json({
      success: false,
      msg: languageMessages.internalServerError,
      error: error,
    });
  }
};

// sigle musa details end

//  musaa comment start
const musaComment = (request, response) => {
  const { user_id, musa_id, comment, comment_type, comment_text } =
    request.body;
  try {
    if (!user_id) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "user_id",
      });
    }
    if (!musa_id) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "musa_id",
      });
    }

    if (!comment_type) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "comment_type",
      });
    }
    // check user start
    var sqlcheak =
      "SELECT user_id,active_flag,f_name FROM user_master where user_id = ? and delete_flag = 0";
    connection.query(sqlcheak, [user_id], (err, result) => {
      if (err) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.internalServerError,
          error: err,
        });
      }
      if (result.length <= 0) {
        return response
          .status(200)
          .json({ success: false, msg: languageMessages.msgUserNotFound });
      }
      if (result[0].active_flag === 0) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.accountdeactivated,
          active_status: 0,
        });
      }
      var username = result[0].f_name;

      //  check user start

      // check musa start
      var sqlcheak1 =
        "SELECT musa_id,user_id FROM musa_master where musa_id = ? and delete_flag = 0";
      connection.query(sqlcheak1, [musa_id], (err1, result1) => {
        if (err1) {
          return response.status(200).json({
            success: false,
            msg: languageMessages.internalServerError,
            error: err,
          });
        }
        if (result1.length <= 0) {
          return response.status(200).json({
            success: false,
            msg: languageMessages.musaNotFound,
            key: "1",
          });
        }
        var other_user_id = result1[0].user_id;

        //  check musa start
        var sqlComment =
          "INSERT INTO comment_master(user_id, musa_id, comment, comment_type, createtime, updatetime,comment_text) values (? , ?, ?, ?,now(),now(),?)";
        connection.query(
          sqlComment,
          [user_id, musa_id, comment, comment_type, comment_text],
          (err, result) => {
            if (err) {
              return response.status(200).json({
                success: false,
                msg: languageMessages.internalServerError,
                error: err,
              });
            }

            // send notification start

            if (user_id != other_user_id) {
              const user_id_notification = user_id;
              const other_user_id_notification = other_user_id;
              const action = "musa_comment";
              const action_id = musa_id;
              const title = "Comment";
              const messages = username + " commented on your musa";
              const action_data = {
                user_id: user_id_notification,
                other_user_id: other_user_id_notification,
                action_id: action_id,
                action: action,
                musa_id : musa_id,
                user_id : user_id,
                comment_type : comment_type,
              };
              if (user_id_notification != other_user_id_notification) {
                getNotificationArrSingle(
                  user_id_notification,
                  other_user_id_notification,
                  action,
                  action_id,
                  title,
                  title,
                  title,
                  title,
                  title,
                  messages,
                  messages,
                  messages,
                  messages,
                  messages,
                  action_data,
                  async (notification_arr_check) => {
                    let notification_arr_check_new = [notification_arr_check];

                    if (
                      notification_arr_check_new &&
                      notification_arr_check_new.length !== 0
                    ) {
                      const notiSendStatus =
                        await oneSignalNotificationSendCall(
                          notification_arr_check_new
                        );
                    } else {
                      console.log("Notification array is empty");
                    }
                  }
                );
              }
            }

            // send notification end
            
            return response.status(200).json({
              success: true,
              msg: languageMessages.commentSuccesfully,
            });
          }
        );
      });
    });
  } catch (error) {
    return response.status(200).json({
      success: false,
      msg: languageMessages.internalServerError,
      error: error,
    });
  }
};
// musa comment end

// musa like unlike start

const musaLikeUnlike = (request, response) => {
  const { user_id, musa_id } = request.body;
  if (!user_id) {
    return response.status(200).json({
      success: false,
      msg: languageMessages.msg_empty_param,
      key: "user_id",
    });
  }
  if (!musa_id) {
    return response.status(200).json({
      success: false,
      msg: languageMessages.msg_empty_param,
      key: "musa_id",
    });
  }
  // check user exists or not start
  var sqlcheak =
    "SELECT user_id,active_flag,f_name FROM user_master where user_id = ? and delete_flag = 0";
  connection.query(sqlcheak, [user_id], (err, result) => {
    if (err) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.internalServerError,
        error: err,
      });
    }
    if (result.length <= 0) {
      return response
        .status(200)
        .json({ success: false, msg: languageMessages.msgUserNotFound });
    }
    if (result[0].active_flag === 0) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.accountdeactivated,
        active_status: 0,
      });
    }

    // check user exists or not end
    var username = result[0].f_name;

    var sqlcheak1 =
      "SELECT user_id FROM musa_master where musa_id = ? and delete_flag = 0";
    connection.query(sqlcheak1, [musa_id], (err1, result1) => {
      if (err1) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.internalServerError,
          error: err,
        });
      }
      if (result1.length <= 0) {
        return response
          .status(200)
          .json({ success: false, msg: languageMessages.musaNotFound });
      }
      // if(result[0].active_flag === 0){
      //     return response.status(200).json({ success: false, msg: languageMessages.accountdeactivated,active_status : 0 });
      // }
      var musa_user_id = result1[0].user_id;

      var select_like =
        "SELECT musa_id,type FROM musa_like_master WHERE user_id=? and musa_id=?  and delete_flag=0 and type= 0 ";
      var select_values = [user_id, musa_id];
      connection.query(
        select_like,
        select_values,
        async (selectLikeError, SelectLikeResult) => {
          if (selectLikeError) {
            return response.status(200).json({
              success: false,
              msg: languageMessages.internalServerError,
              data: 1,
            });
          }
          if (SelectLikeResult.length > 0) {
            var deleteLike =
              "DELETE FROM musa_like_master WHERE user_id = ? and musa_id = ? and type=0";
            var deleteValues = [user_id, musa_id];
            connection.query(
              deleteLike,
              deleteValues,
              (deleteLikeError, deleteLikeResult) => {
                if (deleteLikeError) {
                  return response.status(200).json({
                    success: false,
                    msg: languageMessages.internalServerError,
                    data: 2,
                  });
                }
                return response.status(200).json({
                  success: true,
                  msg: languageMessages.UnlikeSuccessfully,
                });
              }
            );
          } else {
            // insert like start
            var insertLike =
              "INSERT INTO musa_like_master (musa_id, user_id, createtime, updatetime) VALUES (?,?,now(),now())";
            var insertValues = [musa_id, user_id];
            connection.query(
              insertLike,
              insertValues,
              (InsertError, InsertResult) => {
                if (InsertError) {
                  return response.status(200).json({
                    success: false,
                    msg: languageMessages.UnlikeSuccessfully,
                    error: InsertError,
                  });
                }
                // if(InsertResult){
                // send notification start

                const user_id_notification = user_id;

                const other_user_id_notification = musa_user_id;

                const action = "MUSA like";

                const action_id = musa_id;

                const title = "MUSA like";

                const messages = username + " Liked your musa ";

                const action_data = {
                  user_id: user_id_notification,
                  other_user_id: other_user_id_notification,
                  action_id: action_id,
                  action: action,
                };

                if (user_id_notification != other_user_id_notification) {
                  getNotificationArrSingle(
                    user_id_notification,
                    other_user_id_notification,
                    action,
                    action_id,
                    title,
                    title,
                    title,
                    title,
                    title,
                    messages,
                    messages,
                    messages,
                    messages,
                    messages,
                    action_data,
                    async (notification_arr_check) => {
                      let notification_arr_check_new = [notification_arr_check];

                      if (
                        notification_arr_check_new &&
                        notification_arr_check_new.length !== 0
                      ) {
                        const notiSendStatus =
                          await oneSignalNotificationSendCall(
                            notification_arr_check_new
                          );
                      } else {
                        console.log("Notification array is empty");
                      }
                    }
                  );
                }

                // send notification end

                return response.status(200).json({
                  success: true,
                  msg: languageMessages.likeSuccessfully,
                });
              }
            );
            // insert like end
          }
        }
      );
    });
  });
};

// musa like unlike end

const postReport = (request, response) => {
  try {
    const { user_id, musa_id, type, reason } = request.body;
    if (!user_id) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "user_id",
      });
    }
    if (!musa_id) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "musa_id",
      });
    }
    if (!type) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "type",
      });
    }
    if (!reason) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "reason",
      });
    }
    // check user exists or not
    var sql1 =
      "SELECT user_id from user_master where delete_flag = 0 AND user_id = ?";
    var values = [user_id];
    connection.query(sql1, values, async (err, information) => {
      if (err) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.internalServerError,
          data: {},
        });
      }
      if (information.length <= 0) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.msgUserNotFound,
          data: {},
        });
      }
      if (information[0].active_flag === 0) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.accountdeactivated,
          active_status: 0,
        });
      }
      var insertSql =
        "INSERT into report_master(user_id,musa_id,type,reason,createtime,updatetime) values (?, ?, ?, ?, now(), now())";
      connection.query(
        insertSql,
        [user_id, musa_id, type, reason],
        (error, data) => {
          if (error) {
            return response.status(200).json({
              success: false,
              msg: languageMessages.internalServerError,
            });
          } else {
            return response.status(200).json({
              success: true,
              msg: languageMessages.reportsuccessfully,
            });
          }
        }
      );
    });
  } catch (error) {
    return response.status(200).json({
      success: false,
      msg: languageMessages.internalServerError,
      data: {},
    });
  }
};

const Contributor = (request, response) => {
  const { user_id, other_user_id } = request.query;
  try {
    if (!user_id) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "user_id",
      });
    }
    if (!other_user_id) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "other_user_id",
      });
    }

    // check user start
    var sqlcheak =
      "SELECT user_id,active_flag FROM user_master where user_id = ? and delete_flag = 0";
    connection.query(sqlcheak, [user_id], (err, result) => {
      if (err) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.internalServerError,
          error: err,
        });
      }
      if (result.length <= 0) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.msgDataNotFound,
          key: "1",
        });
      }
      if (result[0].active_flag === 0) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.accountdeactivated,
          active_status: 0,
        });
      }
    });
    // check user end

    var sqlSelect =
      "SELECT cfm.contributor_id, cfm.user_id,cfm.other_user_id, cfm.follow_status from contributor_follow_master as cfm , user_master as um where cfm.other_user_id = ? AND cfm.delete_flag = 0 and cfm.user_id=um.user_id and um.delete_flag=0";
    // connection.query(sqlSelect,[user_id], async(err,resultData)=>{
    connection.query(sqlSelect, [other_user_id], async (err, resultData) => {
      if (err) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.internalServerError,
          error: err,
        });
      }
      // if(resultData.length <= 0){
      //    return response.status(200).json({success : false ,msg: languageMessages.msgDataNotFound,key:"1"})
      // }
      var user_data_arr = [];
      for (var data of resultData) {
        var userData = await getUser(data.user_id);
        // var follow_status_new = await getFollowStatus1(other_user_id,data.user_id);
        var follow_status_new = await getFollowStatus1(user_id, data.user_id);
        user_data_arr.push({
          name: userData ? userData.name : null,
          f_name: userData ? userData.f_name : null,
          l_name: userData ? userData.l_name : null,
          image: userData ? userData.image : null,
          follow_status: follow_status_new,
          follow_status_new: follow_status_new,
          user_id: data.user_id,
          other_user_id: data.other_user_id,
        });
      }
      if (user_data_arr.length <= 0) {
        user_data_arr = "NA";
      }
      return response.status(200).json({
        success: true,
        msg: languageMessages.msgDataFound,
        user_data_arr: user_data_arr,
      });
    });
  } catch (error) {
    return response.status(200).json({
      success: false,
      msg: languageMessages.internalServerError,
      resultData: resultData,
    });
  }
};

const myContributor = (request, response) => {
  const { user_id, other_user_id } = request.query;
  try {
    if (!user_id) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "user_id",
      });
    }
    if (!other_user_id) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "other_user_id",
      });
    }

    // check user start
    var sqlcheak =
      "SELECT user_id,active_flag FROM user_master where user_id = ? and delete_flag = 0";
    connection.query(sqlcheak, [user_id], (err, result) => {
      if (err) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.internalServerError,
          error: err,
        });
      }
      if (result.length <= 0) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.msgDataNotFound,
          key: "1",
        });
      }
      if (result[0].active_flag === 0) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.accountdeactivated,
          active_status: 0,
        });
      }
    });
    // check user end

    var sqlSelect =
      "SELECT contributor_id, user_id,other_user_id, follow_status from contributor_follow_master where user_id = ? AND delete_flag = 0";
    connection.query(sqlSelect, [user_id], async (err, resultData) => {
      if (err) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.internalServerError,
          error: err,
        });
      }

      var user_data_arr = [];
      for (var data of resultData) {
        var userData = await getUser(data.other_user_id);
        var follow_status_new = await getFollowStatus1(
          other_user_id,
          data.other_user_id
        );
        user_data_arr.push({
          name: userData ? userData.name : null,
          f_name: userData ? userData.f_name : null,
          l_name: userData ? userData.l_name : null,
          image: userData ? userData.image : null,
          follow_status: follow_status_new,
          follow_status_new: follow_status_new,
          user_id: data.user_id,
          other_user_id: data.other_user_id,
        });
      }
      if (user_data_arr <= 0) {
        user_data_arr = "NA";
      }
      return response.status(200).json({
        success: true,
        msg: languageMessages.msgDataFound,
        user_data_arr: user_data_arr,
      });
    });
  } catch (error) {
    return response.status(200).json({
      success: false,
      msg: languageMessages.internalServerError,
      resultData: resultData,
    });
  }
};

const saveMusaa = (request, response) => {
  const { user_id, musa_id } = request.body;
  try {
    if (!user_id) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "user_id",
      });
    }
    if (!musa_id) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "musa_id",
      });
    }

    // check user start
    var sqlCheckUser =
      "SELECT user_id, active_flag FROM user_master WHERE user_id = ? AND delete_flag = 0";
    connection.query(sqlCheckUser, [user_id], (err, result) => {
      if (err) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.internalServerError,
          error: err,
        });
      }
      if (result.length === 0) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.msgUserNotFound,
          key: "1",
        });
      }
      if (result[0].active_flag === 0) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.accountdeactivated,
          active_status: 0,
        });
      }
      if (result.length > 0) {
        // check musa start
        var type = "";
        var sqlmusa =
          "SELECT user_id,type FROM musa_master where musa_id = ? and delete_flag = 0";
        connection.query(sqlmusa, [musa_id], (err, musaResult) => {
          if (err) {
            return response.status(200).json({
              success: false,
              msg: languageMessages.internalServerError,
              error: err,
            });
          }
          if (musaResult.length <= 0) {
            return response.status(200).json({
              success: false,
              msg: languageMessages.musaNotFound,
              key: "1",
            });
          }
          if (musaResult.length > 0) {
            var selectmusa =
              "SELECT * from saved_musaa where user_id = ? AND musa_id = ? AND delete_flag = 0";
            connection.query(selectmusa, [user_id, musa_id], (err, result) => {
              if (err) {
                return response.status(200).json({
                  success: false,
                  msg: languageMessages.internalServerError,
                  error: err,
                });
              }
              if (result.length > 0) {
                var insertData =
                  "DELETE FROM saved_musaa WHERE user_id = ? AND musa_id = ? AND delete_flag = 0";
                connection.query(
                  insertData,
                  [user_id, musa_id],
                  (err, data) => {
                    if (err) {
                      return response.status(200).json({
                        success: false,
                        msg: languageMessages.internalServerError,
                        error: err,
                      });
                    } else {
                      return response.status(200).json({
                        success: true,
                        msg: languageMessages.postUnSavedSuccessfully,
                      });
                    }
                  }
                );
              }
              if (result.length <= 0) {
                type = musaResult[0].type;
                // check musa end
                var insertData =
                  "INSERT INTO saved_musaa(user_id,musa_id,type,createtime,updatetime) values (? ,? ,?, now(),now())";
                connection.query(
                  insertData,
                  [user_id, musa_id, type],
                  (err, data) => {
                    if (err) {
                      return response.status(200).json({
                        success: false,
                        msg: languageMessages.internalServerError,
                        error: err,
                      });
                    } else {
                      return response.status(200).json({
                        success: true,
                        msg: languageMessages.postSavedSuccessfully,
                      });
                    }
                  }
                );
              }
            });
          }
        });
      }
      // check user end
    });
  } catch (error) {
    return response.status(200).json({
      success: false,
      msg: languageMessages.internalServerError,
      error: error,
    });
  }
};

const myAlbumSubalbum = async (request, response) => {
  const { user_id } = request.query;
  try {
    if (!user_id) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "user_id",
      });
    }

    // Check user
    var sqlcheak =
      "SELECT user_id,active_flag FROM user_master WHERE user_id = ? AND delete_flag = 0";
    connection.query(sqlcheak, [user_id], async (err, result) => {
      if (err) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.internalServerError,
          error: err,
        });
      }

      if (result.length <= 0) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.msgDataNotFound,
          key: "1",
        });
      }

      if (result[0].active_flag === 0) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.accountdeactivated,
          active_status: 0,
        });
      }

      if (result.length > 0) {
        var sqlmusas =
          "SELECT musa_id, category,sub_category, user_id, description, type FROM musa_master WHERE user_id = ? AND delete_flag = 0";
        connection.query(sqlmusas, [user_id], async (err, info) => {
          if (err) {
            return response.status(200).json({
              success: false,
              msg: languageMessages.internalServerError,
              error: err,
            });
          }

          if (info.length > 0) {
            var subcategory_arr = [];
            var category = await getCategory(user_id);
            var subcategory;
            var album_image;
            // const subcategory = await getSubcategory(user_id, data.category_id);
            for (var data of info) {
              console.log(user_id, data.musa_id, "user_id,data.musa_id");
              subcategory = await getSubcategory(user_id, data.sub_category);
              album_image = await getMusaImage1(user_id, data.musa_id);
              subcategory_arr.push({
                subcategory: subcategory,
                album_image: album_image,
              });
            }

            var album_subalbum_Data = {
              category: category,
              subcategory_arr: subcategory_arr,
            };
            return response.status(200).json({
              success: true,
              msg: languageMessages.msgDataFound,
              subcategory_arr: album_subalbum_Data,
            });
          } else {
            return response
              .status(200)
              .json({ success: false, msg: languageMessages.msgDataNotFound });
          }
        });
      }
    });
  } catch (error) {
    return response.status(200).json({
      success: false,
      msg: languageMessages.internalServerError,
      error: error,
    });
  }
};

async function getCategory(user_id) {
  // Database connection should be established before executing queries
  const query =
    "SELECT category_id, title FROM category_master WHERE (user_id = 1 OR user_id = ?) AND delete_flag = 0";

  try {
    const rows = await new Promise((resolve, reject) => {
      connection.query(query, [user_id], (error, rows) => {
        if (error) {
          reject(error);
        } else {
          resolve(rows);
        }
      });
    });

    return rows.length <= 0 ? "NA" : rows;
  } catch (error) {
    throw error;
  }
}

async function getSubcategory(user_id, category_id) {
  const subCategoryData = [];
  console.log(category_id, "category_id");
  var sub_category_ids = category_id.split(",").map((id) => id.trim());

  const sqlSubCategory =
    "SELECT sub_category_id, user_id, category_id, title FROM sub_category_master WHERE sub_category_id IN (?) AND (user_id=1 OR user_id=?) AND delete_flag = 0";

  try {
    const subResult = await new Promise((resolve, reject) => {
      connection.query(
        sqlSubCategory,
        [sub_category_ids, user_id],
        (err, result) => {
          if (err) {
            console.error("Error : ", err);
            reject(err);
          } else {
            resolve(result.length <= 0 ? "NA" : result);
          }
        }
      );
    });

    subCategoryData.push(...subResult);
    return subCategoryData;
  } catch (error) {
    throw error;
  }
}

async function getMusaImage1(user_id, musa_id) {
  const sqlSelect =
    "SELECT musa_image_id, image_audio FROM musa_image_master WHERE musa_id = ? AND user_id = ? AND image_audio_type = 1 AND delete_flag = 0";
  return new Promise((resolve, reject) => {
    connection.query(sqlSelect, [musa_id, user_id], (err, result) => {
      if (err) {
        reject(err);
      } else {
        let musa_image_data_arr = [];
        if (result.length > 0) {
          musa_image_data_arr.push({
            musa_image_id: result[0].musa_image_id,
            image_audio: result[0].image_audio,
          });
        }
        if (musa_image_data_arr.length <= 0) {
          musa_image_data_arr = "NA";
        }
        resolve(musa_image_data_arr);
      }
    });
  });
}

const getMyAlbum = async (request, response) => {
  const { user_id } = request.query;
  try {
    if (!user_id) {
      return response
        .status(200)
        .json({ success: false, msg: languageMessages.msg_empty_param });
    }

    // Check user
    var sqlcheak =
      "SELECT user_id,active_flag FROM user_master WHERE user_id = ? AND delete_flag = 0";
    connection.query(sqlcheak, [user_id], async (err, result) => {
      if (err) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.internalServerError,
          error: err,
        });
      }

      if (result.length <= 0) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.msgUserNotFound,
          key: "1",
        });
      }

      if (result[0].active_flag === 0) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.accountdeactivated,
          active_status: 0,
        });
      }

      var subcategory_arr = [];

      var sqlmusas =
        "SELECT musa_id, category, sub_category, user_id, description, type FROM musa_master WHERE user_id = ? AND delete_flag = 0";

      connection.query(sqlmusas, [user_id], async (err, info) => {
        if (err) {
          return response.status(200).json({
            success: false,
            msg: languageMessages.internalServerError,
            error: err,
          });
        }

        if (info.length > 0) {
          for (var data of info) {
            const query =
              "SELECT category_id, title FROM category_master WHERE (user_id = 1 OR user_id = ?) AND delete_flag = 0";
            const categoryRows = await new Promise((resolve, reject) => {
              connection.query(query, [user_id], (error, rows) => {
                if (error) {
                  reject(error);
                } else {
                  resolve(rows);
                }
              });
            });
            const sqlSelect =
              "SELECT musa_image_id, image_audio FROM musa_image_master WHERE musa_id = ? AND user_id = ? AND image_audio_type = 1 AND delete_flag = 0";
            const rowsimage = await new Promise((resolve, reject) => {
              connection.query(
                sqlSelect,
                [data.musa_id, user_id],
                (error, rows) => {
                  if (error) {
                    reject(error);
                  } else {
                    resolve(rows);
                  }
                }
              );
            });

            var rowssub;
            if (categoryRows.length > 0) {
              var sub_category = data.sub_category;
              console.log(sub_category, "sub_category");
              var sub_category_ids = sub_category
                .split(",")
                .map((id) => id.trim());
              console.log(sub_category_ids, "sub_category_ids--->");
              const sqlSubCategory =
                "SELECT sub_category_id, user_id, category_id, title FROM sub_category_master WHERE sub_category_id IN (?)   AND (user_id=1 OR user_id=?) AND delete_flag = 0";
              rowssub = await new Promise((resolve, reject) => {
                connection.query(
                  sqlSubCategory,
                  [sub_category_ids, user_id],
                  (error, rows) => {
                    if (error) {
                      reject(error);
                    } else {
                      resolve(rows);
                    }
                  }
                );
              });
            }
            subcategory_arr.push({
              category: categoryRows.length > 0 ? categoryRows : "NA",
              sub_category: rowssub.length > 0 ? rowssub : "NA",
              image: rowsimage.length > 0 ? rowsimage : "NA",
            });
          }
        }
        if (subcategory_arr.length <= 0) {
          subcategory_arr = "NA";
        }

        return response.status(200).json({
          success: true,
          msg: languageMessages.msgDataFound,
          subcategory_arr: subcategory_arr,
        });
      });
    });
  } catch (error) {
    return response.status(200).json({
      success: false,
      msg: languageMessages.msg_empty_param,
      error: error,
    });
  }
};

const homeAlbum = async (request, response) => {
  const { user_id } = request.query;
  try {
    if (!user_id) {
      return response.status(400).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "user_id",
      });
    }

    // Check user
    const sqlCheck =
      "SELECT user_id, active_flag FROM user_master WHERE user_id = ? AND delete_flag = 0";
    connection.query(sqlCheck, [user_id], async (err, userResult) => {
      if (err) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.internalServerError,
          error: err,
        });
      }

      if (userResult.length <= 0) {
        return response
          .status(200)
          .json({ success: false, msg: languageMessages.msgUserNotFound });
      }

      if (userResult[0].active_flag === 0) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.accountdeactivated,
          active_status: 0,
        });
      }

      const sqlMusas =
        "SELECT musa_id, category, sub_category,description,type,title FROM musa_master WHERE user_id = ? AND delete_flag = 0 order by musa_id desc LIMIT 5";

      await connection.query(sqlMusas, [user_id], async (err1, musaResult) => {
        if (err1) {
          return response.status(200).json({
            success: false,
            msg: languageMessages.internalServerError,
            error: err1,
          });
        }

        var album_ar = [];

        if (musaResult.length <= 0) {
          album_ar = "NA";
        } else {
          for (const musaData of musaResult) {
            const categoryData = await getSingleAllCategoryData(
              musaData.category
            );
            const subcategory = await getSingleSubCategoryData(
              musaData.sub_category,
              musaData.category
            );

            const images = await getHomevideoImages(musaData.musa_id, user_id);
            // if (categoryData.length > 0) {

            album_ar.push({
              musa_id: musaData.musa_id,
              title: musaData.title,
              category_title: categoryData ? categoryData : "NA",
              subcategory: subcategory,
              images: images,
            });
          }
        }
        const notification_count = await getNotificationCountOtherUser(user_id);
        await fetchUserData(user_id, async (error, userDataArray) => {
          if (error) {
            console.error("Error fetching user data:", error);

            return response.status(200).json({
              success: false,
              msg: languageMessages.internalServerError,
            });
          }

          // return response.status(200).json({ success: true, msg: languageMessages.otpsend,userDataArray: userDataArray });
          return response.status(200).json({
            success: true,
            msg: languageMessages.msgDataFound,
            album_ar: album_ar,
            notification_count: notification_count,
            userDataArray: userDataArray,
          });
        });
        // return response.status(200).json({ success: true, msg: languageMessages.msgDataFound, album_ar: album_ar, notification_count:notification_count});
      });
    });
  } catch (error) {
    return response.status(200).json({
      success: false,
      msg: languageMessages.msg_empty_param,
      error: error,
    });
  }
};

function getSingleAllCategoryData(categoryId) {
  return new Promise((resolve, reject) => {
    const query =
      "SELECT category_id, title FROM category_master WHERE category_id = ? AND  delete_flag = 0";
    connection.query(query, [categoryId], (err, categoryData) => {
      if (err) {
        reject(err);
      } else {
        if (categoryData.length > 0) {
          resolve(categoryData);
        } else {
          resolve("NA");
        }
      }
    });
  });
}

async function getSingleSubCategoryData(subcategoryId, categoryId) {
  console.log(subcategoryId, "subcategoryId");
  let subcategory_arr = [];
  if (subcategoryId) {
    var subcat_ids = subcategoryId.split(",");

    for (var ids of subcat_ids) {
      const sqlSelect =
        "SELECT sub_category_id, user_id, category_id, title FROM sub_category_master where sub_category_id = ?  AND category_id = ? AND delete_flag = 0";
      const data = await new Promise((resolve, reject) => {
        connection.query(sqlSelect, [ids, categoryId], (err, data1) => {
          if (err) reject(err);
          else resolve(data1);
        });
      });

      for (const row of data) {
        subcategory_arr.push({
          sub_category_id: row.sub_category_id,
          category_id: row.category_id,
          sub_category: row.sub_category,
          title: row.title,
          user_id: row.user_id,
        });
      }
    }
  }

  if (subcategory_arr.length <= 0) {
    return "NA";
  }

  return subcategory_arr;
}

async function getAllCategoryData1(user_id) {
  return new Promise((resolve, reject) => {
    const query =
      "SELECT category_id, title,user_id FROM category_master WHERE  (user_id = 1 OR user_id = ?) AND delete_flag = 0";
    connection.query(query, [user_id], (err, categoryData) => {
      if (err) {
        reject(err);
      } else {
        resolve(categoryData);
      }
    });
  });
}

const getAllCategoryData = async (request, response) => {
  const { user_id, life_status } = request.query;
  try {
    if (!user_id) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "user_id",
      });
    }

    // Check user
    var sqlcheak =
      "SELECT user_id,active_flag FROM user_master WHERE user_id = ? AND delete_flag = 0";
    connection.query(sqlcheak, [user_id], async (err, result) => {
      if (err) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.internalServerError,
          error: err,
        });
      }

      if (result.length <= 0) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.msgUserNotFound,
          key: "1",
        });
      }

      if (result[0].active_flag === 0) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.accountdeactivated,
          active_status: 0,
        });
      }
    });
    // cheak user end
    const query =
      "SELECT user_id,category_id, title FROM category_master WHERE (user_id = 1 OR user_id = ?) AND delete_flag = 0";
    connection.query(query, [user_id], (err, categoryData) => {
      if (err) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.internalServerError,
          error: err,
        });
      }
      var category_arr = [];
      if (categoryData.length > 0) {
        for (var data of categoryData) {
          let status = false;
          if (life_status == 1 && data.title == "Musa Life") {
            status = true;
          }

          category_arr.push({
            user_id: data.user_id,
            category_id: data.category_id,
            title: data.title,
            status: status,
          });
        }
      }
      return response.status(200).json({
        success: true,
        msg: languageMessages.msgDataFound,
        category_arr: category_arr.length > 0 ? category_arr : "NA",
      });
    });
  } catch (error) {
    return response.status(200).json({
      success: false,
      msg: languageMessages.msg_empty_param,
      error: error,
    });
  }
};

const getAllSubCategory = (request, response) => {
  const { user_id, category_id, life_status } = request.query;
  try {
    if (!user_id) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "user_id",
      });
    }
    if (!category_id) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "category_id",
      });
    }
    // Check user
    var sqlcheak =
      "SELECT user_id,active_flag FROM user_master WHERE user_id = ? AND delete_flag = 0";
    connection.query(sqlcheak, [user_id], async (err, result) => {
      if (err) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.internalServerError,
          error: err,
        });
      }

      if (result.length <= 0) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.msgUserNotFound,
          key: "1",
        });
      }

      if (result[0].active_flag === 0) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.accountdeactivated,
          active_status: 0,
        });
      }
      // cheak user end
      const query =
        "SELECT `sub_category_id`, `user_id`, `category_id`, `title` FROM `sub_category_master` WHERE (category_id = ?)  AND delete_flag = 0";
      connection.query(query, [category_id], (err, subCategoryData) => {
        if (err) {
          return response.status(200).json({
            success: false,
            msg: languageMessages.internalServerError,
            error: err,
          });
        }
        var sub_category_arr = [];
        if (subCategoryData.length > 0) {
          for (var data of subCategoryData) {
            let status = false;
            if (life_status == 1 && category_id == 159) {
              status = true;
            }

            sub_category_arr.push({
              user_id: data.user_id,
              sub_category_id: data.sub_category_id,
              category_id: data.category_id,
              title: data.title,
              status: status,
            });
          }
        }
        return response.status(200).json({
          success: true,
          msg: languageMessages.msgDataFound,
          sub_category_arr:
            sub_category_arr.length > 0 ? sub_category_arr : "NA",
        });
      });
    });
    // cheak user end
  } catch (error) {
    return response.status(200).json({
      success: false,
      msg: languageMessages.msg_empty_param,
      error: error,
    });
  }
};

const addNewCategory = async (request, response) => {
  const { user_id, title } = request.body;
  try {
    if (!user_id) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "user_id",
      });
    }
    if (!title) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "title",
      });
    }

    // Check user
    var sqlcheak =
      "SELECT user_id,active_flag FROM user_master WHERE user_id = ? AND delete_flag = 0";
    connection.query(sqlcheak, [user_id], async (err, result) => {
      if (err) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.internalServerError,
          error: err,
        });
      }

      if (result.length <= 0) {
        return response
          .status(200)
          .json({ success: false, msg: languageMessages.msgUserNotFound });
      }

      if (result[0].active_flag === 0) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.accountdeactivated,
          active_status: 0,
        });
      }

      // cheak user end
      const query =
        "SELECT user_id,category_id, title FROM category_master WHERE (user_id = 1 OR user_id = ?) AND delete_flag = 0";
      connection.query(query, [user_id], async (err, categoryData) => {
        if (err) {
          return response.status(200).json({
            success: false,
            msg: languageMessages.internalServerError,
            error: err,
          });
        } else {
          if (categoryData.length > 0) {
            for (var data of categoryData) {
              if (data.title === title) {
                var catData = await getAllCategoryData1(user_id);
                return response.status(200).json({
                  success: false,
                  msg: languageMessages.catogeryAlreadyExists,
                  user_id: data.user_id,
                  categoryData: catData,
                });
              }
            }
            const insertCat =
              "INSERT INTO category_master(user_id, title,createtime,updatetime) VALUES (?,?,now(),now())";
            connection.query(
              insertCat,
              [user_id, title],
              async (err, result) => {
                if (err) {
                  return response.status(200).json({
                    success: true,
                    msg: languageMessages.internalServerError,
                    err: err,
                  });
                }
                var catData = await getAllCategoryData1(user_id);
                return response.status(200).json({
                  success: true,
                  msg: languageMessages.catogeryAddSuccessfully,
                  categoryData: catData,
                });
              }
            );
          }
        }
      });
    });
  } catch (error) {
    return response.status(200).json({
      success: false,
      msg: languageMessages.msg_empty_param,
      error: error,
    });
  }
};

const addCategorySubcategory = (request, response) => {
  const { user_id, category_id, subCategory_title } = request.body;
  try {
    if (!user_id) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "user_id",
      });
    }
    if (!category_id) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "category_id",
      });
    }
    if (!subCategory_title) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "subCategory_title",
      });
    }

    // Check user
    var sqlcheak =
      "SELECT user_id,active_flag FROM user_master WHERE user_id = ? AND delete_flag = 0";
    connection.query(sqlcheak, [user_id], async (err, result) => {
      if (err) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.internalServerError,
          error: err,
        });
      }

      if (result.length <= 0) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.msgUserNotFound,
          key: "1",
        });
      }

      if (result[0].active_flag === 0) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.accountdeactivated,
          active_status: 0,
        });
      }

      // Check category and subcategory
      // const query = "SELECT user_id,category_id, title FROM category_master WHERE (user_id = 1 OR user_id = ?) AND delete_flag = 0";
      // connection.query(query, [user_id], async (err, categoryData) => {
      //     if (err) {
      //         return response.status(200).json({ success: false, msg: languageMessages.internalServerError, error: err });
      //     } else {
      //         if (categoryData.length > 0) {
      //             for (var data of categoryData) {
      //                 if (data.title === category_title) {
      //                     return response.status(200).json({ success: false, msg: languageMessages.catogeryAlreadyExists,user_id : data.user_id });
      //                 }
      //             }
      const sql_sub_cat_data =
        "SELECT sub_category_id, user_id,category_id,title FROM  sub_category_master WHERE  category_id = ? AND (user_id = 1 OR user_id = ?) AND delete_flag = 0";
      connection.query(
        sql_sub_cat_data,
        [category_id, user_id],
        async (err, info) => {
          if (err) {
            return response.status(200).json({
              success: false,
              msg: languageMessages.internalServerError,
              error: err,
            });
          }
          for (var data of info) {
            if (data.title === subCategory_title) {
              return response.status(200).json({
                success: false,
                msg: languageMessages.subCatogeryAlreadyExists,
                user_id: data.user_id,
              });
            }
          }
          // var category_id = 0;
          // const insertCat = "INSERT INTO category_master(user_id, , createtime, updatetime) VALUES (?,?,now(),now())";
          // connection.query(insertCat, [user_id, category_id], async (err, result) => {
          //     if (err) {
          //         return response.status(200).json({ success: false, msg: languageMessages.internalServerError, err: err });
          //     }
          // category_id = result.insertId;
          const insertsubCat =
            "INSERT INTO sub_category_master(user_id, category_id, title, createtime, updatetime) VALUES (?, ?, ?, now(), now())";
          connection.query(
            insertsubCat,
            [user_id, category_id, subCategory_title],
            (err, subResult) => {
              if (err) {
                return response.status(200).json({
                  success: false,
                  msg: languageMessages.internalServerError,
                  err: err,
                });
              }
              return response.status(200).json({
                success: true,
                msg: languageMessages.categoryANDsubCategoryAdd,
              });
            }
          );
          //     });
        }
      );
      //             }
      //         }
      //     });
    });
  } catch (error) {
    return response.status(200).json({
      success: false,
      msg: languageMessages.msg_empty_param,
      error: error,
    });
  }
};

// const removeContributor = (request,response) =>{
//     const {user_id,other_user_id} = request.query;
//     try {
//         if (!user_id) {
//             return response.status(200).json({ success: false, msg: languageMessages.msg_empty_param, key: "user_id" });
//         }
//         if (!other_user_id) {
//             return response.status(200).json({ success: false, msg: languageMessages.msg_empty_param, key: "other_user_id" });
//         }
//          // Check user
//         var sqlcheak = "SELECT user_id,active_flag FROM user_master WHERE user_id = ? AND delete_flag = 0";
//         connection.query(sqlcheak, [user_id], async (err, result) => {
//             if (err) {
//                 return response.status(200).json({ success: false, msg: languageMessages.internalServerError, error: err });
//             }

//             if (result.length <= 0) {
//                 return response.status(200).json({ success: false, msg: languageMessages.msgUserNotFound});
//             }

//             if (result[0].active_flag === 0) {
//                 return response.status(200).json({ success: false, msg: languageMessages.accountdeactivated, active_status: 0 });
//             }
//             // OTHER USER CHEAK
//             var sqlcheak = "SELECT user_id,active_flag FROM user_master WHERE user_id = ? AND delete_flag = 0";
//             connection.query(sqlcheak, [other_user_id], async (err, resultOtherUSER) => {
//                 if (err) {
//                     return response.status(200).json({ success: false, msg: languageMessages.internalServerError, error: err });
//                 }

//                 if (resultOtherUSER.length <= 0) {
//                     return response.status(200).json({ success: false, msg: languageMessages.msgOtherNotFound});
//                 }

//                 if (resultOtherUSER[0].active_flag === 0) {
//                     return response.status(200).json({ success: false, msg: languageMessages.accountdeactivated, active_status: 0 });
//                 }
//                 var removeSQL = "delete from contributor_follow_master where user_id = ? AND delete_flag = 0";
//                 connection.query(removeSQL,[other_user_id],(err,result)=>{
//                     if(err) {
//                         return response.status(200).json({ success: false, msg: languageMessages.internalServerError, error: err });
//                     } else {
//                         return response.status(200).json({ success: true, msg: languageMessages.removeRequest });
//                     }
//                 });
//             });
//         });
//     } catch (error) {
//         return response.status(200).json({ success: false, msg: languageMessages.msg_empty_param, error: error });
//     }
// }

const removeContributor = (request, response) => {
  const { user_id, other_user_id } = request.query;

  // Check if user_id and other_user_id are provided
  if (!user_id) {
    return response.status(200).json({
      success: false,
      msg: languageMessages.msg_empty_param,
      key: "user_id",
    });
  }
  if (!other_user_id) {
    return response.status(200).json({
      success: false,
      msg: languageMessages.msg_empty_param,
      key: "other_user_id",
    });
  }

  // Check user
  const checkUserSQL =
    "SELECT user_id, active_flag FROM user_master WHERE user_id = ? AND delete_flag = 0";
  connection.query(checkUserSQL, [user_id], (err, userResult) => {
    if (err) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.internalServerError,
        error: err,
      });
    }

    if (userResult.length <= 0) {
      return response
        .status(200)
        .json({ success: false, msg: languageMessages.msgUserNotFound });
    }

    if (userResult[0].active_flag === 0) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.accountdeactivated,
        active_status: 0,
      });
    }

    // Check other user
    connection.query(checkUserSQL, [other_user_id], (err, otherUserResult) => {
      if (err) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.internalServerError,
          error: err,
        });
      }

      if (otherUserResult.length <= 0) {
        return response
          .status(200)
          .json({ success: false, msg: languageMessages.msgOtherNotFound });
      }

      if (otherUserResult[0].active_flag === 0) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.accountdeactivated,
          active_status: 0,
        });
      }

      // Remove contributor
      const removeSQL =
        "DELETE FROM contributor_follow_master WHERE user_id = ? AND other_user_id = ? AND delete_flag = 0";
      connection.query(removeSQL, [other_user_id, user_id], (err, result) => {
        if (err) {
          return response.status(200).json({
            success: false,
            msg: languageMessages.internalServerError,
            error: err,
          });
        }

        return response
          .status(200)
          .json({ success: true, msg: languageMessages.removeRequest });
      });
    });
  });
};

const imageUpload = (request, response) => {
  const { user_id } = request.body;
  try {
    if (!user_id) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "user_id",
      });
    }

    // Check user
    var sqlcheak =
      "SELECT user_id,active_flag FROM user_master WHERE user_id = ? AND delete_flag = 0";
    connection.query(sqlcheak, [user_id], async (err, result) => {
      if (err) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.internalServerError,
          error: err,
        });
      }

      if (result.length <= 0) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.msgUserNotFound,
          key: "1",
        });
      }

      if (result[0].active_flag === 0) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.accountdeactivated,
          active_status: 0,
        });
      }
      var image = request.files["image"];
      if (image) {
        return response.status(200).json({
          success: true,
          msg: "image uplode successfully...!!",
          image: image,
        });
      } else {
        return response
          .status(200)
          .json({ success: false, msg: "send image...!!" });
      }
    });
  } catch (error) {
    return response.status(200).json({
      success: false,
      msg: languageMessages.msg_empty_param,
      error: error,
    });
  }
};

const audioUpload = (request, response) => {
  const { user_id } = request.body;
  try {
    if (!user_id) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "user_id",
      });
    }
    // Check user
    var sqlcheak =
      "SELECT user_id,active_flag FROM user_master WHERE user_id = ? AND delete_flag = 0";
    connection.query(sqlcheak, [user_id], async (err, result) => {
      if (err) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.internalServerError,
          error: err,
        });
      }

      if (result.length <= 0) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.msgUserNotFound,
          key: "1",
        });
      }

      if (result[0].active_flag === 0) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.accountdeactivated,
          active_status: 0,
        });
      }
      var audio = request.files["audio"];
      if (audio) {
        return response.status(200).json({
          success: true,
          msg: "audio uplode successfully...!!",
          audio: audio,
        });
      } else {
        return response
          .status(200)
          .json({ success: true, msg: "plase send audio...!!", audio: audio });
      }
    });
  } catch (error) {
    return response.status(200).json({
      success: false,
      msg: languageMessages.msg_empty_param,
      error: error,
    });
  }
};

const videoUploadFolder = (request, response) => {
  const { user_id } = request.body;
  try {
    if (!user_id) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "user_id",
      });
    }
    // Check user
    var sqlcheak =
      "SELECT user_id,active_flag FROM user_master WHERE user_id = ? AND delete_flag = 0";
    connection.query(sqlcheak, [user_id], async (err, result) => {
      if (err) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.internalServerError,
          error: err,
        });
      }

      if (result.length <= 0) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.msgUserNotFound,
          key: "1",
        });
      }

      if (result[0].active_flag === 0) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.accountdeactivated,
          active_status: 0,
        });
      }
      var video = request.files["video"];
      if (video) {
        return response.status(200).json({
          success: true,
          msg: "video uplode successfully...!!",
          video: video,
        });
      } else {
        return response
          .status(200)
          .json({ success: true, msg: "plase send video...!!" });
      }
    });
  } catch (error) {
    return response.status(200).json({
      success: false,
      msg: languageMessages.msg_empty_param,
      error: error,
    });
  }
};

function fetchUserData1(userId, callback) {
  const query = "SELECT * FROM user_master WHERE user_id = ? ";

  connection.query(query, [userId], async (error, results) => {
    if (error) {
      callback(error, null);
      return;
    }
    if (results.length > 0) {
      const userData = results[0];
      let dob_new = formatDate(userData.dob);
      const formatted_dob = moment(dob_new).format("MM/DD/YYYY");
      const age = moment().diff(dob_new, "years");

      const userDataArray = {
        user_id: userData.user_id,
        user_type: userData.user_type,
        user_type_lable: "0=admin,1= user",
        login_type: userData.login_type,
        login_type_lable: "0=app, 1=google, 2=apple",
        email: userData.email,
        f_name: userData.f_name,
        l_name: userData.l_name,
        name: userData.name,
        dob: dob_new,
        formatted_dob: formatted_dob,
        age: age,
        phone_code: userData.phone_code,
        mobile: userData.mobile,
        otp: userData.otp,
        otp_verify: userData.otp_verify,
        image: userData.image,
        bio: userData.bio,
        bio_type: userData.bio_type,
        zipcode: userData.zipcode,
        bio_type_lable: "0 for text, 1 for audio	",
        gender_lable: "1=men,2=women,3=other",
        gender: userData.gender,
        address: userData.address,
        latitude: userData.latitude,
        longitude: userData.longitude,
        active_flag: userData.active_flag,
        approve_flag: userData.approve_flag,
        profile_complete: userData.profile_complete,
        language_id: userData.language_id,
        facebook_id: userData.facebook_id,
        google_id: userData.google_id,
        apple_id: userData.apple_id,
        notification_status: userData.notification_status,
        delete_reason: userData.delete_reason,
        createtime: userData.createtime,
        updatetime: userData.updatetime,
      };
      callback(null, userDataArray);
    } else {
      callback(null, null);
    }
  });
}

function formatDate(dateString) {
  const date = new Date(dateString);
  const year = date.getFullYear();
  const month = String(date.getMonth() + 1).padStart(2, "0");
  const day = String(date.getDate()).padStart(2, "0");
  return `${year}-${month}-${day}`;
}

async function getUser(userId) {
  return new Promise((resolve, reject) => {
    const query1 =
      "SELECT  `user_id`,`image`, `login_type`, `login_type_first`, `user_type`, `email`, `password`, `username`, `f_name`, `l_name`, `name`, `dob`, `age`, `phone_code`, `mobile`, `otp`, `otp_verify`, `image`, `gender`, `address`, `latitude`, `longitude`, `zipcode`, `bio`, `bio_type`, `active_flag`, `approve_flag`, `profile_complete`, `language_id`, `facebook_id`, `google_id`, `twitter_id`, `instagram_id`, `apple_id`, `notification_status`, `delete_flag`, `delete_reason`, `createtime`, `updatetime`, `mysqltime`, `signup_step`, `avatar_id`, `currect_location_permanent`, `about` FROM user_master WHERE user_id = ? and delete_flag=0";

    connection.query(query1, [userId], async (error, results) => {
      if (error) {
        reject(error);
        return;
      }

      if (results.length > 0) {
        const user = results[0];
        try {
          const userDataArray = {
            user_id: user.user_id,
            login_type: user.login_type,
            login_type_first: user.login_type_first,
            user_type: user.user_type,
            email: user.email,
            username: user.username,
            f_name: user.f_name,
            l_name: user.l_name,
            name: user.name,
            image: user.image,
          };
          resolve(userDataArray);
        } catch (err) {
          reject(err);
        }
      } else {
        resolve(null);
      }
    });
  });
}

async function getMusaImage1(musa_id, user_id) {
  const sqlSelect =
    "SELECT `musa_image_id`, `image_audio`, `thumbnail_image`, `musa_id`, `user_id`, `type`, `image_audio_type`,`duration`  FROM `musa_image_master` WHERE musa_id = ? AND user_id = ?";
  return new Promise((resolve, reject) => {
    connection.query(sqlSelect, [musa_id, user_id], (err, result) => {
      if (err) {
        reject(err);
      } else {
        let musa_image_data_arr = [];
        for (const row of result) {
          musa_image_data_arr.push({
            musa_image_id: row.musa_image_id,
            image_audio: row.image_audio,
            musa_id: row.musa_id,
            user_id: row.user_id,
            type: row.type,
            duration: row.duration,
            type_label: "0 for public, 1 for contributor, 3 for only me",
            image_audio_type: row.image_audio_type,
            image_audio_type_label: "1 for image, 2 for audio 3 for video",
            thumbnail_image: row.thumbnail_image ? row.thumbnail_image : null,
          });
        }
        if (musa_image_data_arr.length <= 0) {
          musa_image_data_arr = "NA";
        }
        // resolve(comment_arr);
        resolve(musa_image_data_arr);
      }
    });
  });
}

async function getFollowStatus1(user_id, other_user_id) {
  var sqlstatus =
    "SELECT user_id,other_user_id from contributor_follow_master where user_id= ? AND other_user_id = ? AND delete_flag = 0";
  return new Promise((resolve, reject) => {
    connection.query(sqlstatus, [user_id, other_user_id], (err, result) => {
      if (err) {
        reject(err);
      } else {
        if (result.length > 0) resolve(1);
        else resolve(0);
      }
    });
  });
}

async function getlikeStatus1(musa_id, user_id) {
  var sqlstatus =
    "SELECT user_id,type from musa_like_master where user_id= ? AND musa_id = ? AND delete_flag = 0 ";
  return new Promise((resolve, reject) => {
    connection.query(sqlstatus, [user_id, musa_id], (err, result) => {
      if (err) {
        reject(err);
      } else {
        if (result.length > 0) resolve(1);
        else resolve(0);
      }
    });
  });
}

async function getComment1(musa_id) {
  var sqlcheak =
    "SELECT comment_id, user_id, other_user_id, musa_id, comment, comment_type,createtime from comment_master where musa_id = ? AND delete_flag = 0 order by comment_id desc";
  return new Promise((resolve, reject) => {
    connection.query(sqlcheak, [musa_id], async (err, result) => {
      if (err) {
        reject(err);
      } else {
        // var comment_arr = [];
        // for (const row of result) {
        //     let userData = await getUser(row.other_user_id);
        //     comment_arr.push({
        //         name: (userData ? userData.name : null),
        //         f_name: (userData ? userData.f_name : null),
        //         l_name: (userData ? userData.l_name : null),
        //         comment_id: row.comment_id,
        //         user_id: row.user_id,
        //         time :row.createtime,
        //         other_user_id : row.other_user_id,
        //         musa_id : row.musa_id,
        //         comment : row.comment,
        //         comment_type: row.comment_type,
        //         type_label: "0 for typing type, 1 for audio",

        //     });
        // }
        // if(comment_arr.length <= 0){
        //     comment_arr = "NA";
        // }
        resolve(result.length);
      }
    });
  });
}

async function timeAgo(dateTime) {
  const timestamp = moment(dateTime);
  const currentTime = moment();

  if (!timestamp.isValid()) {
    return "Invalid date";
  }

  const diffSeconds = currentTime.diff(timestamp, "seconds");

  if (diffSeconds < 60) {
    return diffSeconds + " seconds ago";
  }

  const diffMinutes = currentTime.diff(timestamp, "minutes");

  if (diffMinutes < 60) {
    return diffMinutes + " minutes ago";
  }

  const diffHours = currentTime.diff(timestamp, "hours");

  if (diffHours < 24) {
    return diffHours + " hours ago";
  }

  const diffDays = currentTime.diff(timestamp, "days");

  if (diffDays < 30) {
    return diffDays + " days ago";
  }

  const diffMonths = currentTime.diff(timestamp, "months");

  if (diffMonths < 12) {
    return diffMonths + " months ago";
  }

  const diffYears = currentTime.diff(timestamp, "years");
  return diffYears + " years ago";
}

const deleteMusaController = async (request, response) => {
  const { user_id, musa_id } = request.body;

  try {
    if (!user_id) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "user_id",
      });
    }

    if (!musa_id) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "musa_id",
      });
    }

    // check user start
    var sqlcheak =
      "SELECT user_id,active_flag FROM user_master where user_id = ? and delete_flag = 0";

    connection.query(sqlcheak, [user_id], (err, result) => {
      if (err) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.internalServerError,
          error: err,
        });
      }
      if (result.length <= 0) {
        return response
          .status(200)
          .json({ success: false, msg: languageMessages.msgUserNotFound });
      }
      if (result[0].active_flag === 0) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.accountdeactivated,
          active_status: 0,
        });
      }

      var sqlUpdate =
        "UPDATE musa_master SET delete_flag  = 1  WHERE musa_id = ? AND user_id = ?  AND delete_flag = 0";

      connection.query(sqlUpdate, [musa_id, user_id], (err, result) => {
        if (err) {
          return response.status(200).json({
            success: false,
            msg: languageMessages.internalServerError,
            error: err.message,
          });
        }

        if (result.affectedRows > 0) {
          return response.status(200).json({
            success: true,
            msg: languageMessages.musadeleteSuccesssfully,
          });
        }
      });
    });
    //  check user start
  } catch (error) {
    return response.status(200).json({
      success: false,
      msg: languageMessages.internalServerError,
      error: err.message,
    });
  }
};

const getMyAllCategory = async (request, response) => {
  const { user_id } = request.query;

  try {
    if (!user_id) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "user_id",
      });
    }

    // check user start
    var sqlcheak =
      "SELECT user_id,active_flag FROM user_master where user_id = ? and delete_flag = 0";

    connection.query(sqlcheak, [user_id], (err, result) => {
      if (err) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.internalServerError,
          error: err,
        });
      }
      if (result.length <= 0) {
        return response
          .status(200)
          .json({ success: false, msg: languageMessages.msgUserNotFound });
      }
      if (result[0].active_flag === 0) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.accountdeactivated,
          active_status: 0,
        });
      }

      const query =
        "SELECT category_id, title,user_id FROM category_master WHERE  user_id = ? AND delete_flag = 0";

      connection.query(query, [user_id], async (err, categoryData) => {
        if (err) {
          return response.status(200).json({
            success: false,
            msg: languageMessages.internalServerError,
            error: err.message,
          });
        }
        var category_arr = [];
        if (categoryData.length > 0) {
          for (var data of categoryData) {
            var musa_count = await getMusaCount(data.category_id, user_id);
            category_arr.push({
              category_id: data.category_id,
              title: data.title,
              user_id: data.user_id,
              musa_count: musa_count,
            });
          }
        }
        if (category_arr.length <= 0) {
          category_arr = "NA";
        }
        return response.status(200).json({
          success: true,
          msg: languageMessages.msgDataFound,
          category_arr: category_arr,
        });
      });
    });
  } catch (error) {
    return response.status(200).json({
      success: false,
      msg: languageMessages.internalServerError,
      error: error.message,
    });
  }
};

async function getMusaCount(category_id, user_id) {
  var sqlstatus =
    "SELECT count(musa_id) as musa_count from musa_master where user_id= ? AND category = ? AND delete_flag = 0 ";
  return new Promise((resolve, reject) => {
    connection.query(sqlstatus, [user_id, category_id], (err, result) => {
      if (err) {
        reject(err);
      } else {
        if (result.length > 0) resolve(result[0].musa_count);
        else resolve(0);
      }
    });
  });
}

async function getMusaCountSubCategory(subcategory_id, user_id) {
  var sqlstatus =
    "SELECT count(musa_id) as musa_count from musa_master where user_id= ? AND sub_category = ? AND delete_flag = 0 ";
  return new Promise((resolve, reject) => {
    connection.query(sqlstatus, [user_id, subcategory_id], (err, result) => {
      if (err) {
        reject(err);
      } else {
        if (result.length > 0) resolve(result[0].musa_count);
        else resolve(0);
      }
    });
  });
}

const editMyCategory = async (request, response) => {
  const { user_id, category_id, title } = request.body;

  try {
    if (!user_id) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "user_id",
      });
    }

    if (!category_id) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "category_id",
      });
    }

    if (!title) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "title",
      });
    }

    // check user start
    var sqlcheak =
      "SELECT user_id,active_flag FROM user_master where user_id = ? and delete_flag = 0";

    connection.query(sqlcheak, [user_id], (err, result) => {
      if (err) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.internalServerError,
          error: err,
        });
      }
      if (result.length <= 0) {
        return response
          .status(200)
          .json({ success: false, msg: languageMessages.msgUserNotFound });
      }
      if (result[0].active_flag === 0) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.accountdeactivated,
          active_status: 0,
        });
      }

      var sqlUpdate =
        "UPDATE category_master SET title  = ? , updatetime = now()  WHERE category_id = ? AND user_id = ?  AND delete_flag = 0";

      connection.query(
        sqlUpdate,
        [title, category_id, user_id],
        (err, result) => {
          if (err) {
            return response.status(200).json({
              success: false,
              msg: languageMessages.internalServerError,
              error: err.message,
            });
          }

          if (result.affectedRows > 0) {
            return response.status(200).json({
              success: true,
              msg: languageMessages.categoryupdateSuccessfully,
            });
          }
        }
      );
    });
  } catch (error) {
    return response.status(200).json({
      success: false,
      msg: languageMessages.internalServerError,
      error: err.message,
    });
  }
};

const getMyAllSubCategory = async (request, response) => {
  const { user_id } = request.query;

  try {
    if (!user_id) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "user_id",
      });
    }

    // check user start
    var sqlcheak =
      "SELECT user_id,active_flag FROM user_master where user_id = ? and delete_flag = 0";

    connection.query(sqlcheak, [user_id], (err, result) => {
      if (err) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.internalServerError,
          error: err,
        });
      }
      if (result.length <= 0) {
        return response
          .status(200)
          .json({ success: false, msg: languageMessages.msgUserNotFound });
      }
      if (result[0].active_flag === 0) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.accountdeactivated,
          active_status: 0,
        });
      }

      const query =
        "SELECT `sub_category_id`, `user_id`, `category_id`, `title` FROM `sub_category_master` WHERE  user_id = ? AND delete_flag = 0";

      connection.query(query, [user_id], async (err, subcategoryData) => {
        if (err) {
          return response.status(200).json({
            success: false,
            msg: languageMessages.internalServerError,
            error: err.message,
          });
        }
        var subcategory_arr = [];
        if (subcategoryData.length > 0) {
          for (var data of subcategoryData) {
            var musa_count = await getMusaCountSubCategory(
              data.sub_category_id,
              user_id
            );
            subcategory_arr.push({
              sub_category_id: data.sub_category_id,
              category_id: data.category_id,
              title: data.title,
              user_id: data.user_id,
              musa_count: musa_count,
            });
          }
        }
        if (subcategory_arr.length <= 0) {
          subcategory_arr = "NA";
        }
        return response.status(200).json({
          success: true,
          msg: languageMessages.msgDataFound,
          subcategory_arr: subcategory_arr,
        });
      });
    });
  } catch (error) {
    return response.status(200).json({
      success: false,
      msg: languageMessages.internalServerError,
      error: error.message,
    });
  }
};

const editMySubCategory = async (request, response) => {
  const { user_id, category_id, title, sub_category_id } = request.body;

  try {
    if (!user_id) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "user_id",
      });
    }

    if (!category_id) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "category_id",
      });
    }

    if (!sub_category_id) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "sub_category_id",
      });
    }

    if (!title) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "title",
      });
    }

    // check user start
    var sqlcheak =
      "SELECT user_id,active_flag FROM user_master where user_id = ? and delete_flag = 0";

    connection.query(sqlcheak, [user_id], (err, result) => {
      if (err) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.internalServerError,
          error: err,
        });
      }
      if (result.length <= 0) {
        return response
          .status(200)
          .json({ success: false, msg: languageMessages.msgUserNotFound });
      }
      if (result[0].active_flag === 0) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.accountdeactivated,
          active_status: 0,
        });
      }

      var sqlUpdate =
        "UPDATE sub_category_master SET title  = ? , category_id = ?, updatetime = now()  WHERE sub_category_id = ? AND user_id = ?  AND delete_flag = 0";

      connection.query(
        sqlUpdate,
        [title, category_id, sub_category_id, user_id],
        (err, result) => {
          if (err) {
            return response.status(200).json({
              success: false,
              msg: languageMessages.internalServerError,
              error: err.message,
            });
          }

          if (result.affectedRows > 0) {
            return response.status(200).json({
              success: true,
              msg: languageMessages.subcategoryupdateSuccessfully,
            });
          }
        }
      );
    });
  } catch (error) {
    return response.status(200).json({
      success: false,
      msg: languageMessages.internalServerError,
      error: err.message,
    });
  }
};
async function getMusaMedia(musa_id, user_id) {
  const sqlSelect =
    "SELECT `musa_image_id`, `image_audio`, `thumbnail_image`, `musa_id`, `user_id`, `type`, `image_audio_type`,`duration` FROM `musa_image_master` WHERE musa_id = ? AND user_id = ?";
  return new Promise((resolve, reject) => {
    connection.query(sqlSelect, [musa_id, user_id], (err, result) => {
      if (err) {
        reject(err);
      } else {
        var url = "https://d38i924xuvvpvh.cloudfront.net/images/";
        let musa_image_arr = [];

        for (const row of result) {
          if (row.image_audio_type === 1) {
            musa_image_arr.push({
              musa_image_id: row.musa_image_id,
              duration: row.duration,
              image_audio: url + row.image_audio,
            });
          }
        }

        if (musa_image_arr.length === 0) {
          musa_image_arr = "NA";
        }

        resolve(musa_image_arr);
      }
    });
  });
}

async function getMusaImage(musa_id, user_id) {
  const sqlSelect =
    "SELECT `musa_image_id`, `image_audio`, `thumbnail_image`, `musa_id`, `user_id`, `type`, `image_audio_type`,`duration` FROM `musa_image_master` WHERE musa_id = ? AND user_id = ?";
  return new Promise((resolve, reject) => {
    connection.query(sqlSelect, [musa_id, user_id], (err, result) => {
      if (err) {
        reject(err);
      } else {
        var url = "https://d38i924xuvvpvh.cloudfront.net/images/";
        let musa_image_arr = [];

        for (const row of result) {
          if (row.image_audio_type === 1) {
            musa_image_arr.push(url + row.image_audio);
          }
        }

        if (musa_image_arr.length === 0) {
          musa_image_arr = "NA";
        }
        resolve(musa_image_arr);
      }
    });
  });
}

async function getMusaAudio(musa_id, user_id) {
  const sqlSelect =
    "SELECT `musa_image_id`, `image_audio`, `thumbnail_image`, `musa_id`, `user_id`, `type`, `image_audio_type`,`duration` FROM `musa_image_master` WHERE musa_id = ? AND user_id = ?";
  return new Promise((resolve, reject) => {
    connection.query(sqlSelect, [musa_id, user_id], (err, result) => {
      if (err) {
        reject(err);
      } else {
        var url = "https://d38i924xuvvpvh.cloudfront.net/images/";
        let musa_audio_arr = [];

        for (const row of result) {
          if (row.image_audio_type === 2) {
            musa_audio_arr.push({
              musa_image_id: row.musa_image_id,
              duration: row.duration,
              image_audio: url + row.image_audio,
              name : row.image_audio,
            });
          }
        }

        if (musa_audio_arr.length === 0) {
          musa_audio_arr = "NA";
        }

        resolve(musa_audio_arr);
      }
    });
  });
}

async function getMusavideo(musa_id, user_id) {
  const sqlSelect =
    "SELECT `musa_image_id`, `image_audio`, `thumbnail_image`, `musa_id`, `user_id`, `type`, `image_audio_type`,`duration` FROM `musa_image_master` WHERE musa_id = ? and delete_flag=0";
  return new Promise((resolve, reject) => {
    connection.query(sqlSelect, [musa_id], (err, result) => {
      if (err) {
        reject(err);
      } else {
        var url = "https://d38i924xuvvpvh.cloudfront.net/images/";
        let musa_video_arr = [];

        for (const row of result) {
          if (row.image_audio_type === 3) {
            musa_video_arr.push({
              musa_image_id: row.musa_image_id,
              duration: row.duration,
              image_audio: url + row.image_audio,
              thumbnail_image: "NA",
            });
          }
        }

        if (musa_video_arr.length === 0) {
          musa_video_arr = "NA";
        }

        resolve(musa_video_arr);
      }
    });
  });
}

async function getlikecount(musa_id) {
  var sqlstatus =
    "SELECT count(like_id) as like_count from musa_like_master where  musa_id = ? AND delete_flag = 0 ";
  return new Promise((resolve, reject) => {
    connection.query(sqlstatus, [musa_id], (err, result) => {
      if (err) {
        reject(err);
      } else {
        if (result.length > 0) resolve(result[0].like_count);
        else resolve(0);
      }
    });
  });
}

const deleteCategory = async (request, response) => {
  const { user_id, category_id } = request.body;

  try {
    if (!user_id) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "user_id",
      });
    }

    if (!category_id) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "category_id",
      });
    }
    // check user start
    var sqlcheak =
      "SELECT user_id,active_flag FROM user_master where user_id = ? and delete_flag = 0";

    connection.query(sqlcheak, [user_id], (err, result) => {
      if (err) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.internalServerError,
          error: err,
        });
      }
      if (result.length <= 0) {
        return response
          .status(200)
          .json({ success: false, msg: languageMessages.msgUserNotFound });
      }
      if (result[0].active_flag === 0) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.accountdeactivated,
          active_status: 0,
        });
      }

      var sqlUpdate =
        "UPDATE category_master SET delete_flag  = 1  WHERE category_id = ? AND user_id = ?  AND delete_flag = 0";

      connection.query(sqlUpdate, [category_id, user_id],async(err, result) => {
        if (err) {
          return response.status(200).json({
            success: false,
            msg: languageMessages.internalServerError,
            error: err.message,
          });
        }

        if (result.affectedRows > 0) {

          await connection.query( "UPDATE sub_category_master SET delete_flag  = 1  WHERE category_id = ? AND delete_flag = 0", [category_id], (err, result) => {
           
          });


          return response.status(200).json({
            success: true,
            msg: languageMessages.categorydeleteSuccesssfully,
          });
        }
      });
    });
  } catch (error) {
    return response.status(200).json({
      success: false,
      msg: languageMessages.internalServerError,
      error: err.message,
    });
  }
};

const deleteSubCategory = async (request, response) => {
  const { user_id, sub_category_id } = request.body;

  try {
    if (!user_id) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "user_id",
      });
    }

    if (!sub_category_id) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "sub_category_id",
      });
    }
    // check user start
    var sqlcheak =
      "SELECT user_id,active_flag FROM user_master where user_id = ? and delete_flag = 0";

    connection.query(sqlcheak, [user_id], (err, result) => {
      if (err) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.internalServerError,
          error: err,
        });
      }
      if (result.length <= 0) {
        return response
          .status(200)
          .json({ success: false, msg: languageMessages.msgUserNotFound });
      }
      if (result[0].active_flag === 0) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.accountdeactivated,
          active_status: 0,
        });
      }

      var sqlUpdate =
        "UPDATE sub_category_master SET delete_flag  = 1  WHERE sub_category_id = ? AND user_id = ?  AND delete_flag = 0";

      connection.query(sqlUpdate, [sub_category_id, user_id], (err, result) => {
        if (err) {
          return response.status(200).json({
            success: false,
            msg: languageMessages.internalServerError,
            error: err.message,
          });
        }

        if (result.affectedRows > 0) {
          return response.status(200).json({
            success: true,
            msg: languageMessages.subcategorydeleteSuccesssfully,
          });
        }
      });
    });
  } catch (error) {
    return response.status(200).json({
      success: false,
      msg: languageMessages.internalServerError,
      error: err.message,
    });
  }
};

const getCommentData = (musa_id) => {
  var sqlcheak =
    "SELECT comment_id, user_id, musa_id, comment, comment_type,createtime from comment_master where musa_id = ? AND delete_flag = 0 order by comment_id desc";
  return new Promise((resolve, reject) => {
    connection.query(sqlcheak, [musa_id], async (err, result) => {
      if (err) {
        reject(err);
      } else {
        var comment_arr = [];
        for (const row of result) {
          let userData = await getUserData(row.user_id);
          const time = await timeAgo(row.createtime);
          comment_arr.push({
            name: userData ? userData.name : "NA",
            f_name: userData ? userData.f_name : "NA",
            l_name: userData ? userData.l_name : "NA",
            username: userData ? userData.username : "NA",
            comment_id: row.comment_id,
            user_id: row.user_id,
            time: time,
            musa_id: row.musa_id,
            comment: row.comment,
            comment_type: row.comment_type,
            type_label: "0 for typing type, 1 for audio",
          });
        }
        if (comment_arr.length <= 0) {
          comment_arr = "NA";
        }
        resolve(comment_arr);
      }
    });
  });
};

const editMusa = async (request, response) => {
  const {
    user_id,
    musa_id,
    description,
    image,
    audio,
    type,
    category_id,
    sub_category_id,
    musa_title,
  } = request.body;

  try {
    // Check for required parameters
    const requiredParams = {
      user_id,
      category_id,
      sub_category_id,
      type,
      musa_title,
      musa_id,
    };
    for (const [key, value] of Object.entries(requiredParams)) {
      if (!value) {
        return response
          .status(200)
          .json({ success: false, msg: languageMessages.msg_empty_param, key });
      }
    }

    // Function to execute SQL query with a promise
    const query = (sql, params) => {
      return new Promise((resolve, reject) => {
        connection.query(sql, params, (err, result) => {
          if (err) {
            reject(err);
          } else {
            resolve(result);
          }
        });
      });
    };

    // Check if user exists and is active
    const sqlCheckUser =
      "SELECT user_id, active_flag FROM user_master WHERE user_id = ? AND delete_flag = 0";
    const userResult = await query(sqlCheckUser, [user_id]);

    if (userResult.length <= 0) {
      return response
        .status(200)
        .json({ success: false, msg: languageMessages.msgUserNotFound });
    }
    if (userResult[0].active_flag === 0) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.accountdeactivated,
        active_status: 0,
      });
    }

    // Update musa details
    const sqlUpdateMusa =
      "UPDATE musa_master SET description = ?, title = ?, type = ?, sub_category = ?, category = ? WHERE user_id = ? AND musa_id = ? AND delete_flag = 0";
    const updateResult = await query(sqlUpdateMusa, [
      description,
      musa_title,
      type,
      sub_category_id,
      category_id,
      user_id,
      musa_id,
    ]);

    if (updateResult.affectedRows > 0) {
      // Delete existing images
      const sqlDeleteImages =
        "DELETE FROM musa_image_master WHERE musa_id = ? AND user_id = ? AND delete_flag = 0";
      await query(sqlDeleteImages, [musa_id, user_id]);

      // Insert new images
      if (image) {
        for (const imageData of image) {
          const sqlInsertImage =
            "INSERT INTO musa_image_master(image_audio, size, musa_id, user_id, image_audio_type, createtime, updatetime,thumbnail_image,size_flag,duration) VALUES (?, ?, ?, ?, ?, NOW(), NOW(),?,?,?)";
          const values = [
            imageData.image,
            imageData.size,
            musa_id,
            user_id,
            imageData.type,
            imageData.thumbnail,
            imageData.size_flag,
            imageData.duration
          ];
          await query(sqlInsertImage, values);
        }
      }

      // Insert new audio files
      if (audio) {
        for (const audioData of audio) {
          const sqlInsertAudio =
            "INSERT INTO musa_image_master(image_audio, size, musa_id, user_id, image_audio_type, createtime, updatetime) VALUES (?, ?, ?, ?, ?, NOW(), NOW())";
          const values = [audioData.audio, audioData.size, musa_id, user_id, 2];
          await query(sqlInsertAudio, values);
        }
      }

      return response
        .status(200)
        .json({ success: true, msg: languageMessages.musaUpdateSuccessfully });
    }
  } catch (error) {
    return response.status(200).json({
      success: false,
      msg: languageMessages.internalServerError,
      error: error.message,
    });
  }
};

async function getUserData(userId) {
  return new Promise((resolve, reject) => {
    const query1 =
      "SELECT  `user_id`, `login_type`, `login_type_first`, `user_type`, `email`, `password`, `username`, `f_name`, `l_name`, `name`, `dob`, `age`, `phone_code`, `mobile`, `otp`, `otp_verify`, `image`, `gender`, `address`, `latitude`, `longitude`, `zipcode`, `bio`, `bio_type`, `active_flag`, `approve_flag`, `profile_complete`, `language_id`, `facebook_id`, `google_id`, `twitter_id`, `instagram_id`, `apple_id`, `notification_status`, `delete_flag`, `delete_reason`, `createtime`, `updatetime`, `mysqltime`, `signup_step`, `avatar_id`, `currect_location_permanent`, `about` FROM user_master WHERE user_id = ? and delete_flag=0";

    connection.query(query1, [userId], async (error, results) => {
      if (error) {
        reject(error);
        return;
      }

      if (results.length > 0) {
        const userData = results[0];
        try {
          const userDataArray = {
            username: userData.username,
            f_name: userData.f_name,
            l_name: userData.l_name,
            name: userData.name,
            image: "https://d38i924xuvvpvh.cloudfront.net/" + userData.image,
            image_new: userData.image,
          };
          resolve(userDataArray);
        } catch (err) {
          reject(err);
        }
      } else {
        resolve(null);
      }
    });
  });
}

function getHomevideoImages(musa_id, user_id) {
  const sqlSelect =
    "SELECT `musa_image_id`, `image_audio`, `thumbnail_image`, `musa_id`, `user_id`, `type`, `image_audio_type`,`duration` FROM `musa_image_master` WHERE musa_id = ? AND user_id = ? AND (image_audio_type = 1 OR image_audio_type = 3) ";

  return new Promise((resolve, reject) => {
    connection.query(sqlSelect, [musa_id, user_id], (err, result) => {
      if (err) {
        return reject(err);
      }

      if (result.length === 0) {
        return resolve("NA");
      } else {
        const firstItem = {
          musa_image_id: result[0].musa_image_id,
          duration: result[0].duration,
          image_audio_type_text: "1=image,3=video",
          image_audio_type: result[0].image_audio_type,
          image_audio: `https://d38i924xuvvpvh.cloudfront.net/images/${result[0].image_audio}`,
          thumbnail_image: `https://d38i924xuvvpvh.cloudfront.net/${result[0].thumbnail_image}`,
        };
        return resolve(firstItem);
      }
    });
  });
}

// get images and videos for home end

// display request start

const displayRequest = (request, response) => {
  const { user_id, musa_id } = request.body;
  try {
    if (!user_id) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "user_id",
      });
    }

    if (!musa_id) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "musa_id",
      });
    }

    // check user start
    var sqlcheak =
      "SELECT user_id,active_flag,f_name FROM user_master where user_id = ? and delete_flag = 0";

    connection.query(sqlcheak, [user_id], (err, result) => {
      if (err) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.internalServerError,
          error: err,
        });
      }
      if (result.length <= 0) {
        return response
          .status(200)
          .json({ success: false, msg: languageMessages.msgUserNotFound });
      }
      if (result[0].active_flag === 0) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.accountdeactivated,
          active_status: 0,
        });
      }

      // check musa exist or not start

      const checkMusa =
        "SELECT musa_id,user_id FROM `musa_master` WHERE musa_id=? and delete_flag=0";
      const valMusa = [musa_id];
      connection.query(checkMusa, valMusa, async (musaErr, musaResult) => {
        if (musaErr) {
          return response.status(200).json({
            success: false,
            msg: languageMessages.internalServerError,
            error: err,
          });
        }
        if (musaResult.length <= 0) {
          return response
            .status(200)
            .json({ success: false, msg: languageMessages.musaNotFound });
        }

        // send request start

        const requestQuery =
          "INSERT INTO `display_request_master`(`musa_id`, `user_id`, `createtime`, `updatetime`) VALUES (?,?,NOW(),NOW())";
        const requestVal = [musa_id, user_id];
        connection.query(
          requestQuery,
          requestVal,
          async (requestErr, requestResult) => {
            if (requestErr) {
              return response.status(200).json({
                success: false,
                msg: languageMessages.internalServerError,
                error: err,
              });
            }

            let request_id = await requestResult.insertId;

            var username = result[0].f_name;
            // send notification start

            const user_id_notification = user_id;

            const other_user_id_notification = musaResult[0].user_id;

            const action = "display_request";

            const action_id = musa_id;

            const title = "Display Request";

            const messages = username + " has send you display mode request.";

            const action_data = {
              user_id: user_id_notification,
              other_user_id: other_user_id_notification,
              action_id: action_id,
              action: action,
            };

            getNotificationArrSingle(
              user_id_notification,
              other_user_id_notification,
              action,
              action_id,
              title,
              title,
              title,
              title,
              title,
              messages,
              messages,
              messages,
              messages,
              messages,
              action_data,
              async (notification_arr_check) => {
                let notification_arr_check_new = [notification_arr_check];

                if (
                  notification_arr_check_new &&
                  notification_arr_check_new.length !== 0
                ) {
                  const notiSendStatus = await oneSignalNotificationSendCall(
                    notification_arr_check_new
                  );
                } else {
                  console.log("Notification array is empty");
                }
                return response.status(200).json({
                  success: true,
                  msg: languageMessages.displayRequestsentSuccessfully,
                });
              }
            );
            // send notification end
          }
        );

        // send request end
      });

      // check musa exist or not end
    });
  } catch (error) {
    return response.status(200).json({
      success: false,
      msg: languageMessages.internalServerError,
      error: err,
    });
  }
};

// display request end

// accept and reject request start

const acceptRejectRequest = (request, response) => {
  const { user_id, request_id, musa_id, status } = request.body;

  try {
    // Check if required parameters are present
    if (!user_id || !request_id || !musa_id || typeof status === "undefined") {
      const missingParam = !user_id
        ? "user_id"
        : !request_id
        ? "request_id"
        : !musa_id
        ? "musa_id"
        : "status";
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: missingParam,
      });
    }

    // Check user existence and status
    const sqlCheckUser =
      "SELECT user_id, active_flag, f_name FROM user_master WHERE user_id = ? AND delete_flag = 0";
    connection.query(sqlCheckUser, [user_id], (err, userResult) => {
      if (err) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.internalServerError,
          error: err,
        });
      }
      if (userResult.length === 0) {
        return response
          .status(200)
          .json({ success: false, msg: languageMessages.msgUserNotFound });
      }
      if (userResult[0].active_flag === 0) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.accountdeactivated,
          active_status: 0,
        });
      }

      // Check if Musa exists
      const sqlCheckMusa =
        "SELECT musa_id FROM musa_master WHERE musa_id = ? AND delete_flag = 0";
      connection.query(sqlCheckMusa, [musa_id], (musaErr, musaResult) => {
        if (musaErr) {
          return response.status(200).json({
            success: false,
            msg: languageMessages.internalServerError,
            error: musaErr,
          });
        }
        if (musaResult.length === 0) {
          return response
            .status(200)
            .json({ success: false, msg: languageMessages.musaNotFound });
        }

        // Check if request exists
        const sqlCheckRequest =
          "SELECT musa_id, user_id FROM display_request_master WHERE musa_id = ? AND request_id = ? AND delete_flag = 0";
        connection.query(
          sqlCheckRequest,
          [musa_id, request_id],
          (reqErr, reqResult) => {
            if (reqErr) {
              return response.status(200).json({
                success: false,
                msg: languageMessages.internalServerError,
                error: reqErr,
              });
            }
            if (reqResult.length === 0) {
              return response.status(200).json({
                success: false,
                msg: languageMessages.requestNotFound,
              });
            }

            // Update request status
            const sqlUpdateRequest =
              "UPDATE display_request_master SET status = ?, updatetime = NOW() WHERE delete_flag = 0 AND request_id = ?";
            connection.query(
              sqlUpdateRequest,
              [status, request_id],
              (updateErr, updateResult) => {
                if (updateErr) {
                  return response.status(200).json({
                    success: false,
                    msg: languageMessages.internalServerError,
                    error: updateErr,
                  });
                }

                // Send notification
                const userIdNotification = user_id;
                const otherUserIdNotification = reqResult[0].user_id;
                let action, title, messages;

                if (status == 1) {
                  action = "display_request_accept";
                  title = "Request Accepted";
                  messages = `${userResult[0].f_name} has accepted your request.`;
                } else {
                  action = "display_request_reject";
                  title = "Request Rejected";
                  messages = `${userResult[0].f_name} has rejected your request.`;
                }

                const actionId = musa_id;
                const actionData = {
                  user_id: userIdNotification,
                  other_user_id: otherUserIdNotification,
                  action_id: actionId,
                  action: action,
                };

                getNotificationArrSingle(
                  userIdNotification,
                  otherUserIdNotification,
                  action,
                  actionId,
                  title,
                  title,
                  title,
                  title,
                  title,
                  messages,
                  messages,
                  messages,
                  messages,
                  messages,
                  actionData,
                  async (notificationArrCheck) => {
                    let notificationArrCheckNew = [notificationArrCheck];

                    if (
                      notificationArrCheckNew &&
                      notificationArrCheckNew.length !== 0
                    ) {
                      const notiSendStatus =
                        await oneSignalNotificationSendCall(
                          notificationArrCheckNew
                        );
                    } else {
                      console.log("Notification array is empty");
                    }

                    let successMessage =
                      status == 1
                        ? languageMessages.displayAcceptedsentSuccessfully
                        : languageMessages.displayRejectedsentSuccessfully;
                    return response
                      .status(200)
                      .json({ success: true, msg: successMessage });
                  }
                );
              }
            );
          }
        );
      });
    });
  } catch (error) {
    return response.status(200).json({
      success: false,
      msg: languageMessages.internalServerError,
      error: error,
    });
  }
};

// accept and reject request end

// get all display request start

const getAllDisplayRequest = async (request, response) => {
  const { user_id } = request.query;

  try {
    if (!user_id) {
      return response.status(400).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "user_id",
      });
    }

    const sqlCheck =
      "SELECT user_id, active_flag, f_name FROM user_master WHERE user_id = ? AND delete_flag = 0";
    connection.query(sqlCheck, [user_id], (err, result) => {
      if (err) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.internalServerError,
          error: err,
        });
      }

      if (result.length <= 0) {
        return response
          .status(404)
          .json({ success: false, msg: languageMessages.msgUserNotFound });
      }

      if (result[0].active_flag === 0) {
        return response.status(403).json({
          success: false,
          msg: languageMessages.accountdeactivated,
          active_status: 0,
        });
      }

      const getMusaQuery = `
                SELECT mm.musa_id, mm.category, drm.request_id, drm.user_id, drm.status 
                FROM musa_master AS mm 
                INNER JOIN display_request_master AS drm 
                ON mm.musa_id = drm.musa_id 
                WHERE mm.delete_flag = 0 AND mm.user_id = ? and drm.status = 0
            `;
      const getMusaValues = [user_id];

      connection.query(
        getMusaQuery,
        getMusaValues,
        async (getMusaErr, getMusaResult) => {
          if (getMusaErr) {
            return response.status(200).json({
              success: false,
              msg: languageMessages.internalServerError,
              error: getMusaErr,
            });
          }

          let requestArr = [];

          if (getMusaResult.length > 0) {
            try {
              for (const data of getMusaResult) {
                const category = await getSingleAllCategoryData(data.category);
                let userData = await getUserData(data.user_id);
                requestArr.push({
                  name: userData ? userData.name : "NA",
                  f_name: userData ? userData.f_name : "NA",
                  l_name: userData ? userData.l_name : "NA",
                  username: userData ? userData.username : "NA",
                  image: userData ? userData.image : "NA",
                  musa_id: data.musa_id,
                  category_id: data.category,
                  category: category,
                  request_id: data.request_id,
                  user_id: data.user_id,
                  status_text: "0=pending,1=accept,2=reject",
                  status: data.status,
                });
              }
              return response.status(200).json({
                success: true,
                msg: languageMessages.msgDataFound,
                request_arr: requestArr,
              });
            } catch (asyncErr) {
              return response.status(200).json({
                success: false,
                msg: languageMessages.internalServerError,
                error: asyncErr,
              });
            }
          } else {
            return response.status(200).json({
              success: true,
              msg: languageMessages.msgDataFound,
              request_arr: "NA",
            });
          }
        }
      );
    });
  } catch (error) {
    return response.status(200).json({
      success: false,
      msg: languageMessages.internalServerError,
      error: error,
    });
  }
};

// my musa on category or subcategory wise start

const getMyMusaCategoryWise = async (request, response) => {
  const { user_id } = request.query;

  try {
    if (!user_id) {
      return response.status(400).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "user_id",
      });
    }

    const sqlCheck =
      "SELECT user_id, active_flag, f_name FROM user_master WHERE user_id = ? AND delete_flag = 0";
    connection.query(sqlCheck, [user_id], async (err, result) => {
      if (err) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.internalServerError,
          error: err,
        });
      }

      if (result.length <= 0) {
        return response
          .status(404)
          .json({ success: false, msg: languageMessages.msgUserNotFound });
      }

      if (result[0].active_flag === 0) {
        return response.status(403).json({
          success: false,
          msg: languageMessages.accountdeactivated,
          active_status: 0,
        });
      }

      // get all categories start
      // const getCategory =
      //   "SELECT cm.category_id, cm.title FROM category_master as cm , musa_master as mm WHERE cm.delete_flag = 0 AND (cm.user_id = 1 OR cm.user_id = ?) and mm.category = cm.category_id and mm.user_id=? and mm.delete_flag=0";
      const getCategory =
      "SELECT category_id, title FROM category_master WHERE delete_flag = 0 AND (user_id = 1 OR user_id = ?) ";
      await connection.query(
        getCategory,
        [user_id],
        async (getCategoryErr, getCategoryResult) => {
          if (getCategoryErr) {
            return response.status(200).json({
              success: false,
              msg: languageMessages.internalServerError,
              error: getCategoryErr,
            });
          }

          if (getCategoryResult.length > 0) {
            let my_musa_arr = [];
            for (const data of getCategoryResult) {
              try {
                let getSubCategories = await getSubCategories1(
                  data.category_id,
                  user_id
                );
                let category_images = await getAllImagesOfCategories(
                  data.category_id,
                  user_id
                );
                let category_videos = await getAllVideosOfCategories(
                  data.category_id,
                  user_id
                );
                let category_audios = await getAllAudiosOfCategories(
                  data.category_id,
                  user_id
                );

                my_musa_arr.push({
                  category_id: data.category_id,
                  img_video: category_images,
                  img_video_new: category_videos,
                  category_audios: category_audios,
                  title: data.title,
                  getSubCategories:
                    getSubCategories.length > 0 ? getSubCategories : "NA",
                  status: false,
                });
              } catch (getSubCategoriesErr) {
                return response.status(200).json({
                  success: false,
                  msg: languageMessages.internalServerError,
                  error: getSubCategoriesErr,
                });
              }
            }
            my_musa_arr.sort((a, b) => {
              if (a.title === "Musa Life") {
                return -1;
              } else if (b.title === "Musa Life") {
                return 1;
              } else {
                return 0;
              }
            });
            return response.status(200).json({
              success: true,
              msg: languageMessages.msgDataFound,
              my_musa_arr: my_musa_arr,
            });
          } else {
            return response.status(200).json({
              success: true,
              msg: languageMessages.msgDataFound,
              my_musa_arr: "NA",
            });
          }
        }
      );
      // get all categories end
    });
  } catch (error) {
    return response.status(200).json({
      success: false,
      msg: languageMessages.internalServerError,
      error: error,
    });
  }
};

// my musa on category or subcategory wise end

async function getSubCategories1(category_id, user_id) {
  const getSubCategoriesQuery =
    "SELECT scm.sub_category_id, scm.title FROM sub_category_master as scm , musa_master as mm WHERE scm.delete_flag=0 and scm.category_id=? and scm.sub_category_id = mm.sub_category and mm.user_id=? and mm.delete_flag=0";
  return new Promise((resolve, reject) => {
    connection.query(
      getSubCategoriesQuery,
      [category_id,user_id],
      async (getcatErr, getCatResult) => {
        if (getcatErr) {
          return reject(getcatErr);
        }

        try {
          let musa_arr = await Promise.all(
            getCatResult.map(async (data) => {
              let subCategory = await getMusaSubCategoryWise(
                data.sub_category_id,
                user_id
              );
              let subCategory_images = await getAllImagesOfSubCategories(
                data.sub_category_id,
                user_id
              );
              let subCategory_videos = await getAllVideosOfSubCategories(
                data.sub_category_id,
                user_id
              );
              let subCategory_audios = await getAllAudiosOfSubCategories(
                data.sub_category_id,
                user_id
              );
              if (data.sub_category_id > 0) {
                return {
                  sub_category_id: data.sub_category_id,
                  title: data.title,
                  subCategory_image: subCategory.musa_count,
                  subCategory_audio: subCategory.musa_audio_count,
                  subCategory_video: subCategory.musa_video_count,
                  image: subCategory.image,
                  img_video:
                    subCategory_images.length > 0 ? subCategory_images : "NA",
                  img_video_new:
                    subCategory_videos.length > 0 ? subCategory_videos : "NA",
                  subCategory_audios:
                    subCategory_audios.length > 0 ? subCategory_audios : "NA",
                  status: false,
                };
              }
            })
          );
          resolve(musa_arr);
        } catch (err) {
          reject(err);
        }
      }
    );
  });
}

// get categories sub categories end

async function getMusaSubCategoryWise(sub_category_id, user_id) {
  const getMusaQuery =
    "SELECT musa_id FROM musa_master WHERE delete_flag=0 and sub_category=? and user_id=?";
  const getCountQueryImage =
    "SELECT count(musa_image_id) as count FROM musa_image_master WHERE delete_flag=0 and musa_id=? and image_audio_type = 1";
  const getCountQueryAudio =
    "SELECT count(musa_image_id) as count FROM musa_image_master WHERE delete_flag=0 and musa_id=? and image_audio_type = 2";
  const getCountQueryVideo =
    "SELECT count(musa_image_id) as count FROM musa_image_master WHERE delete_flag=0 and musa_id=? and image_audio_type = 3";

  const query = util.promisify(connection.query).bind(connection);

  try {
    const musaResult = await query(getMusaQuery, [sub_category_id, user_id]);
    let musa_count = 0;
    let musa_audio_count = 0;
    let musa_video_count = 0;

    for (const data of musaResult) {
      const imageResult = await query(getCountQueryImage, [data.musa_id]);
      const imageResultAudio = await query(getCountQueryAudio, [data.musa_id]);
      const imageResultVideo = await query(getCountQueryVideo, [data.musa_id]);
      musa_count += imageResult[0].count;
      musa_audio_count += imageResultAudio[0].count;
      musa_video_count += imageResultVideo[0].count;
    }

    let singleImage = await getSingleImageOnMusaSubcategories(
      sub_category_id,
      user_id
    );
    const musa_obj = {
      musa_count: musa_count,
      musa_audio_count: musa_audio_count,
      musa_video_count: musa_video_count,
      image: singleImage != null ? singleImage.image_audio : "NA",
    };
    return musa_obj;
  } catch (err) {
    throw err;
  }
}

async function getSingleImageOnMusaSubcategories(sub_category_id, user_id) {
  const getImage = `
        SELECT mim.image_audio 
        FROM musa_image_master as mim 
        INNER JOIN musa_master as mm ON mim.musa_id = mm.musa_id 
        WHERE mim.delete_flag = 0 AND mm.delete_flag = 0 
          AND mim.image_audio_type = 1 
          AND mm.sub_category = ? 
          AND mm.user_id = ?
    `;

  return new Promise((resolve, reject) => {
    connection.query(
      getImage,
      [sub_category_id, user_id],
      (getImageErr, getImageResult) => {
        if (getImageErr) {
          reject(getImageErr);
        } else {
          if (getImageResult.length > 0) {
            const ImageObj = {
              image_audio:
                "https://d38i924xuvvpvh.cloudfront.net/images/" +
                getImageResult[0].image_audio,
            };
            resolve(ImageObj);
          } else {
            resolve(null); // Or you can choose to reject with a custom message
          }
        }
      }
    );
  });
}

// get all images of subcategories wise start

async function getAllImagesOfSubCategories(sub_category_id, user_id) {
  const getImages =
    "SELECT mim.musa_image_id,mim.image_audio,mim.size_flag,mim.image_audio_type,mim.thumbnail_image,mim.duration FROM `musa_image_master` as mim INNER JOIN musa_master as mm ON mim.musa_id=mm.musa_id WHERE mim.delete_flag = 0 and mm.user_id=? and mm.sub_category=? and mim.image_audio_type=1 and mm.delete_flag=0";
  return new Promise((resolve, reject) => {
    connection.query(
      getImages,
      [user_id, sub_category_id],
      async (getImagesErr, result) => {
        if (getImagesErr) {
          reject(getImagesErr);
        } else {
          let videos = [];
          let images = [];
          let largeImages = [];

          const increments = [1, 1, 5];
          let current_value = 17;
          let normalPlacementIndexes = [3, 4, 5, 10, 11, 12, 13, 17];

          while (current_value < 1000) {
            for (let increment of increments) {
              current_value += increment;
              if (current_value > 1000) {
                break;
              }
              normalPlacementIndexes.push(current_value);
            }
          }

          console.log(normalPlacementIndexes);

          let videoPlacementIndexes = [
            1, 2, 8, 9, 15, 16, 22, 23, 29, 30, 36, 37, 43, 44, 50, 51, 57, 58,
            64, 65, 71, 72, 78, 79, 85, 86, 92, 93, 99, 100, 106, 107, 113, 114,
            120, 121, 127, 128, 134, 135, 141, 142, 148, 149, 155, 156, 162,
            163, 169, 170, 176, 177, 183, 184, 190, 191, 197, 198, 204, 205,
            211, 212, 218, 219, 225, 226, 232, 233, 239, 240, 246, 247, 253,
            254, 260, 261, 267, 268, 274, 275, 281, 282, 288, 289, 295, 296,
            302, 303, 309, 310,
          ];

          let largeImagePlacementIndexes = [0, 6, 7, 14];
          const increments1 = [6, 1];
          let current_value2 = 14;

          while (current_value2 < 1000) {
            for (let increments of increments1) {
              current_value2 += increments;
              if (current_value2 > 1000) {
                break;
              }
              largeImagePlacementIndexes.push(current_value2);
            }
          }

          console.log(largeImagePlacementIndexes);

          for (const row of result) {
            if (row.image_audio_type === 3) {
              // Video
              videos.push({
                musa_image_id: row.musa_image_id,
                size_flag: row.size_flag,
                size_flag_text: "0=normal,1=lengthy",
                image_audio_type: row.image_audio_type,
                image_audio_type_text: "1=image,3=video",
                image_audio:
                  "https://d38i924xuvvpvh.cloudfront.net/images/" +
                  row.image_audio,
                thumbnail_image:
                  "https://d38i924xuvvpvh.cloudfront.net/" +
                  row.thumbnail_image,
              });
            } else if (row.image_audio_type === 1) {
              // Image
              if (row.size_flag === 1) {
                largeImages.push({
                  musa_image_id: row.musa_image_id,
                  duration: row.duration,
                  size_flag: row.size_flag,
                  size_flag_text: "0=normal,1=lengthy",
                  image_audio_type: row.image_audio_type,
                  image_audio_type_text: "1=image,3=video",
                  image_audio:
                    "https://d38i924xuvvpvh.cloudfront.net/images/" +
                    row.image_audio,
                  thumbnail_image: row.thumbnail_image
                    ? "https://d38i924xuvvpvh.cloudfront.net/" +
                      row.thumbnail_image
                    : "NA",
                });
              } else {
                images.push({
                  musa_image_id: row.musa_image_id,
                  duration: row.duration,
                  size_flag: row.size_flag,
                  size_flag_text: "0=normal,1=lengthy",
                  image_audio_type: row.image_audio_type,
                  image_audio_type_text: "1=image,3=video",
                  image_audio:
                    "https://d38i924xuvvpvh.cloudfront.net/images/" +
                    row.image_audio,
                  thumbnail_image: row.thumbnail_image
                    ? "https://d38i924xuvvpvh.cloudfront.net/" +
                      row.thumbnail_image
                    : "NA",
                });
              }
            }
          }

          let maxIndex = Math.max(
            ...videoPlacementIndexes,
            ...largeImagePlacementIndexes,
            ...normalPlacementIndexes
          );
          let finalArr = new Array(maxIndex + 1).fill("NA");

          // Place videos at specific indexes
          videos.forEach((video, index) => {
            if (index < videoPlacementIndexes.length) {
              finalArr[videoPlacementIndexes[index]] = video;
            }
          });

          // Place large images at specific indexes
          largeImages.forEach((image, index) => {
            if (index < largeImagePlacementIndexes.length) {
              finalArr[largeImagePlacementIndexes[index]] = image;
            }
          });

          // Place other images in the remaining slots
          images.forEach((image, index) => {
            if (index < normalPlacementIndexes.length) {
              finalArr[normalPlacementIndexes[index]] = image;
            }
          });

          // Trim 'NA' from the end if no images or videos exist after the last defined index
          while (
            finalArr.length > 0 &&
            finalArr[finalArr.length - 1] === "NA"
          ) {
            finalArr.pop();
          }

          resolve(finalArr);
        }
      }
    );
  });
}

// get all images of subcategories wise end

// get all images of categories wise start

async function getAllImagesOfCategories(category_id, user_id) {
  const getImages =
    "SELECT mim.musa_image_id,mim.image_audio,mim.size_flag,mim.image_audio_type,mim.thumbnail_image,mim.duration FROM `musa_image_master` as mim INNER JOIN musa_master as mm ON mim.musa_id=mm.musa_id WHERE mim.delete_flag = 0 and mm.user_id=? and mm.category=? and mim.image_audio_type=1 and mm.delete_flag=0";
  return new Promise((resolve, reject) => {
    connection.query(
      getImages,
      [user_id, category_id],
      async (getImagesErr, result) => {
        if (getImagesErr) {
          reject(getImagesErr);
        } else {
          let videos = [];
          let images = [];
          let largeImages = [];

          const increments = [1, 1, 5];
          let current_value = 17;
          let normalPlacementIndexes = [3, 4, 5, 10, 11, 12, 13, 17];

          while (current_value < 1000) {
            for (let increment of increments) {
              current_value += increment;
              if (current_value > 1000) {
                break;
              }
              normalPlacementIndexes.push(current_value);
            }
          }

          console.log(normalPlacementIndexes);

          let videoPlacementIndexes = [
            1, 2, 8, 9, 15, 16, 22, 23, 29, 30, 36, 37, 43, 44, 50, 51, 57, 58,
            64, 65, 71, 72, 78, 79, 85, 86, 92, 93, 99, 100, 106, 107, 113, 114,
            120, 121, 127, 128, 134, 135, 141, 142, 148, 149, 155, 156, 162,
            163, 169, 170, 176, 177, 183, 184, 190, 191, 197, 198, 204, 205,
            211, 212, 218, 219, 225, 226, 232, 233, 239, 240, 246, 247, 253,
            254, 260, 261, 267, 268, 274, 275, 281, 282, 288, 289, 295, 296,
            302, 303, 309, 310, 316, 317, 323, 324, 330, 331, 337, 343, 344,
            350, 351, 357, 358, 364, 365, 371, 372, 378, 379, 385, 386, 392,
            393, 399, 400, 406, 407, 413, 414, 420, 421, 427, 428, 434, 435,
            441, 442, 448, 449, 455, 456, 462, 463, 469, 470, 476, 477, 483,
            484, 490, 491, 497, 498, 504, 505, 511, 512, 518, 519, 525, 526,
            532, 533, 539, 540, 546, 547, 553, 554, 560, 561, 567, 568, 574,
            575, 581, 582, 588, 589, 595, 596, 602, 603, 609, 610, 616, 617,
            623, 624, 630, 631, 637, 638, 644, 645, 651, 652, 658, 659, 665,
            666, 672, 673, 679, 680, 686, 687, 693, 694, 700, 701, 707, 708,
            714, 715, 721, 722, 728, 729, 735, 736, 742, 743, 749, 750, 756,
            757, 763, 764, 770, 771, 777, 778, 784, 785, 791, 792, 798, 799,
            805, 806, 812, 813, 819, 820, 826, 827, 833, 834, 840, 841, 847,
            848, 856, 857, 863, 864, 870, 871, 877, 878, 884, 885, 891, 892,
            898, 899, 905, 906, 912, 913, 919, 920, 926, 927, 933, 934, 940,
            941, 947, 948, 954, 955, 961, 962, 968, 969, 975, 976, 982, 983,
            989, 990, 996, 997, 1003, 1004, 1010, 1011, 1017,
          ];

          let largeImagePlacementIndexes = [0, 6, 7, 14];
          const increments1 = [6, 1];
          let current_value2 = 14;

          while (current_value2 < 1000) {
            for (let increments of increments1) {
              current_value2 += increments;
              if (current_value2 > 1000) {
                break;
              }
              largeImagePlacementIndexes.push(current_value2);
            }
          }

          console.log(largeImagePlacementIndexes);

          for (const row of result) {
            if (row.image_audio_type === 3) {
              // Video
              videos.push({
                musa_image_id: row.musa_image_id,
                duration: row.duration,
                size_flag: row.size_flag,
                size_flag_text: "0=normal,1=lengthy",
                image_audio_type: row.image_audio_type,
                image_audio_type_text: "1=image,3=video",
                image_audio:
                  "https://d38i924xuvvpvh.cloudfront.net/images/" +
                  row.image_audio,
                thumbnail_image:
                  "https://d38i924xuvvpvh.cloudfront.net/" +
                  row.thumbnail_image,
              });
            } else if (row.image_audio_type === 1) {
              // Image
              if (row.size_flag === 1) {
                largeImages.push({
                  musa_image_id: row.musa_image_id,
                  duration: row.duration,
                  size_flag: row.size_flag,
                  size_flag_text: "0=normal,1=lengthy",
                  image_audio_type: row.image_audio_type,
                  image_audio_type_text: "1=image,3=video",
                  image_audio:
                    "https://d38i924xuvvpvh.cloudfront.net/images/" +
                    row.image_audio,
                  thumbnail_image: row.thumbnail_image
                    ? "https://d38i924xuvvpvh.cloudfront.net/" +
                      row.thumbnail_image
                    : "NA",
                });
              } else {
                images.push({
                  musa_image_id: row.musa_image_id,
                  duration: row.duration,
                  size_flag: row.size_flag,
                  size_flag_text: "0=normal,1=lengthy",
                  image_audio_type: row.image_audio_type,
                  image_audio_type_text: "1=image,3=video",
                  image_audio:
                    "https://d38i924xuvvpvh.cloudfront.net/images/" +
                    row.image_audio,
                  thumbnail_image: row.thumbnail_image
                    ? "https://d38i924xuvvpvh.cloudfront.net/" +
                      row.thumbnail_image
                    : "NA",
                });
              }
            }
          }

          let maxIndex = Math.max(
            ...videoPlacementIndexes,
            ...largeImagePlacementIndexes,
            ...normalPlacementIndexes
          );
          let finalArr = new Array(maxIndex + 1).fill("NA");

          // Place videos at specific indexes
          videos.forEach((video, index) => {
            if (index < videoPlacementIndexes.length) {
              finalArr[videoPlacementIndexes[index]] = video;
            }
          });

          // Place large images at specific indexes
          largeImages.forEach((image, index) => {
            if (index < largeImagePlacementIndexes.length) {
              finalArr[largeImagePlacementIndexes[index]] = image;
            }
          });

          // Place other images in the remaining slots
          images.forEach((image, index) => {
            if (index < normalPlacementIndexes.length) {
              finalArr[normalPlacementIndexes[index]] = image;
            }
          });

          // Trim 'NA' from the end if no images or videos exist after the last defined index
          while (
            finalArr.length > 0 &&
            finalArr[finalArr.length - 1] === "NA"
          ) {
            finalArr.pop();
          }
          if (finalArr.length <= 0) {
            finalArr = "NA";
          }
          resolve(finalArr);
        }
      }
    );
  });
}

// get all images of subcategories wise end

// get all videos of subcategories start

async function getAllVideosOfSubCategories(sub_category_id, user_id) {
  const getImages =
    "SELECT mim.musa_image_id,mim.image_audio,mim.size_flag,mim.image_audio_type,mim.thumbnail_image,mim.duration FROM `musa_image_master` as mim INNER JOIN musa_master as mm ON mim.musa_id=mm.musa_id WHERE mim.delete_flag = 0 and mm.user_id=? and mm.sub_category=? and mim.image_audio_type=3 and mm.delete_flag=0";
  return new Promise((resolve, reject) => {
    connection.query(
      getImages,
      [user_id, sub_category_id],
      async (getImagesErr, result) => {
        if (getImagesErr) {
          reject(getImagesErr);
        } else {
          let videos = [];

          for (const row of result) {
            videos.push({
              musa_image_id: row.musa_image_id,
              duration: row.duration,
              size_flag: row.size_flag,
              size_flag_text: "0=normal,1=lengthy",
              image_audio_type: row.image_audio_type,
              image_audio_type_text: "1=image,3=video",
              image_audio:
                "https://d38i924xuvvpvh.cloudfront.net/images/" +
                row.image_audio,
              thumbnail_image:
                "https://d38i924xuvvpvh.cloudfront.net/" + row.thumbnail_image,
            });
          }

          resolve(videos);
        }
      }
    );
  });
}

// get all videos of subcategories end

// get all videos of categories start

async function getAllVideosOfCategories(category_id, user_id) {
  const getImages =
    "SELECT mim.musa_image_id,mim.image_audio,mim.size_flag,mim.image_audio_type,mim.thumbnail_image,mim.duration FROM `musa_image_master` as mim INNER JOIN musa_master as mm ON mim.musa_id=mm.musa_id WHERE mim.delete_flag = 0 and mm.user_id=? and mm.category=? and mim.image_audio_type=3 and mm.delete_flag=0";
  return new Promise((resolve, reject) => {
    connection.query(
      getImages,
      [user_id, category_id],
      async (getImagesErr, result) => {
        if (getImagesErr) {
          reject(getImagesErr);
        } else {
          let videos = [];

          for (const row of result) {
            videos.push({
              musa_image_id: row.musa_image_id,
              duration: row.duration,
              size_flag: row.size_flag,
              size_flag_text: "0=normal,1=lengthy",
              image_audio_type: row.image_audio_type,
              image_audio_type_text: "1=image,3=video",
              image_audio:
                "https://d38i924xuvvpvh.cloudfront.net/images/" +
                row.image_audio,
              thumbnail_image:
                "https://d38i924xuvvpvh.cloudfront.net/" + row.thumbnail_image,
            });
          }

          if (videos.length <= 0) {
            videos = "NA";
          }
          resolve(videos);
        }
      }
    );
  });
}

// get all videos of categories end

// get all audios start

async function getAllAudiosOfSubCategories(sub_category_id, user_id) {
  const getImages =
    "SELECT mim.musa_image_id,mim.image_audio,mim.size_flag,mim.image_audio_type,mim.thumbnail_image,mim.duration FROM `musa_image_master` as mim INNER JOIN musa_master as mm ON mim.musa_id=mm.musa_id WHERE mim.delete_flag = 0 and mm.user_id=? and mm.sub_category=? and mim.image_audio_type=2 and mm.delete_flag=0";
  return new Promise((resolve, reject) => {
    connection.query(
      getImages,
      [user_id, sub_category_id],
      async (getImagesErr, result) => {
        if (getImagesErr) {
          reject(getImagesErr);
        } else {
          let audios = [];

          for (const row of result) {
            audios.push({
              musa_image_id: row.musa_image_id,
              duration: row.duration,
              size_flag: row.size_flag,
              size_flag_text: "0=normal,1=lengthy",
              image_audio_type: row.image_audio_type,
              image_audio_type_text: "1=image,3=video",
              image_audio:
                "https://d38i924xuvvpvh.cloudfront.net/images/" +
                row.image_audio,
              thumbnail_image:
                "https://d38i924xuvvpvh.cloudfront.net/" + row.thumbnail_image,
            });
          }

          resolve(audios);
        }
      }
    );
  });
}

// get all audios end

// get all category audios start

async function getAllAudiosOfCategories(category_id, user_id) {
  const getImages =
    "SELECT mim.musa_image_id,mim.image_audio,mim.size_flag,mim.image_audio_type,mim.thumbnail_image,mim.duration FROM `musa_image_master` as mim INNER JOIN musa_master as mm ON mim.musa_id=mm.musa_id WHERE mim.delete_flag = 0 and mm.user_id=? and mm.category=? and mim.image_audio_type=2 and mm.delete_flag=0";
  return new Promise((resolve, reject) => {
    connection.query(
      getImages,
      [user_id, category_id],
      async (getImagesErr, result) => {
        if (getImagesErr) {
          reject(getImagesErr);
        } else {
          let audios = [];

          for (const row of result) {
            audios.push({
              musa_image_id: row.musa_image_id,
              duration: row.duration,
              size_flag: row.size_flag,
              size_flag_text: "0=normal,1=lengthy",
              image_audio_type: row.image_audio_type,
              image_audio_type_text: "1=image,3=video",
              image_audio:
                "https://d38i924xuvvpvh.cloudfront.net/images/" +
                row.image_audio,
              thumbnail_image:
                "https://d38i924xuvvpvh.cloudfront.net/" + row.thumbnail_image,
            });
          }

          if (audios.length <= 0) {
            audios = "NA";
          }
          resolve(audios);
        }
      }
    );
  });
}

// get all category audios end

// get all comments start

const getAllCommentsOnMusa = async (request, response) => {
  const { user_id, musa_id } = request.query;

  if (!user_id) {
    return response.status(400).json({
      success: false,
      msg: languageMessages.msg_empty_param,
      key: "user_id",
    });
  }

  if (!musa_id) {
    return response.status(400).json({
      success: false,
      msg: languageMessages.msg_empty_param,
      key: "musa_id",
    });
  }

  try {
    const sqlCheck =
      "SELECT user_id, active_flag, f_name FROM user_master WHERE user_id = ? AND delete_flag = 0";
    connection.query(sqlCheck, [user_id], async (err, result) => {
      if (err) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.internalServerError,
          error: err,
        });
      }

      if (result.length <= 0) {
        return response
          .status(404)
          .json({ success: false, msg: languageMessages.msgUserNotFound });
      }

      if (result[0].active_flag === 0) {
        return response.status(403).json({
          success: false,
          msg: languageMessages.accountdeactivated,
          active_status: 0,
        });
      }

      // Get musa creator details
      let userDetail = {};
      const detail =
        "SELECT mm.category, mm.sub_category, mm.user_id, mm.description, us.name, us.image FROM musa_master AS mm INNER JOIN user_master AS us ON mm.user_id = us.user_id WHERE mm.delete_flag = 0 AND mm.musa_id = ?";
      connection.query(detail, [musa_id], async (detailErr, detailResult) => {
        if (detailErr) {
          return response.status(200).json({
            success: false,
            msg: languageMessages.internalServerError,
            error: detailErr,
          });
        }

        if (detailResult.length > 0) {
          let category_detail = await getSingleAllCategoryData(
            detailResult[0].category
          );
          let sub_category_detail = await getSingleSubCategoryData(
            detailResult[0].sub_category,
            detailResult[0].category
          );

          userDetail = {
            user_id: detailResult[0].user_id,
            name: detailResult[0].name,
            image:
              detailResult[0].image != null
                ? "https://d38i924xuvvpvh.cloudfront.net/" +
                  detailResult[0].image
                : "NA",
            category_id: detailResult[0].category,
            category_name:
              category_detail != "NA" ? category_detail[0].title : "NA",
            subcategory_name:
              sub_category_detail != "NA" ? sub_category_detail[0].title : "NA",
            sub_category_id: detailResult[0].sub_category,
            description: detailResult[0].description,
          };
        }

        // Get all comments
        const getComment =
          "SELECT comment_id, user_id, musa_id, comment, comment_type, createtime, comment_text FROM comment_master WHERE musa_id = ? AND delete_flag = 0 order by comment_id desc";
        connection.query(
          getComment,
          [musa_id],
          async (getCommentErr, getCommentResult) => {
            if (getCommentErr) {
              return response.status(200).json({
                success: false,
                msg: languageMessages.internalServerError,
                error: getCommentErr,
              });
            }

            if (getCommentResult.length > 0) {
              let comment_arr = [];

              for (const row of getCommentResult) {
                let userData = await getUserData(row.user_id);
                const time = await timeAgo(row.createtime);
                comment_arr.push({
                  name: userData ? userData.name : "NA",
                  f_name: userData ? userData.f_name : "NA",
                  l_name: userData ? userData.l_name : "NA",
                  username: userData ? userData.username : "NA",
                  image: userData ? userData.image_new : "NA",
                  comment_id: row.comment_id,
                  user_id: row.user_id,
                  my_user_id : userDetail.user_id,
                  time: time,
                  musa_id: row.musa_id,
                  comment_type_text: row.comment_type === 1 ? "Text" : "Audio",
                  comment:
                    row.comment_type === 1
                      ? row.comment
                      : "https://d38i924xuvvpvh.cloudfront.net/images/" +
                        row.comment,
                  comment_type: row.comment_type,
                  type_label: row.comment_type === 0 ? "Typing" : "Audio",
                  comment_text: row.comment_text,
                  status: false,
                });
              }

              return response.status(200).json({
                success: true,
                msg: languageMessages.msgDataFound,
                comment_arr: comment_arr,
                userDetail: userDetail,
              });
            } else {
              return response.status(200).json({
                success: true,
                msg: languageMessages.msgDataFound,
                comment_arr: "NA",
                userDetail: userDetail,
              });
            }
          }
        );
      });
    });
  } catch (error) {
    return response.status(200).json({
      success: false,
      msg: languageMessages.internalServerError,
      error: error,
    });
  }
};

// get all comments end

// comments getting through image video id start

// const getAllCommentsOnMusaThroughImage = async (request, response) => {
//     const { user_id, musa_image_id } = request.query;

//     if (!user_id) {
//         return response.status(200).json({ success: false, msg: languageMessages.msg_empty_param, key: "user_id" });
//     }

//     if (!musa_image_id) {
//         return response.status(200).json({ success: false, msg: languageMessages.msg_empty_param, key: "musa_image_id" });
//     }

//     try {
//         const sqlCheck = "SELECT user_id, active_flag, f_name FROM user_master WHERE user_id = ? AND delete_flag = 0";
//         connection.query(sqlCheck, [user_id], async (err, result) => {
//             if (err) {
//                 return response.status(200).json({ success: false, msg: languageMessages.internalServerError, error: err });
//             }

//             if (result.length <= 0) {
//                 return response.status(200).json({ success: false, msg: languageMessages.msgUserNotFound });
//             }

//             if (result[0].active_flag === 0) {
//                 return response.status(200).json({ success: false, msg: languageMessages.accountdeactivated, active_status: 0 });
//             }

//             let musa_id = 0;

//             await connection.query(
//                 "SELECT mim.musa_id FROM musa_image_master as mim,musa_master as mm WHERE mm.delete_flag = 0 and mim.delete_flag=0 AND musa_image_id = ? and mim.musa_id=mm.musa_id",
//                 [musa_image_id],
//                 async(imageErr, ImageResult) => {
//                     if (imageErr) {
//                         return response.status(200).json({ success: false, msg: languageMessages.internalServerError, error: imageErr });
//                     } else if (ImageResult.length > 0) {
//                         musa_id = ImageResult[0].musa_id;
//                     }else if (ImageResult.length <= 0) {
//                         return response.status(200).json({ success: false, msg: languageMessages.musaNotFound});
//                     }else{
//                         return response.status(200).json({ success: false, msg: languageMessages.musaNotFound});
//                     }

//                     // Get musa creator details
//                     let userDetail = {};
//                     const detail = `
//                         SELECT mm.category, mm.sub_category, mm.user_id, mm.description, us.f_name, us.l_name, us.image
//                         FROM musa_master AS mm
//                         INNER JOIN user_master AS us ON mm.user_id = us.user_id
//                         WHERE mm.delete_flag = 0 AND mm.musa_id = ?
//                     `;

//                     connection.query(detail, [musa_id], async (detailErr, detailResult) => {
//                         if (detailErr) {
//                             return response.status(200).json({ success: false, msg: languageMessages.internalServerError, error: detailErr });
//                         }

//                         if (detailResult.length > 0) {
//                             // const category_detail = await getSingleAllCategoryData(detailResult[0].category);
//                             // const sub_category_detail = await getSingleSubCategoryData(detailResult[0].sub_category, detailResult[0].category);

//                             userDetail = {
//                                 user_id: detailResult[0].user_id,
//                                 name: detailResult[0].f_name,
//                                 l_name: detailResult[0].l_name,
//                                 image: detailResult[0].image != null ? "https://d38i924xuvvpvh.cloudfront.net/" + detailResult[0].image : 'NA',
//                                 category_id: detailResult[0].category,
//                                 // category_name: category_detail != 'NA' ? category_detail[0].title : 'NA',
//                                 // subcategory_name: sub_category_detail != 'NA' ? sub_category_detail[0].title : 'NA',
//                                 sub_category_id: detailResult[0].sub_category,
//                                 description: detailResult[0].description,
//                             };

//                         }

//                         // Get all comments
//                         const getComment = `
//                             SELECT comment_id, user_id, musa_id, comment, comment_type, createtime, comment_text
//                             FROM comment_master
//                             WHERE musa_id = ? AND delete_flag = 0 ORDER BY comment_id DESC
//                         `;
//                         connection.query(getComment, [musa_id], async (getCommentErr, getCommentResult) => {
//                             if (getCommentErr) {
//                                 return response.status(200).json({ success: false, msg: languageMessages.internalServerError, error: getCommentErr });
//                             }

//                             const comment_arr = [];

//                             if (getCommentResult.length > 0) {
//                                 for (const row of getCommentResult) {
//                                     const userData = await getUserData(row.user_id);
//                                     const time = await timeAgo(row.createtime);
//                                     comment_arr.push({
//                                         name: userData ? userData.name : "NA",
//                                         f_name: userData ? userData.f_name : "NA",
//                                         l_name: userData ? userData.l_name : "NA",
//                                         username: userData ? userData.username : "NA",
//                                         image: userData ? userData.image_new : "NA",
//                                         comment_id: row.comment_id,
//                                         user_id: row.user_id,
//                                         time: time,
//                                         musa_id: row.musa_id,
//                                         comment_type_text: row.comment_type === 1 ? 'Text' : 'Audio',
//                                         comment: row.comment_type === 1 ? row.comment : "https://d38i924xuvvpvh.cloudfront.net/images/" + row.comment,
//                                         comment_type: row.comment_type,
//                                         type_label: row.comment_type === 0 ? 'Typing' : 'Audio',
//                                         comment_text: row.comment_text,
//                                         status: false,
//                                     });
//                                 }
//                             }

//                             return response.status(200).json({ success: true, msg: languageMessages.msgDataFound, comment_arr: comment_arr.length > 0 ? comment_arr : 'NA', userDetail: userDetail });
//                         });
//                     });
//                 }
//             );
//         });
//     } catch (error) {
//         return response.status(200).json({ success: false, msg: languageMessages.internalServerError, error: error });
//     }
// };



async function userDetailNew(musa_id, type) {
  let detail;

  if (type != 1) {
    detail = `
    SELECT mm.category, mm.sub_category, mm.user_id, mm.description, us.f_name, us.l_name, us.image
    FROM musa_master AS mm
    INNER JOIN user_master AS us ON mm.user_id = us.user_id
    WHERE mm.delete_flag = 0 AND mm.musa_id = ?
    `;
  } else {
    detail = `
    SELECT user_id, f_name, l_name, image
    FROM user_master 
    WHERE delete_flag = 0 AND user_id = ?
    `;
  }

  return new Promise(async(resolve, reject) => {
    connection.query(detail, [musa_id],async(err, result) => {
      if (err) {
        return reject(0);
      }

      let category_detail = 'NA';
      let sub_category_detail = 'NA';
      if(type!=1){
          category_detail = await getSingleAllCategoryData(result[0].category);
          sub_category_detail = await getSingleSubCategoryData(result[0].sub_category, result[0].category);
      }

      let userDetail = {};
      if (result.length > 0) {
        userDetail = {
          user_id: result[0].user_id,
          name: result[0].f_name,
          l_name: result[0].l_name,
          image:
            result[0].image != null
              ? "https://d38i924xuvvpvh.cloudfront.net/" + result[0].image
              : "NA",
          category_id: type !=1 ? result[0].category : "NA",
            category_name: category_detail != 'NA' && type !=1 ? category_detail[0].title : 'NA',
           subcategory_name: sub_category_detail != 'NA' && type !=1 ? sub_category_detail[0].title : 'NA',
          sub_category_id: type !=1 ? result[0].sub_category : "NA",
          description: type !=1 ? result[0].description : "NA",
        };
      }

      resolve(userDetail);
    });
  });
}


const getAllCommentsOnMusaThroughImage = async (request, response) => {
  const { user_id, musa_image_id, type } = request.query;
  try {
    if (!user_id) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "user_id",
      });
    }
    if (!musa_image_id) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "musa_image_id",
      });
    }
    let image_gallery_id = musa_image_id;
    // Check if user exists
    const sqlCheckUser =
      "SELECT user_id, active_flag FROM user_master WHERE user_id = ? AND delete_flag = 0";
    connection.query(sqlCheckUser, [user_id], async (err, result) => {
      if (err) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.internalServerError,
          error: err,
        });
      }
      if (result.length <= 0) {
        return response
          .status(200)
          .json({ success: false, msg: languageMessages.msgDataNotFound });
      }
      if (result[0].active_flag === 0) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.accountdeactivated,
          active_status: 0,
        });
      }
    });

    // check images exists or not start
    var musa_id = 0;
    // if(type != 1){
      let detailNew = 'SELECT mim.musa_id FROM musa_image_master as mim,musa_master as mm WHERE mm.delete_flag = 0 and mim.delete_flag=0 AND musa_image_id = ? and mim.musa_id=mm.musa_id';
      await connection.query(detailNew,[image_gallery_id],async(err2,result21)=>{
        if(err2){
          return response.status(200).json({
            success: false,
            msg: languageMessages.internalServerError,
            err2: err2,
          });
        }
       
        
        if(result21.length > 0){
          musa_id = result21[0].musa_id;
         
        }
     
   
    var userDetail = 'NA';
        // let user_id_new = CommentedData[0].user_id;
let unique_id = type==0 ? musa_id : user_id;

         userDetail = await userDetailNew(unique_id,type)
         
    // get all comments start
    // 0=musa image,1=gallery image
    let sql;

    if (type == 0) {
      sql =
        "SELECT mm.musa_id,icm.image_gallery_id,icm.image_comment_id,icm.user_id,icm.comment,icm.createtime,um.f_name,um.l_name,um.image,icm.comment_type,um.username,um.name FROM image_comment_master as icm , user_master as um , musa_image_master as mim , musa_master as mm WHERE mm.musa_id=mim.musa_id and mm.delete_flag=0 and mim.musa_image_id = icm.image_gallery_id and mim.delete_flag=0 and icm.delete_flag=0 and icm.image_gallery_id=? and icm.type=0 and icm.user_id=um.user_id and um.delete_flag=0 order by icm.image_comment_id desc";
    } else if (type == 1) {
     
      sql =
        "SELECT icm.image_gallery_id,icm.image_comment_id,icm.user_id,icm.comment,icm.createtime,um.f_name,um.l_name,um.image,icm.comment_type,um.username,um.name FROM image_comment_master as icm , user_master as um , gallery_master as gm WHERE gm.gallery_id=icm.image_gallery_id and gm.delete_flag=0 and icm.delete_flag=0 and icm.image_gallery_id=? and icm.type=1 and icm.user_id=um.user_id and um.delete_flag=0 order by icm.image_comment_id desc";
    }

    let val = [image_gallery_id];

    await connection.query(sql, val, async (err1, CommentedData) => {
      if (err1) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.internalServerError,
          err1: err1,
        });
      }
      let comment_arr = [];
     
      if (CommentedData.length > 0) {
        
// if(type==0){
//         userDetail = await userDetailNew(CommentedData[0].musa_id,type)         
// }

        comment_arr = await Promise.all(
          CommentedData.map(async (data) => {
            return {
              musa_id: data.image_gallery_id,
              comment_id: data.image_comment_id,
              user_id: data.user_id,
              createtime: data.createtime,
              time: await timeAgo(data.createtime),
              username: data.username != null ? data.username : "NA",
              name: data.name != null ? data.name : "NA",
              f_name: data.f_name,
              l_name: data.l_name,
              comment_type: data.comment_type,
              type_label: "0 for typing type, 1 for audio",
              image: data.image != null ? data.image : "NA",
              comment_type_text: data.comment_type != 2 ? "Text" : "Audio",
              comment:
                data.comment_type != 2
                  ? data.comment
                  : "https://d38i924xuvvpvh.cloudfront.net/images/" +
                    data.comment,
              type_label: data.comment_type === 0 ? "Typing" : "Audio",
              status: false,
            };
          })
        );
      }

      comment_arr = comment_arr.length <= 0 ? "NA" : comment_arr;

      return response.status(200).json({
        success: true,
        msg: languageMessages.msgDataFound,
        comment_arr: comment_arr,
        userDetail: userDetail,
      });
    });
   
  })
// }
    // get all comments end
  } catch (error) {
    return response.status(200).json({
      success: false,
      msg: languageMessages.internalServerError,
      error,
    });
  }
};

// for user details start

async function userDetailGetForMusaComment(musa_id) {
  const query = `
                        SELECT mm.category, mm.sub_category, mm.user_id, mm.description, us.f_name,us.l_name, us.image
                        FROM musa_master AS mm
                        INNER JOIN user_master AS us ON mm.user_id = us.user_id
                        WHERE mm.delete_flag = 0 AND mm.musa_id = ?
                    `;
  return new Promise(async (resolve, reject) => {
    await connection.query(
      detail,
      [musa_id],
      async (detailErr, detailResult) => {
        if (detailErr) {
          return response.status(200).json({
            success: false,
            msg: languageMessages.internalServerError,
            error: detailErr,
          });
        }

        if (detailResult.length > 0) {
          const category_detail = await getSingleAllCategoryData(
            detailResult[0].category
          );
          const sub_category_detail = await getSingleSubCategoryData(
            detailResult[0].sub_category,
            detailResult[0].category
          );

          userDetail = {
            user_id: detailResult[0].user_id,
            name: detailResult[0].f_name,
            l_name: detailResult[0].l_name,
            image:
              detailResult[0].image != null
                ? "https://d38i924xuvvpvh.cloudfront.net/" +
                  detailResult[0].image
                : "NA",
            category_id: detailResult[0].category,
            category_name:
              category_detail != "NA" ? category_detail[0].title : "NA",
            subcategory_name:
              sub_category_detail != "NA" ? sub_category_detail[0].title : "NA",
            sub_category_id: detailResult[0].sub_category,
            description: detailResult[0].description,
          };
          resolve(userDetail);
        } else {
          resolve("NA");
        }
      }
    );
  });
}

// for user details end

// comments getting through image video id end

async function getMusavideoImageSingleArr(musa_id, user_id) {
  const sqlSelect =
    "SELECT `musa_image_id`, `image_audio`, `thumbnail_image`, `musa_id`, `user_id`, `type`, `image_audio_type`, `size_flag`,`duration` FROM `musa_image_master` WHERE musa_id = ? and delete_flag=0";
  return new Promise((resolve, reject) => {
    connection.query(sqlSelect, [musa_id], (err, result) => {
      if (err) {
        reject(err);
      } else {
        let videos = [];
        let images = [];
        let audios = [];

        // Separate videos, images, and large images based on type and size_flag
        for (const row of result) {
          if (row.image_audio_type == 3) {
            // Video
            videos.push({
              musa_image_id: row.musa_image_id,
              duration: row.duration,
              size_flag: row.size_flag,
              size_flag_text: "0=normal,1=lengthy",
              image_audio_type: row.image_audio_type,
              image_audio_type_text: "1=image,3=video",
              image_audio: row.image_audio,
              thumbnail_image_audio: row.thumbnail_image,
              path:
                "https://d38i924xuvvpvh.cloudfront.net/images/" +
                row.image_audio,
              thumbnail_image:
                "https://d38i924xuvvpvh.cloudfront.net/" + row.thumbnail_image,
            });
          } else if (row.image_audio_type == 1) {
            // Image
            images.push({
              musa_image_id: row.musa_image_id,
              duration: row.duration,
              size_flag: row.size_flag,
              size_flag_text: "0=normal,1=lengthy",
              image_audio_type: row.image_audio_type,
              image_audio_type_text: "1=image,3=video",
              image_audio: row.image_audio,
              thumbnail_image_audio: row.thumbnail_image,
              path:
                "https://d38i924xuvvpvh.cloudfront.net/images/" +
                row.image_audio,
              thumbnail_image: row.thumbnail_image
                ? "https://d38i924xuvvpvh.cloudfront.net/" + row.thumbnail_image
                : "NA",
            });
          } else if (row.image_audio_type == 2) {
            // Image
            audios.push({
              musa_image_id: row.musa_image_id,
              duration: row.duration,
              size_flag: row.size_flag,
              size_flag_text: "0=normal,1=lengthy",
              image_audio_type: row.image_audio_type,
              image_audio_type_text: "1=image,3=video",
              image_audio: row.image_audio,
              thumbnail_image_audio: row.thumbnail_image,
              path:
                "https://d38i924xuvvpvh.cloudfront.net/images/" +
                row.image_audio,
              thumbnail_image: row.thumbnail_image
                ? "https://d38i924xuvvpvh.cloudfront.net/" + row.thumbnail_image
                : "NA",
            });
          }
        }

        // Initialize final array and push images first, then videos
        let finalArr = [];
        if (images.length > 0) {
          finalArr.push(...images);
        }
        if (videos.length > 0) {
          finalArr.push(...videos);
        }
        if (audios.length > 0) {
          finalArr.push(...audios);
        }

        if (finalArr.length <= 0) {
          finalArr = "NA";
        }

        resolve(finalArr);
      }
    });
  });
}

// get display request status start

async function displayStatus(musa_id, user_id) {
  const display =
    "SELECT status FROM display_request_master WHERE delete_flag=0 and musa_id=? and user_id=? order by request_id desc";
  return new Promise((resolve, reject) => {
    connection.query(display, [musa_id, user_id], async (err, result) => {
      if (err) {
        reject(err);
      } else {
        let newStatus = 0;
        if (result.length > 0) {
          if (result[0].status == 0) {
            newStatus = 1;
          } else if (result[0].status == 1) {
            newStatus = 2;
          }
          resolve(newStatus);
        }
        resolve(newStatus);
      }
    });
  });
}

// get display request status end

async function getMusavideoImage(musa_id, user_id) {
  const sqlSelect =
    "SELECT `musa_image_id`, `image_audio`, `thumbnail_image`, `musa_id`, `user_id`, `type`, `image_audio_type`, `size_flag`,`duration` FROM `musa_image_master` WHERE musa_id = ? AND user_id = ? and image_audio_type = 1";
  return new Promise((resolve, reject) => {
    connection.query(sqlSelect, [musa_id, user_id], (err, result) => {
      if (err) {
        reject(err);
      } else {
        let videos = [];
        let images = [];
        let largeImages = [];

        const increments = [1, 1, 5];
        let current_value = 17;
        let normalPlacementIndexes = [3, 4, 5, 10, 11, 12, 13, 17];

        while (current_value < 1000) {
          for (let increment of increments) {
            current_value += increment;
            if (current_value > 1000) {
              break;
            }
            normalPlacementIndexes.push(current_value);
          }
        }

        console.log(normalPlacementIndexes);

        let videoPlacementIndexes = [
          1, 2, 8, 9, 15, 16, 22, 23, 29, 30, 36, 37, 43, 44, 50, 51, 57, 58,
          64, 65, 71, 72, 78, 79, 85, 86, 92, 93, 99, 100, 106, 107, 113, 114,
          120, 121, 127, 128, 134, 135, 141, 142, 148, 149, 155, 156, 162, 163,
          169, 170, 176, 177, 183, 184, 190, 191, 197, 198, 204, 205, 211, 212,
          218, 219, 225, 226, 232, 233, 239, 240, 246, 247, 253, 254, 260, 261,
          267, 268, 274, 275, 281, 282, 288, 289, 295, 296, 302, 303, 309, 310,
          316, 317, 323, 324, 330, 331, 337, 343, 344, 350, 351, 357, 358, 364,
          365, 371, 372, 378, 379, 385, 386, 392, 393, 399, 400, 406, 407, 413,
        ];
        let largeImagePlacementIndexes = [0, 6, 7, 14];
        const increments1 = [6, 1];
        let current_value2 = 14;

        while (current_value2 < 1000) {
          for (let increments of increments1) {
            current_value2 += increments;
            if (current_value2 > 1000) {
              break;
            }
            largeImagePlacementIndexes.push(current_value2);
          }
        }

        console.log(largeImagePlacementIndexes);

        // Separate videos, images, and large images based on type and size_flag
        for (const row of result) {
          if (row.image_audio_type === 3) {
            // Video
            videos.push({
              musa_image_id: row.musa_image_id,
              duration: row.duration,
              size_flag: row.size_flag,
              size_flag_text: "0=normal,1=lengthy",
              image_audio_type: row.image_audio_type,
              image_audio_type_text: "1=image,3=video",
              image_audio:
                "https://d38i924xuvvpvh.cloudfront.net/images/" +
                row.image_audio,
              thumbnail_image:
                "https://d38i924xuvvpvh.cloudfront.net/" + row.thumbnail_image,
            });
          } else if (row.image_audio_type === 1) {
            // Image
            if (row.size_flag === 1) {
              largeImages.push({
                musa_image_id: row.musa_image_id,
                duration: row.duration,
                size_flag: row.size_flag,
                size_flag_text: "0=normal,1=lengthy",
                image_audio_type: row.image_audio_type,
                image_audio_type_text: "1=image,3=video",
                image_audio:
                  "https://d38i924xuvvpvh.cloudfront.net/images/" +
                  row.image_audio,
                thumbnail_image: row.thumbnail_image
                  ? "https://d38i924xuvvpvh.cloudfront.net/" +
                    row.thumbnail_image
                  : "NA",
              });
            } else {
              images.push({
                musa_image_id: row.musa_image_id,
                duration: row.duration,
                size_flag: row.size_flag,
                size_flag_text: "0=normal,1=lengthy",
                image_audio_type: row.image_audio_type,
                image_audio_type_text: "1=image,3=video",
                image_audio:
                  "https://d38i924xuvvpvh.cloudfront.net/images/" +
                  row.image_audio,
                thumbnail_image: row.thumbnail_image
                  ? "https://d38i924xuvvpvh.cloudfront.net/" +
                    row.thumbnail_image
                  : "NA",
              });
            }
          }
        }

        // Initialize finalArr with 'NA' for all specified indexes
        let maxIndex = Math.max(
          ...videoPlacementIndexes,
          ...largeImagePlacementIndexes,
          ...normalPlacementIndexes
        );
        let finalArr = new Array(maxIndex + 1).fill("NA");

        // Place videos at specific indexes
        videos.forEach((video, index) => {
          if (index < videoPlacementIndexes.length) {
            finalArr[videoPlacementIndexes[index]] = video;
          }
        });

        // Place large images at specific indexes
        largeImages.forEach((image, index) => {
          if (index < largeImagePlacementIndexes.length) {
            finalArr[largeImagePlacementIndexes[index]] = image;
          }
        });

        // Place other images in the remaining slots
        images.forEach((image, index) => {
          if (index < normalPlacementIndexes.length) {
            finalArr[normalPlacementIndexes[index]] = image;
          }
        });

        // Trim 'NA' from the end if no images or videos exist after the last defined index
        while (finalArr.length > 0 && finalArr[finalArr.length - 1] === "NA") {
          finalArr.pop();
        }

        resolve(finalArr);
      }
    });
  });
}

async function getMusavideoNew(musa_id, user_id) {
  const sqlSelect =
    "SELECT `musa_image_id`, `image_audio`, `thumbnail_image`, `musa_id`, `user_id`, `type`, `image_audio_type`, `size_flag`,`duration` FROM `musa_image_master` WHERE musa_id = ? and image_audio_type = 3";
  return new Promise((resolve, reject) => {
    connection.query(sqlSelect, [musa_id], (err, result) => {
      if (err) {
        reject(err);
      } else {
        let videos = [];

        for (const row of result) {
          videos.push({
            musa_image_id: row.musa_image_id,
            duration: row.duration,
            size_flag: row.size_flag,
            size_flag_text: "0=normal,1=lengthy",
            image_audio_type: row.image_audio_type,
            image_audio_type_text: "1=image,3=video",
            image_audio:
              "https://d38i924xuvvpvh.cloudfront.net/images/" + row.image_audio,
            thumbnail_image:
              "https://d38i924xuvvpvh.cloudfront.net/" + row.thumbnail_image,
          });
        }

        resolve(videos);
      }
    });
  });
}

// getting user_id of people i contributed them start

async function getContributedIds(user_id) {
  const sql =
    "SELECT other_user_id FROM contributor_follow_master WHERE delete_flag=0 and user_id=?";
  return new Promise((resolve, reject) => {
    connection.query(sql, [user_id], (err, result) => {
      if (err) {
        reject(err);
      } else {
        let ids_arr = [];
        if (result.length > 0) {
          ids_arr = result.map((data) => data.other_user_id);

          let commaSeparatedIds = ids_arr.join(",");
          resolve(commaSeparatedIds);
        } else {
          resolve(0);
        }
      }
    });
  });
}

// getting user_id of people i contributed them end

// category search start

async function getCategorySearchId(category_text) {
  let searchTextLIke = "%" + category_text + "%";
  const sql =
    "SELECT category_id FROM category_master WHERE delete_flag=0 and title LIKE ?";
  return new Promise((resolve, reject) => {
    connection.query(sql, [searchTextLIke], (err, result) => {
      if (err) {
        reject(err);
      } else {
        let ids_arr = [];
        if (result.length > 0) {
          ids_arr = result.map((data) => data.category_id);

          let commaSeparatedIds = ids_arr.join(",");
          resolve(commaSeparatedIds);
        } else {
          resolve(0);
        }
      }
    });
  });
}

// category search end

// increase share app count start

const increaseShareCount = async (request, response) => {
  const { musa_id } = request.query;

  try {
    if (!musa_id) {
      return response.status(400).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "musa_id",
      });
    }

    var select =
      "SELECT share_count FROM musa_master WHERE delete_flag=0 and musa_id=?";
    var val = [musa_id];
    connection.query(select, val, async (err, result) => {
      if (err) {
        return response.status(200).json({
          success: false,
          msg: languageMessages.internalServerError,
          err: err,
        });
      }
      if (result.length > 0) {
        let share_count = result[0].share_count + 1;

        var sql =
          "UPDATE musa_master SET share_count=? WHERE delete_flag=0 and musa_id=?";
        var sqlVal = [share_count, musa_id];
        connection.query(sql, sqlVal, async (sqlErr, sqlResult) => {
          if (sqlErr) {
            return response.status(200).json({
              success: false,
              msg: languageMessages.internalServerError,
              sqlErr: sqlErr,
            });
          } else {
            return response.status(200).json({
              success: true,
              msg: languageMessages.musaSharedSuccess,
              share_count: share_count,
            });
          }
        });
      } else {
        return response
          .status(200)
          .json({ success: false, msg: languageMessages.musaNotFound });
      }
    });
  } catch (error) {
    return response.status(200).json({
      success: false,
      msg: languageMessages.internalServerError,
      error: error,
    });
  }
};

// increase share app count end

// share count start

async function getShareCount(musa_id, response) {
  try {
    const sql =
      "SELECT share_count FROM musa_master WHERE delete_flag=0 and musa_id=?";
    let result = await new Promise((resolve, reject) => {
      connection.query(sql, [musa_id], async (err, result1) => {
        if (err) {
          reject(err);
        } else {
          resolve(result1);
        }
      });
    });
    let share_count = result[0].share_count;
    return share_count;
  } catch (error) {
    return response.status(200).json({
      success: false,
      msg: languageMessages.internalServerError,
      error: error,
    });
  }
}

// share count end

// get single musa details start

const getSingleMusaDetailsByMusaId = async (request, response) => {
  const { user_id, musa_id, move, musa_status, save_status } = request.query;
  try {
    if (!user_id) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "user_id",
      });
    }
    if (!musa_id) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.msg_empty_param,
        key: "musa_id",
      });
    }

    // check user start
    const sqlcheak =
      "SELECT user_id,active_flag FROM user_master WHERE user_id = ? AND delete_flag = 0";
    const userResult = await new Promise((resolve, reject) => {
      connection.query(sqlcheak, [user_id], (err, result) => {
        if (err) reject(err);
        else resolve(result);
      });
    });

    if (userResult.length <= 0) {
      return response
        .status(200)
        .json({ success: false, msg: languageMessages.msgUserNotFound });
    }
    if (userResult[0].active_flag === 0) {
      return response.status(200).json({
        success: false,
        msg: languageMessages.accountdeactivated,
        active_status: 0,
      });
    }
    // check user end

    let sqlSelect;
    let val = [];

    //             if(save_status == 0){

    // sqlSelect = `SELECT musa_id, category, sub_category, user_id, description, type, title, createtime, updatetime FROM musa_master WHERE delete_flag = 0 and type = 0`;

    // if (musa_status == 1) {
    //     sqlSelect += ` AND user_id = ?`;
    //     val.push(user_id);
    // }

    // if (move == 1) {
    //     sqlSelect += ` AND musa_id < ?`;
    //     val.push(musa_id);
    // }
    // if (move == 2) {
    //     sqlSelect += ` AND musa_id > ? order by musa_id desc`;
    //     val.push(musa_id);
    // }

    // sqlSelect += ` LIMIT 1`;

    //             }else{

    //                  sqlSelect = `
    // SELECT sm.save_musa_id, sm.musa_id, sm.type AS sm_type, sm.user_id AS sm_user_id, sm.delete_flag AS sm_delete_flag,
    //        sm.createtime AS sm_createtime, sm.updatetime AS sm_updatetime, sm.mysqltime AS sm_mysqltime,
    //        mm.category, mm.sub_category, mm.description, mm.type AS mm_type, mm.title, mm.createtime AS mm_createtime, mm.updatetime AS mm_updatetime
    // FROM saved_musaa sm
    // JOIN (
    //     SELECT musa_id, category, sub_category, user_id, description, type, title, createtime, updatetime
    //     FROM musa_master
    //     WHERE delete_flag = 0 and type=0
    // `;

    // if (musa_status == 1) {
    //     sqlSelect += ` AND user_id = ?`;
    //     val.push(user_id);
    // }

    // if (move == 1) {
    //     sqlSelect += ` AND musa_id < ?`;
    //     val.push(musa_id);
    // } else if (move == 2) {
    //     sqlSelect += ` AND musa_id > ? ORDER BY musa_id DESC`;
    //     val.push(musa_id);
    // }

    // sqlSelect += `) mm ON sm.musa_id = mm.musa_id
    // WHERE sm.user_id = ? AND sm.musa_id = ? and sm.delete_flag=0
    // LIMIT 1`;

    // val.push(user_id, musa_id);

    //             }

    // if musa status is 1 then come all musas of self start
    if (musa_status == 1) {
      sqlSelect = `SELECT musa_id, category, sub_category, user_id, description, type, title, createtime, updatetime FROM musa_master WHERE delete_flag = 0 and type = 0 and user_id=?`;
      val.push(user_id);
      if (move == 1) {
        // sqlSelect += ` AND musa_id < ?`;
        sqlSelect += ` AND musa_id < ? order by musa_id desc`;
        val.push(musa_id);
      }
      if (move == 2) {
        sqlSelect += ` AND musa_id > ? order by musa_id asc`;
        val.push(musa_id);
      }

      sqlSelect += ` LIMIT 1`;
    } else {
      if (save_status == 1) {
        sqlSelect = `SELECT mm.musa_id, mm.category, mm.sub_category, mm.user_id, mm.description, mm.type, mm.title, mm.createtime, mm.updatetime FROM musa_master as mm , saved_musaa as sm , display_request_master as drm WHERE mm.delete_flag = 0 and mm.type = 0 and sm.user_id=${user_id} and sm.musa_id=mm.musa_id and ((mm.user_id != ${user_id} and drm.musa_id=mm.musa_id and drm.status=1 and drm.user_id = ${user_id}) OR (mm.user_id=${user_id}) )`;

        if (move == 1) {
          // sqlSelect += ` AND mm.musa_id < ?`;
          sqlSelect += ` AND mm.musa_id < ? order by mm.musa_id desc`;
          val.push(musa_id);
        }
        if (move == 2) {
          sqlSelect += ` AND mm.musa_id > ? order by mm.musa_id asc`;
          val.push(musa_id);
        }

        sqlSelect += ` LIMIT 1`;
      } else {
        sqlSelect = `SELECT mm.musa_id, mm.category, mm.sub_category, mm.user_id, mm.description, mm.type, mm.title, mm.createtime, mm.updatetime FROM musa_master as mm , display_request_master as drm WHERE mm.delete_flag = 0 and mm.type = 0 and ((mm.user_id != ${user_id} and drm.musa_id=mm.musa_id and drm.status=1 and drm.user_id = ${user_id}) OR (mm.user_id = ${user_id}))`;

        if (move == 1) {
          // sqlSelect += ` AND mm.musa_id < ?`;
          sqlSelect += ` AND mm.musa_id < ? order by mm.musa_id desc`;
          val.push(musa_id);
        }
        if (move == 2) {
          sqlSelect += ` AND mm.musa_id > ? order by mm.musa_id asc`;
          val.push(musa_id);
        }

        sqlSelect += ` LIMIT 1`;
      }
    }

    // if musa status is 1 then come all musas of self start

    var result = await new Promise(async (resolve, reject) => {
      await connection.query(sqlSelect, val, async (err, result) => {
        if (err) reject(err);
        else resolve(result);
      });
    });

    if (result.length > 0) {
      let row = result[0];
      const userData = await getUser(row.user_id);
      const image_arr = await getMusaImage(row.musa_id, row.user_id);
      const musa_video = await getMusavideo(row.musa_id, row.user_id);
      const musa_audio = await getMusaAudio(row.musa_id, row.user_id);
      const musa_image = await getMusaMedia(row.musa_id, row.user_id);
      const img_video = await getMusavideoImage(row.musa_id, row.user_id);
      const img_video_new = await getMusavideoNew(row.musa_id, row.user_id);
      const musa_like_status = await getlikeStatus1(row.musa_id, user_id);
      const followstatus = await getFollowStatus1(user_id, row.user_id);
      // const follow_staus = await getFollowStatus(user_id, row.user_id);
      const categoryData = await getSingleAllCategoryData(row.category);
      const subCategory = await getSingleSubCategoryData(
        row.sub_category,
        row.category
      );
      const comment = await getComment1(row.musa_id);
      // const commentData = await getCommentData(row.musa_id)
      const time = await timeAgo(row.createtime);
      const like_count = await getlikecount(row.musa_id);
      const savemusa_status = await getsaveStatus(row.musa_id, user_id);
      const display_request_status = await displayStatus(row.musa_id, user_id);
      const share_count = await getShareCount(row.musa_id, response);
      musa_data_arr = {
        name: userData ? userData.name : "NA",
        f_name: userData ? userData.f_name : "NA",
        l_name: userData ? userData.l_name : "NA",
        image: userData ? userData.image : "NA",
        musa_id: row.musa_id,
        user_id: row.user_id,
        musa_title: row.title,
        category_data: categoryData ? categoryData[0].title : "NA",
        sub_category: subCategory ? subCategory[0].title : "NA",
        description: row.description,
        createtime: time,
        type: row.type,
        type_label: "0 for public, 1 for contributor, 3 for only me",
        follow_staus: followstatus ? followstatus : 0,
        follow_staus_label: "0 - no , 1- yes ",
        comment: comment,
        // commentData  : (commentData) ? commentData : "NA",
        musa_like_status: musa_like_status ? musa_like_status : "NA",
        musa_like_status_label: "0 - no , 1- yes ",
        like_count: like_count,
        total_share: share_count,
        savemusa_status: savemusa_status,
        display_request_status_text: "0=not sended,1=sended,2=accepted",
        display_request_status: display_request_status,
        savemusaStatus_lable: "0 for no , 1 for yes",
        musa_image: musa_image ? musa_image : "NA",
        image_arr: image_arr ? image_arr : "NA",
        musa_video: musa_video ? musa_video : "NA",
        musa_audio: musa_audio ? musa_audio : "NA",
        img_video: img_video.length > 0 ? img_video : "NA",
        img_video_new: img_video_new.length > 0 ? img_video_new : "NA",
        status: false,
      };

      // get next id start

      // let nextVal = [];
      // let nextSql = `SELECT musa_id FROM musa_master WHERE delete_flag = 0`;

      // if (musa_status == 1) {
      //     nextSql += ` AND user_id = ?`;
      //     nextVal.push(user_id);
      // }

      // if (move == 1) {
      //     nextSql += ` AND musa_id > ?`;
      //     nextVal.push(musa_id);
      // }
      // if (move == 2) {
      //     nextSql += ` AND musa_id < ? order by musa_id desc`;
      //     nextVal.push(musa_id);
      // }

      // nextSql += ` LIMIT 1`;

      // await connection.query(nextSql, nextVal, async (nextErr, nextResult) => {
      //     if (nextErr) {
      //         return response.status(200).json({ success: false, msg: languageMessages.internalServerError, nextErr: nextErr });
      //     }
      //     let next_id = 0;
      //     if (nextResult.length > 0) {
      //         next_id = nextResult[0].musa_id;
      //     }
      //     return response.status(200).json({ success: true, msg: languageMessages.msgDataFound, musa_data_arr: musa_data_arr, next_id: next_id });
      // });

      return response.status(200).json({
        success: true,
        msg: languageMessages.msgDataFound,
        musa_data_arr: musa_data_arr,
      });
      // get next id end
    } else {
      return response.status(200).json({
        success: true,
        msg: languageMessages.msgDataFound,
        musa_data_arr: "NA",
      });
    }
  } catch (error) {
    return response.status(200).json({
      success: false,
      msg: languageMessages.internalServerError,
      error: error.message,
    });
  }
};

// get single musa details end



function fetchUserData(userId, callback) {
  const query = "SELECT * FROM user_master WHERE user_id = ? ";

  connection.query(query, [userId], async (error, results) => {
    if (error) {
      callback(error, null);
      return;
    }
    if (results.length > 0) {
      const userData = results[0];
      let dob_new = formatDate(userData.dob);
      const formatted_dob = moment(dob_new).format("MM/DD/YYYY");
      const age = moment().diff(dob_new, "years");
      const gb = await getGB(userData.user_id);
      const used_gb = await getUsedGb(userData.user_id);
      const total_gb = await totalGb(userData.user_id);

      let oneGb = 1073741824; // one gb is in byte
      let total_kb = total_gb * oneGb;
      let total_remaining_gb = total_kb - used_gb;

      const userDataArray = {
        total_kb: total_kb,
        total_gb: total_gb,
        total_used_gb: used_gb,
        total_remaining_gb: total_remaining_gb > 0 ? total_remaining_gb : 0,
        user_id: userData.user_id,
        user_type: userData.user_type,
        user_type_lable: "0=admin,1= user",
        login_type: userData.login_type,
        login_type_lable: "0=app, 1=google, 2=apple",
        email: userData.email,
        f_name: userData.f_name,
        l_name: userData.l_name,
        name: userData.name,
        dob: dob_new,
        formatted_dob: formatted_dob,
        age: age,
        phone_code: userData.phone_code,
        mobile: userData.mobile,
        otp: userData.otp,
        otp_verify: userData.otp_verify,
        image: userData.image,
        bio: userData.bio,
        gb: gb,
        remaining_gb: gb,
        used_gb: used_gb,
        bio_type: userData.bio_type,
        zipcode: userData.zipcode,
        bio_type_lable: "0 for text, 1 for audio	",
        gender_lable: "1=men,2=women,3=other",
        gender: userData.gender,
        address: userData.address,
        latitude: userData.latitude,
        longitude: userData.longitude,
        active_flag: userData.active_flag,
        approve_flag: userData.approve_flag,
        profile_complete: userData.profile_complete,
        language_id: userData.language_id,
        facebook_id: userData.facebook_id,
        google_id: userData.google_id,
        apple_id: userData.apple_id,
        notification_status: userData.notification_status,
        delete_reason: userData.delete_reason,
        createtime: userData.createtime,
        updatetime: userData.updatetime,
      };
      callback(null, userDataArray);
    } else {
      callback(null, null);
    }
  });
}



async function getGB(user_id) {
  var sqlstatus =
    "SELECT user_id,plan_id from user_plan_master where user_id = ? AND delete_flag = 0 AND status = 1";
  return new Promise((resolve, reject) => {
    connection.query(sqlstatus, [user_id], (err, result) => {
      if (err) {
        reject(err);
      } else {
        if (result.length > 0) {
          var sqlstatus =
            "SELECT gb from plan_master where plan_id = ? AND delete_flag = 0 ";
          // return new Promise((resolve, reject) => {
          connection.query(sqlstatus, [result[0].plan_id], (err, data) => {
            if (err) {
              reject(err);
            }
            if (data.length > 0) {
              resolve(data[0].gb);
            } else {
              resolve("NA");
            }
          });
        } else {
          resolve("NA");
        }
      }
    });
  });
}

async function getUsedGb(user_id) {
  return new Promise((resolve, reject) => {
    let sql =
      "SELECT SUM(mim.size) as size FROM musa_master as mm,musa_image_master as mim where mm.delete_flag=0 and mm.user_id=? and mm.musa_id=mim.musa_id and mim.delete_flag=0 and mm.delete_flag=0";
    connection.query(sql, [user_id], async (err, result) => {
      if (err) {
        reject(err);
      } else {
        if (result.length > 0) {
          resolve(result[0].size);
        } else {
          resolve(0);
        }
      }
    });
  });
}

// get used gb size in musa end

async function totalGb(user_id) {
  let sql =
    "SELECT SUM(gb) as total_gb FROM plan_master as pm , user_plan_master as upm WHERE upm.plan_id=pm.plan_id and upm.delete_flag=0 and upm.user_id=?";

  return new Promise(async (resolve, reject) => {
    await connection.query(sql, [user_id], async (err, result) => {
      if (err) {
        resolve(0);
      } else {
        if (result.length > 0) {
          resolve(result[0].total_gb);
        } else {
          resolve(0);
        }
      }
    });
  });
}

module.exports = {
  editMusa,
  deleteSubCategory,
  deleteCategory,
  editMySubCategory,
  getMyAllSubCategory,
  editMyCategory,
  getMyAllCategory,
  deleteMusaController,
  timeAgo,
  createMussa,
  userFollow,
  getAllSubCategory,
  getSocialMusaDetails,
  getSingleMusaDetails,
  musaComment,
  musaLikeUnlike,
  Contributor,
  myContributor,
  postReport,
  saveMusaa,
  myAlbumSubalbum,
  getMyAlbum,
  homeAlbum,
  getAllCategoryData,
  addNewCategory,
  addCategorySubcategory,
  removeContributor,
  imageUpload,
  audioUpload,
  videoUploadFolder,
  getSingleAllCategoryData,
  getSingleSubCategoryData,
  getFollowStatus1,
  displayRequest,
  acceptRejectRequest,
  getFollowStatus1,
  getAllDisplayRequest,
  getMyMusaCategoryWise,
  getAllCommentsOnMusa,
  displayStatus,
  getAllCommentsOnMusaThroughImage,
  increaseShareCount,
  getSingleMusaDetailsByMusaId,
};
